#include "Camera.h"

// evalue la qualite d'une image ou on doit voir le bras entier :
double Camera::getArmIn() const {
	// on verifie ensuite si le bras est bien inclus dans le cone de vision de la camera:

	// i) P <-- P + vect(CO) ; C == camera, O == origine et P == point sur le bras.
	double* coordTrans3D=(double*)malloc(9*sizeof(double));
	coordTrans3D[0]=coord3D[0]-posX;
	coordTrans3D[1]=coord3D[1]-posY;
	coordTrans3D[2]=coord3D[2]-posZ;
	coordTrans3D[3]=coord3D[3]-posX;
	coordTrans3D[4]=coord3D[4]-posY;
	coordTrans3D[5]=coord3D[5]-posZ;
	coordTrans3D[6]=coord3D[6]-posX;
	coordTrans3D[7]=coord3D[7]-posY;
	coordTrans3D[8]=coord3D[8]-posZ;

	// ii) calcul de la matrice de changement de base "reperes camera / origine"
	double *coeffsX=(double*)malloc(3*sizeof(double)),
			*coeffsY=(double*)malloc(3*sizeof(double)),
			*coeffsZ=(double*)malloc(3*sizeof(double));
	getPassMatrix(coeffsX,coeffsY,coeffsZ,anglePlan,angleVert);

	// on inverse la matrice M obtenue facilement en prenant la transposee : tM;
	// il ne reste plus qu'a calculer tM*coordonnees de chaque point a tester.

	// i) calcul tM*pointEpaule :
	double newX,newY,newZ;
	newX=coeffsX[0]*coordTrans3D[0]+coeffsX[1]*coordTrans3D[1]+coeffsX[2]*coordTrans3D[2];
	newY=coeffsY[0]*coordTrans3D[0]+coeffsY[1]*coordTrans3D[1]+coeffsY[2]*coordTrans3D[2];
	newZ=coeffsZ[0]*coordTrans3D[0]+coeffsZ[1]*coordTrans3D[1]+coeffsZ[2]*coordTrans3D[2];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		// on veut voir (au moins en thorie et dans un premier temps sans optimisations) tout le bras:
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}
	// ii) calculs pour les autres points :
	newX=coeffsX[0]*coordTrans3D[3]+coeffsX[1]*coordTrans3D[4]+coeffsX[2]*coordTrans3D[5];
	newY=coeffsY[0]*coordTrans3D[3]+coeffsY[1]*coordTrans3D[4]+coeffsY[2]*coordTrans3D[5];
	newZ=coeffsZ[0]*coordTrans3D[3]+coeffsZ[1]*coordTrans3D[4]+coeffsZ[2]*coordTrans3D[5];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}
	newX=coeffsX[0]*coordTrans3D[6]+coeffsX[1]*coordTrans3D[7]+coeffsX[2]*coordTrans3D[8];
	newY=coeffsY[0]*coordTrans3D[6]+coeffsY[1]*coordTrans3D[7]+coeffsY[2]*coordTrans3D[8];
	newZ=coeffsZ[0]*coordTrans3D[6]+coeffsZ[1]*coordTrans3D[7]+coeffsZ[2]*coordTrans3D[8];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}

	free(coeffsX);
	free(coeffsY);
	free(coeffsZ);
	free(coordTrans3D);

	// enfin, derniere verification avant algorithme principal : on regarde si le bras
	// n'est pas obstrue par lui-meme, en verifiant que les triplets (joint 1, joint 2, camera)
	// ne soient pas colineaires, quels que soient les joints selectionnes
	double* paramCam=(double*)malloc(3*sizeof(double));
	paramCam[0]=posX;
	paramCam[1]=posY;
	paramCam[2]=posZ;
	if (colineaires(coord3D,coord3D+3,paramCam)
			|| colineaires(coord3D,coord3D+6,paramCam)
			|| colineaires(coord3D+3,coord3D+6,paramCam)) {
		free(paramCam);
		return 0.0; // on ne veut pas de telle vue !
	}
	free(paramCam);
	
	// ensuite on echantillonne point par point sur le bras, enregistrant le pourcentage de collisions :
	// E_PTS points entre epaule et coude,
	// C_PTS points entre coude et poignet
	// P_PTS points sur la sphere du poignet
	double* pointEp=(double*)malloc(3*E_PTS*sizeof(double));
	double* pointCo=(double*)malloc(3*C_PTS*sizeof(double));
	double* pointPo=(double*)malloc(3*P_PTS*sizeof(double));

	/*************************************
	*** ajout d'une perturbation radiale :
	*************************************/

	// on calcule deux vecteurs orthogonaux dans le plan de coupe du cylindre epaule-coude,

	double* vect1=(double*)malloc(3*sizeof(double));
	double* vect2=(double*)malloc(3*sizeof(double));
	double* diffVect=(double*)malloc(3*sizeof(double));
	diffVect[0]=coord3D[3]-coord3D[0];
	diffVect[1]=coord3D[4]-coord3D[1];
	diffVect[2]=coord3D[5]-coord3D[2];

	double phi,theta;
	// transformation en coordonnees spheriques pour obtenir phi et theta :
	getSphericAngles(diffVect,phi,theta);

	double tmpSinP=sin(phi),tmpCosP=cos(phi),tmpCosV=cos(theta);
	vect1[0]=tmpSinP;
	vect1[1]=-tmpCosP;
	vect1[2]=0.0;
	vect2[0]=-tmpCosP*tmpCosV;
	vect2[1]=-tmpSinP*tmpCosV;
	vect2[2]=sin(theta);
	// on normalise ensuite les vecteurs se basant sur RAY_C (rayon d'un cylindre de bras)
	for (int i=2; i>=0; i--) {
		vect1[i]*=RAY_C;
		vect2[i]*=RAY_C;
	}
	
	// puis on echantillonne :
	double alpha;
	int rdB1,rdB2;
	for (int i=E_PTS-1; i>=0; i--) {
		alpha=genAlea();
		pointEp[3*i]=alpha*coord3D[0]+(1-alpha)*coord3D[3];
		pointEp[3*i+1]=alpha*coord3D[1]+(1-alpha)*coord3D[4];
		pointEp[3*i+2]=alpha*coord3D[2]+(1-alpha)*coord3D[5];
		alpha=genAlea();
		rdB1=randBit();
		rdB2=randBit();
		pointEp[3*i]+=rdB1*alpha*vect1[0]+rdB2*(1-alpha)*vect2[0];
		pointEp[3*i+1]+=rdB1*alpha*vect1[1]+rdB2*(1-alpha)*vect2[1];
		pointEp[3*i+2]+=rdB1*alpha*vect1[2]+rdB2*(1-alpha)*vect2[2];
	}

	/***************************************************
	***	meme travail pour la partie du coude au poignet :
	***************************************************/

	diffVect[0]=coord3D[6]-coord3D[3];
	diffVect[1]=coord3D[7]-coord3D[4];
	diffVect[2]=coord3D[8]-coord3D[5];

	// transformation en coordonnees spheriques pour obtenir phi et theta :
	getSphericAngles(diffVect,phi,theta);
	free(diffVect);

	tmpSinP=sin(phi);
	tmpCosP=cos(phi);
	tmpCosV=cos(theta);
	vect1[0]=tmpSinP;
	vect1[1]=-tmpCosP;
	vect1[2]=0.0;
	vect2[0]=-tmpCosP*tmpCosV;
	vect2[1]=-tmpSinP*tmpCosV;
	vect2[2]=sin(theta);
	// on normalise ensuite les vecteurs se basant sur RAY_C (rayon d'un cylindre de bras)
	for (int i=2; i>=0; i--) {
		vect1[i]*=RAY_C;
		vect2[i]*=RAY_C;
	}
	
	// puis on echantillonne :
	for (int i=C_PTS-1; i>=0; i--) {
		alpha=genAlea();
		pointCo[3*i]=alpha*coord3D[3]+(1-alpha)*coord3D[6];
		pointCo[3*i+1]=alpha*coord3D[4]+(1-alpha)*coord3D[7];
		pointCo[3*i+2]=alpha*coord3D[5]+(1-alpha)*coord3D[8];
		alpha=genAlea();
		rdB1=randBit();
		rdB2=randBit();
		pointCo[3*i]+=rdB1*alpha*vect1[0]+rdB2*(1-alpha)*vect2[0];
		pointCo[3*i+1]+=rdB1*alpha*vect1[1]+rdB2*(1-alpha)*vect2[1];
		pointCo[3*i+2]+=rdB1*alpha*vect1[2]+rdB2*(1-alpha)*vect2[2];
	}
	free(vect1);
	free(vect2);

	/*********************************
	**** puis enfin pour le poignet :
	*********************************/

	// on connait le centre ainsi que le rayon :
	// on utilise donc une loi uniforme dans la sphere.
	double tmpSinT;
	for (int i=P_PTS-1; i>=0; i--) {
		// tirage aleatoire de l'angle phi puis theta :
		phi=2*M_PI*genAlea();
		theta=2*M_PI*genAlea();
		
		// calcul des coordonnees x,y,z en fonction des coordonnees spheriques r,phi,theta:
		tmpSinT=sin(theta);
		pointPo[i*3]=coord3D[6]+RAY_P*cos(phi)*tmpSinT;
		pointPo[i*3+1]=coord3D[7]+RAY_P*sin(phi)*tmpSinT;
		pointPo[i*3+2]=coord3D[8]+RAY_P*cos(theta);
	}

	// on arrive au moment ou l'on doit tester les obstructions le long de chaque segment :
	int countObstE=0,countObstC=0,countObstP=0; // compteurs d'obstruction

	int nbPas;
	double pasX,pasY,pasZ;
	double* coordXYZ=(double*)malloc(3*sizeof(double));
	// points epaule-coude :
	for (int i=E_PTS-1; i>=0; i--) {
		if (fabs(posX-pointEp[i*3])>=fabs(posY-pointEp[i*3+1]) && fabs(posX-pointEp[i*3])>=fabs(posZ-pointEp[i*3+2]))
			nbPas=(int)floor((fabs(posX-pointEp[i*3]))/PAS_ELEM);
		else if (fabs(posY-pointEp[i*3+1])>=fabs(posX-pointEp[i*3]) && fabs(posY-pointEp[i*3+1])>=fabs(posZ-pointEp[i*3+2]))
			nbPas=(int)floor((fabs(posY-pointEp[i*3+1]))/PAS_ELEM);
		else nbPas=(int)floor((fabs(posZ-pointEp[i*3+2]))/PAS_ELEM);

		pasX=(posX-pointEp[i*3])/(nbPas+1);
		pasY=(posY-pointEp[i*3+1])/(nbPas+1);
		pasZ=(posZ-pointEp[i*3+2])/(nbPas+1);
		coordXYZ[0]=pointEp[i*3];
		coordXYZ[1]=pointEp[i*3+1];
		coordXYZ[2]=pointEp[i*3+2];
		for (int j=nbPas-1; j>=0; j--) {
			coordXYZ[0]+=pasX;
			coordXYZ[1]+=pasY;
			coordXYZ[2]+=pasZ;
			if (collision(coordXYZ)!=0) {
				countObstE++;
				break;
			}
		}
	}
	// points coude-poignet :
	for (int i=C_PTS-1; i>=0; i--) {
		if (fabs(posX-pointCo[i*3])>=fabs(posY-pointCo[i*3+1]) && fabs(posX-pointCo[i*3])>=fabs(posZ-pointCo[i*3+2]))
			nbPas=(int)floor((fabs(posX-pointCo[i*3]))/PAS_ELEM);
		else if (fabs(posY-pointCo[i*3+1])>=fabs(posX-pointCo[i*3]) && fabs(posY-pointCo[i*3+1])>=fabs(posZ-pointCo[i*3+2]))
			nbPas=(int)floor((fabs(posY-pointCo[i*3+1]))/PAS_ELEM);
		else nbPas=(int)floor((fabs(posZ-pointCo[i*3+2]))/PAS_ELEM);

		pasX=(posX-pointCo[i*3])/(nbPas+1);
		pasY=(posY-pointCo[i*3+1])/(nbPas+1);
		pasZ=(posZ-pointCo[i*3+2])/(nbPas+1);
		coordXYZ[0]=pointCo[i*3];
		coordXYZ[1]=pointCo[i*3+1];
		coordXYZ[2]=pointCo[i*3+2];
		for (int j=nbPas-1; j>=0; j--) {
			coordXYZ[0]+=pasX;
			coordXYZ[1]+=pasY;
			coordXYZ[2]+=pasZ;
			if (collision(coordXYZ)!=0) {
				countObstC++;
				break;
			}
		}
	}
	// points main (poignet) :
	for (int i=P_PTS-1; i>=0; i--) {
		if (fabs(posX-pointPo[i*3])>=fabs(posY-pointPo[i*3+1]) && fabs(posX-pointPo[i*3])>=fabs(posZ-pointPo[i*3+2]))
			nbPas=(int)floor((fabs(posX-pointEp[i*3]))/PAS_ELEM);
		else if (fabs(posY-pointPo[i*3+1])>=fabs(posX-pointPo[i*3]) && fabs(posY-pointPo[i*3+1])>=fabs(posZ-pointPo[i*3+2]))
			nbPas=(int)floor((fabs(posY-pointPo[i*3+1]))/PAS_ELEM);
		else nbPas=(int)floor((fabs(posZ-pointPo[i*3+2]))/PAS_ELEM);

		pasX=(posX-pointPo[i*3])/(nbPas+1);
		pasY=(posY-pointPo[i*3+1])/(nbPas+1);
		pasZ=(posZ-pointPo[i*3+2])/(nbPas+1);
		coordXYZ[0]=pointPo[i*3];
		coordXYZ[1]=pointPo[i*3+1];
		coordXYZ[2]=pointPo[i*3+2];
		for (int j=nbPas-1; j>=0; j--) {
			coordXYZ[0]+=pasX;
			coordXYZ[1]+=pasY;
			coordXYZ[2]+=pasZ;
			if (collision(coordXYZ)!=0) {
				countObstP++;
				break;
			}
		}
	}

	// liberations memoire
	free(pointEp);
	free(pointCo);
	free(pointPo);
	free(coordXYZ);

	// resultat : ratio non_obstrue / total
	return 1.0-(double)(countObstE+countObstC+countObstP)/(E_PTS+C_PTS+P_PTS);
}

// meme boulot pour un unique point (bcp plus simple,
// cas de la main ou du filmage avec zoom si obstacle proche) :
double Camera::getPointIn() const {
	// echantillonnage au voisinage du point :
	// on utilise une loi uniforme dans la sphere, avec P_PTS points et de rayon RAY_P si main,
	// avec floor(20*distObstacles[indiceG]) points sinon, pour un rayon de distObstacles[indiceG]
	double tmpSinT,rayon=0.0;
	int nbPoints=0;
	if (handPoints[indiceG]) {
		nbPoints=P_PTS;
		rayon=RAY_P;
	}
	else {
		nbPoints=(int)floor(20*distObstacles[indiceG]);
		rayon=distObstacles[indiceG];
	}
	double* pointsEch=(double*)malloc(nbPoints*3*sizeof(double));

	double phi,theta;
	for (int i=nbPoints-1; i>=0; i--) {
		// tirage aleatoire de l'angle phi puis theta :
		phi=2*M_PI*genAlea();
		theta=2*M_PI*genAlea();
		
		// calcul des coordonnees x,y,z en fonction des coordonnees spheriques r,phi,theta:
		tmpSinT=sin(theta);
		pointsEch[i*3]=currPR[0]+rayon*cos(phi)*tmpSinT;
		pointsEch[i*3+1]=currPR[1]+rayon*sin(phi)*tmpSinT;
		pointsEch[i*3+2]=currPR[2]+rayon*cos(theta);
	}
	
	// recherche de collisions :
	int nbPas,countObst=0;
	double pasX,pasY,pasZ;
	double* coordXYZ=(double*)malloc(3*sizeof(double));
	for (int i=nbPoints-1; i>=0; i--) {
		if (fabs(posX-pointsEch[i*3])>=fabs(posY-pointsEch[i*3+1]) && fabs(posX-pointsEch[i*3])>=fabs(posZ-pointsEch[i*3+2]))
			nbPas=(int)floor((fabs(posX-pointsEch[i*3]))/PAS_ELEM);
		else if (fabs(posY-pointsEch[i*3+1])>=fabs(posX-pointsEch[i*3]) && fabs(posY-pointsEch[i*3+1])>=fabs(posZ-pointsEch[i*3+2]))
			nbPas=(int)floor((fabs(posY-pointsEch[i*3+1]))/PAS_ELEM);
		else nbPas=(int)floor((fabs(posZ-pointsEch[i*3+2]))/PAS_ELEM);

		pasX=(posX-pointsEch[i*3])/(nbPas+1);
		pasY=(posY-pointsEch[i*3+1])/(nbPas+1);
		pasZ=(posZ-pointsEch[i*3+2])/(nbPas+1);
		coordXYZ[0]=pointsEch[i*3];
		coordXYZ[1]=pointsEch[i*3+1];
		coordXYZ[2]=pointsEch[i*3+2];
		for (int j=nbPas-1; j>=0; j--) {
			coordXYZ[0]+=pasX;
			coordXYZ[1]+=pasY;
			coordXYZ[2]+=pasZ;
			if (collision(coordXYZ)!=0) {
				countObst++;
				break;
			}
		}
	}
	free(pointsEch);
	free(coordXYZ);
	
	return 1-(double)countObst/nbPoints;
}

// fonction un niveau au dessus, qui met a jour les angles (sauf pr la main) 
// avant d'appeler une des fonctions precedente :
double Camera::getVisibilite() {
	// on regarde simplement si la camera est en collision :
	double params[]={posX,posY,posZ};
	if (collision(params)) return 0.0;

	// mise a jour des angles :
	double vecteur[]={currPR[0]-posX,currPR[1]-posY,currPR[2]-posZ};
	getSphericAngles(vecteur,anglePlan,angleVert);
	
	if (handPoints[indiceG] || zoomObstPoints[indiceG]) return getPointIn();

	return getArmIn();
}
