// classe decrivant une camera: 7 degres de liberte (roll ?!)

#ifndef _Camera
#define _Camera

#include <cmath>
#include <cstdlib>
#include "Constantes.h"
#include "Global.h"
#include "Functions.h"
#include "ConfigCamera.h"

class Camera {
	
public:
	
	Camera (double posX=0.0,double posY=0.0,double posZ=0.0,double anglePlan=0.0,double angleVert=0.0,double roll=0.0,double zoom=1.0) {
		this->posX=posX;
		this->posY=posY;
		this->posZ=posZ;
		this->anglePlan=anglePlan; // par rapport  l'axe Oz (comme en physique)
		this->angleVert=angleVert; // par rapport au plan xOy
		this->roll=roll;
		this->zoom=zoom;
	}
	
	Camera (const Camera& cam) {
		posX=cam.posX;
		posY=cam.posY;
		posZ=cam.posZ;
		anglePlan=cam.anglePlan;
		angleVert=cam.angleVert;
		roll=cam.roll;
		zoom=cam.zoom;
	}
	
	~Camera() {}
	
	// fonction renvoyant un rel entre 0 et 1,
	// donnant le degr de visibilit du bras-robot
	// [optionnel ensuite : ajouter autres points de repere visuels..]
	double getArmIn() const;
	
	// meme boulot pour la main (bcp plus simple) :
	double getPointIn() const;
	
	// fonction un niveau au dessus, qui met a jour les angles (sauf pr la main ou milieu segment) 
	// avant d'appeler une des fonctions precedente :
	double getVisibilite();
	
	// fonction retournant la configuration de la camera :
	ConfigCamera getConfig() {
		ConfigCamera res=ConfigCamera(posX,posY,posZ,anglePlan,angleVert,roll,zoom);
		return res;
	}
	
	double posX,posY,posZ; // position spatiale du centre de gravite
	double anglePlan,angleVert; // orientation dans l'espace
	double roll; // rotation de la camera sur elle-meme
	double zoom; // force du zoom (1.0 pour normal)

};

#endif
