#include "CompilIdioms.h"

#include <iostream>

using namespace std;

// compile une suite d'idiomes en placements de cameras :
vector<ConfigCamera>* compilation(vector<Idiom>& idiomes,const vector<Configuration>& configurations) {

	// deux cas : compilation d'un idiome simple (un seul shot),
	// et compilation d'un idiome compose.
	
	// second cas nettement plus delicat car on doit regler les instants de changement de shots.
	// on n'implemente que le premier. Notons tout de meme que le reglage des timeStamps est la seule difficulte,
	// la compilation s'effectuant sur la base des shots seuls.
	
	// IMPORTANT :
	// on initialise zoomObstPoints a false (on ne s'en sert pas dans le cas des idiomes,
	// mais cela pourrait interferer) :
	for (int i=configurations.size()-1; i>=0; i--) zoomObstPoints.push_back(false);
	
	// recuperation des angles de vues pr le film :
	for (int i=0; i<(int)idiomes.size(); i++)
		evalViews(idiomes[i],configurations);
	
	PlaceCamera placeCam=PlaceCamera(); // objet utile pr creer et placer les 3 cameras
	ConfigCamera configCam1=ConfigCamera(),configCam0=ConfigCamera(),configCam2=ConfigCamera();
	vector<ConfigCamera>* resConfigs=new vector<ConfigCamera>[3]; // vecteur des configurations resultat
	double* vecteur=(double*)malloc(3*sizeof(double));
	double* initPR=(double*)malloc(3*sizeof(double));

	// placement des cameras selon ces vues :
	for (int i=0; i<(int)idiomes.size(); i++) { //cout << idiomes[i].name <<"  " << idiomes[i].shots[0].firstInd << " " << idiomes[i].shots.back().lastInd << endl;
	
		// premier cas simpliste : on n'a pas su dire quel type d'idiome c'est,
		// donc on filme comme en resolution de contraintes, image par image,
		// sans oublier de lisser le resultat obtenu :
		if (strcmp(idiomes[i].name,"unknown")==0) { // on espere que ce cas n'arrive pas..

			vector<ConfigCamera> configsCam1,configsCam0,configsCam2;
			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots.back().lastInd; j++) {
				indiceG=j;
				coord3D=configurations[j].conf2coord3D();
	
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
	
				// calcul du nouveau point de repere :
				getRepPoint(j);

				// calcul de la distance de vue :
				getViewDistance(j);
		
				for (int k=2; k>=0; k--) depctRepPoint[k]=currPR[k]-lastRepPoint[k];

				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam1.push_back(placeCam.cameras[1].getConfig());
			
				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam0.push_back(placeCam.cameras[0].getConfig());
	
				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam2.push_back(placeCam.cameras[2].getConfig());
			}

			// lissage :
			vector<Configuration> configs;
			for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots[0].lastInd; j++)
				configs.push_back(configurations[j]);
			configsCam1=lisse(configsCam1,configs);
			configsCam0=lisse(configsCam0,configs);
			configsCam2=lisse(configsCam2,configs);
			
			// ajout au resultat :
			for (int j=0; j<=idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd; j++) {
				resConfigs[1].push_back(configsCam1[j]);
				resConfigs[0].push_back(configsCam0[j]);
				resConfigs[2].push_back(configsCam2[j]);
			}
			
			free(lastRepPoint);
			free(depctRepPoint);
			continue;
		}

		/********************
		* Initialisations :
		********************/

		// determination du point de repere initial :
		coord3D=configurations[idiomes[i].shots[0].firstInd].conf2coord3D();
		getRepPoint(idiomes[i].shots[0].firstInd);
		
		// calcul de la distance de vue :
		getViewDistance(idiomes[i].shots[0].firstInd);

		// ********** CAMERA 1 (centrale) : ****************
		if (idiomes[i].views[0]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(1);
		}
		else {
			putCamInView(placeCam.cameras[1],idiomes[i].views[0]);
		}

		// ********** CAMERA 0 (gauche) : *****************
		if (idiomes[i].views[1]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(0);
		}
		else {
			putCamInView(placeCam.cameras[0],idiomes[i].views[1]);
		}
		
		// *********** CAMERA 2 (droite) : *****************
		if (idiomes[i].views[2]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(2);
		}
		else {
			putCamInView(placeCam.cameras[2],idiomes[i].views[2]);
		}

		// calcul des angles initiaux de chaque camera,
		// et initialisations des configurations :
		
		// ********** CAMERA 1 (centrale) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[1].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[1].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[1].posZ;
		double phi,theta;
		getSphericAngles(vecteur,phi,theta);
		
		configCam1.posX=placeCam.cameras[1].posX;
		configCam1.posY=placeCam.cameras[1].posY;
		configCam1.posZ=placeCam.cameras[1].posZ;
		configCam1.anglePlan=phi;
		configCam1.angleVert=theta;
		configCam1.roll=0.0; // le mysterieux roll..
		configCam1.zoom=1.0; // (toujours 1)
		
		// ********** CAMERA 0 (gauche) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[0].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[0].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[0].posZ;
		getSphericAngles(vecteur,phi,theta);
		
		configCam0.posX=placeCam.cameras[0].posX;
		configCam0.posY=placeCam.cameras[0].posY;
		configCam0.posZ=placeCam.cameras[0].posZ;
		configCam0.anglePlan=phi;
		configCam0.angleVert=theta;
		configCam0.roll=0.0; // le mysterieux roll..
		configCam0.zoom=1.0; // (toujours 1)
		
		// ********** CAMERA 2 (droite) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[2].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[2].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[2].posZ;
		getSphericAngles(vecteur,phi,theta);
		
		configCam2.posX=placeCam.cameras[2].posX;
		configCam2.posY=placeCam.cameras[2].posY;
		configCam2.posZ=placeCam.cameras[2].posZ;
		configCam2.anglePlan=phi;
		configCam2.angleVert=theta;
		configCam2.roll=0.0; // le mysterieux roll..
		configCam2.zoom=1.0; // (toujours 1)

		// sauvegarde de la valeur du point de repere courant (==initial),
		// ainsi que la distance courante au point de repere :
		for (int j=2; j>=0; j--) initPR[j]=currPR[j];
		double distCamIPR=distCamPR;
		// calcul du point de repere dans la configuration finale sur cet idiome :
		coord3D=configurations[idiomes[i].shots.back().lastInd].conf2coord3D();
		getRepPoint(idiomes[i].shots.back().lastInd);
		
		// preliminaire : calcul des pas angulaires (ou de translation) elementaires :
		// tout depend de la position relative de la camera par rapport au bras;
		// et aussi des types d'idiomes.
		double pasY1=0.0,pasPhi1=0.0,pasTheta1=0.0,
				pasY0=0.0,pasPhi0=0.0,pasTheta0=0.0,
				pasY2=0.0,pasPhi2=0.0,pasTheta2=0.0;
		// si static : pas de pas == rien a faire.

		if (strcmp(idiomes[i].name,"panHori")==0) {
			// seul pasPhi va etre affecte :

			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine grossierement la position de la camera pour connaitre le signe :
			int sgn;
			if (configCam1.posX>(initPR[0]+currPR[0])/2.0) {
				if (currPR[1]-initPR[1]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[1]-initPR[1]<0) sgn=-1;
				else sgn=1;
			}
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam1.posX-currPR[0])*(configCam1.posX-currPR[0])
							+ (configCam1.posY-currPR[1])*(configCam1.posY-currPR[1])
							+ (configCam1.posZ-currPR[2])*(configCam1.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasPhi1=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[1]-initPR[1])*fabs(currPR[1]-initPR[1]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/

			// on determine grossierement la position de la camera pour connaitre le signe :
			if (configCam0.posX>(initPR[0]+currPR[0])/2.0) {
				if (currPR[1]-initPR[1]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[1]-initPR[1]<0) sgn=-1;
				else sgn=1;
			}
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam0.posX-currPR[0])*(configCam0.posX-currPR[0])
							+ (configCam0.posY-currPR[1])*(configCam0.posY-currPR[1])
							+ (configCam0.posZ-currPR[2])*(configCam0.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasPhi0=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[1]-initPR[1])*fabs(currPR[1]-initPR[1]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi0/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 2 : droite ************
			******************************************/
			
			// on determine grossierement la position de la camera pour connaitre le signe :
			if (configCam2.posX>(initPR[0]+currPR[0])/2.0) {
				if (currPR[1]-initPR[1]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[1]-initPR[1]<0) sgn=-1;
				else sgn=1;
			}
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam2.posX-currPR[0])*(configCam2.posX-currPR[0])
							+ (configCam2.posY-currPR[1])*(configCam2.posY-currPR[1])
							+ (configCam2.posZ-currPR[2])*(configCam2.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasPhi2=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[1]-initPR[1])*fabs(currPR[1]-initPR[1]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi2/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
		}
		
		else if (strcmp(idiomes[i].name,"panVert")==0) {
			// seul pasTheta va etre affecte : meme boulot

			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine le signe (le meme pr les 3 cameras !) :
			int sgn;
			if (currPR[2]-initPR[2]<0)
				// deplacement "du haut vers le bas", donc signe positif.
				sgn=1;
			else sgn=-1;
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam1.posX-currPR[0])*(configCam1.posX-currPR[0])
							+ (configCam1.posY-currPR[1])*(configCam1.posY-currPR[1])
							+ (configCam1.posZ-currPR[2])*(configCam1.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[2]-initPR[2]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasTheta1=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[2]-initPR[2])*fabs(currPR[2]-initPR[2]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam0.posX-currPR[0])*(configCam0.posX-currPR[0])
							+ (configCam0.posY-currPR[1])*(configCam0.posY-currPR[1])
							+ (configCam0.posZ-currPR[2])*(configCam0.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[2]-initPR[2]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasTheta0=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[2]-initPR[2])*fabs(currPR[2]-initPR[2]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta0/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 2 : droite ************
			******************************************/
			
			// calcul de la distance au point de repere final (EN SUPPOSANT QUE LA CAMERA NE BOUGE PAS !)
			distCamPR=sqrt((configCam2.posX-currPR[0])*(configCam2.posX-currPR[0])
							+ (configCam2.posY-currPR[1])*(configCam2.posY-currPR[1])
							+ (configCam2.posZ-currPR[2])*(configCam2.posZ-currPR[2]));
			// calcul de l'angle global en fonction des 3 cotes |currPR[2]-initPR[2]|, distCamPR, distCamIPR :
			// (on utilise la formule d'Al-Kashi)
			pasTheta2=sgn*acos((distCamIPR*distCamIPR+distCamPR*distCamPR-fabs(currPR[2]-initPR[2])*fabs(currPR[2]-initPR[2]))
							/(2*distCamIPR*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta2/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
		}
		
		else if (strcmp(idiomes[i].name,"track")==0) {
			// seul pasY va etre affecte : bcp plus facile car pas d'angles a prendre en compte.
			
			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine le signe :
			int sgn;
			if (initPR[1]-currPR[1]<0)
				// deplacement "de la gauche vers la droite", donc signe positif.
				sgn=1;
			else sgn=-1;
			
			pasY1=sgn*fabs(currPR[1]-initPR[1]);
			// on normalise pour obtenir le pas :
			pasY1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/
			
			pasY0=pasY1; // logique..
		
			/******************************************
			// ********* camera 1 : droite ************
			******************************************/
			
			pasY2=pasY1;
		}
		
		/***********************
		* Chemin des cameras :
		***********************/
		
		for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots.back().lastInd; j++) {
			resConfigs[1].push_back(configCam1);
			resConfigs[0].push_back(configCam0);
			resConfigs[2].push_back(configCam2);
			
			// calcul de la nouvelle position pour chaque camera :
			configCam1.posY+=pasY1;
			configCam1.anglePlan+=pasPhi1;
			configCam1.angleVert+=pasTheta1;
			
			configCam0.posY+=pasY0;
			configCam0.anglePlan+=pasPhi0;
			configCam0.angleVert+=pasTheta0;
			
			configCam2.posY+=pasY2;
			configCam2.anglePlan+=pasPhi2;
			configCam2.angleVert+=pasTheta2;
		}
	}

	free(vecteur);
	free(initPR);

	return resConfigs;
}
