#ifndef _Constantes
#define _Constantes

#include <cstdlib>
#include <cmath>

class Constantes {

public :
	
	Constantes () {}
	
	~Constantes () {}

	static const int DIM=8; // nombre de degres de libertes du bras-robot

	static const double RAY_STATION=100; // rayon du cylindre definissant la station (tout est en METRES !)
	static const double RAY_STATION_2=10000; // au carre
	static const double LONG_STATION=1000; // longueur de la station, ==hauteur du cylindre
	// vraies valeurs : 108m x 74m

	// constantes d'echantillonnage :
	static const int E_PTS=50;
	static const int C_PTS=50;
	static const int P_PTS=20;

	// nombre maximum d'essais pr obtenir une bonne visibilite :
	static const int MAX_TRIAL_VIS=100;

	// dimensions du bras :
	static const double RAY_C=0.175; // rayon de la coupe d'un cylindre de bras
	static const double RAY_P=0.3; // rayon de la sphere modelisant le poignet
	static const double LONG_EC=8.5; // longueur de l'epaule au coude
	static const double LONG_CP=8.5; // longueur du coude au poignet

	// pas elementaire lors d'un test de collision,
	// lors du calcul de l'image formee par le bras sur le plan de vision de la camera :
	static const double PAS_ELEM=0.5;

	// les coefficients de poids des structures du bras
	static const double COEFF_EP=5;
	static const double COEFF_CO=5;
	static const double COEFF_PO=1;

	static const double MAX_DIST=100000000;
	static const int IM_SEC=24; // nombres d'images par seconde de film

	// seul de tolerance pour les tests a zero (1 millimetre)
	static const double SEUIL_TOL=0.001;

	static const double CHAMP_POIGNET=1.0; // champ de vision du poignet (==diametre), en metres

	static const double ANGLE_CAM=M_PI_4; // demi-angle de vue de chaque camera; valeur arbitraire a corriger.
	static const double TAN_ANGLE=1.0;
	static const double TAN_2_ANGLE=1.0;

	// constante marge de vue :
	static const double MARGE_VUE=1.4;
	
	static const double SEUIL=0.001; // seuil de variation effective dans le mouvement des joints;
									// depend du pas de temps choisi pour la discretisation...
	static const double EPSILON=0.2; // seuil de fusion des segments
		
	// seuils sur l'ecart-type (en metres) :
	static const double S_MIN=0.5;
	static const double S_MAX=10.5;
	static const double S_INC=1.0;

	// seuil a partir duquel est detecte un saut (delicat a regler..)
	static const double SEUIL_OBS=3.0;

	// seuil sur distance aux obstacles :
	static const double SEUIL_DIST_OBST=5.0;
	
	static const int NB_ECH_PTS=13; // nombre de points a echantillonner pr test du shot track
	
	static const int NB_CAMS=14; // nombre de cameras fixees a la station
	
	static const double SEUIL_LISS=3.0; // seuil == frontiere deplacement / saut
								// meme role que SEUIL_OBS dans DecoupObstacles.cpp
	
	static const int NB_VIEWS=7; // nombre de vues differentes dans PlaceCamera.cpp
	
	static const double SEUIL_PREDOM=0.4; // seuil de predominance pour depct_negligeable / depct_remarquable

	static const double SEUIL_RAIL=2.0; // deplacement de 2 metres au moins pour que l'on enregistre principalement deplacement sur rail
										// a ajuster...

	static const int MIN_DEC_SIZE=7; // taille minimale du vecteur des points de coupure pour prendre en compte une moyenne
									// a ajuster...
	
	// seuil de visibilite minimal
	static const double SEUIL_VIS=0.7;
};


#define DIM Constantes::DIM
#define RAY_STATION Constantes::RAY_STATION
#define RAY_STATION_2 Constantes::RAY_STATION_2
#define LONG_STATION Constantes::LONG_STATION
#define E_PTS Constantes::E_PTS
#define C_PTS Constantes::C_PTS
#define P_PTS Constantes::P_PTS
#define MAX_TRIAL_VIS Constantes::MAX_TRIAL_VIS
#define RAY_C Constantes::RAY_C
#define RAY_P Constantes::RAY_P
#define LONG_EC Constantes::LONG_EC
#define LONG_CP Constantes::LONG_CP
#define PAS_ELEM Constantes::PAS_ELEM
#define COEFF_EP Constantes::COEFF_EP
#define COEFF_CO Constantes::COEFF_CO
#define COEFF_PO Constantes::COEFF_PO
#define MAX_DIST Constantes::MAX_DIST
#define IM_SEC Constantes::IM_SEC
#define SEUIL_TOL Constantes::SEUIL_TOL
#define CHAMP_POIGNET Constantes::CHAMP_POIGNET
#define ANGLE_CAM Constantes::ANGLE_CAM
#define TAN_ANGLE Constantes::TAN_ANGLE
#define TAN_2_ANGLE Constantes::TAN_2_ANGLE
#define MARGE_VUE Constantes::MARGE_VUE
#define SEUIL Constantes::SEUIL
#define EPSILON Constantes::EPSILON
#define S_MIN Constantes::S_MIN
#define S_MAX Constantes::S_MAX
#define S_INC Constantes::S_INC
#define SEUIL_OBS Constantes::SEUIL_OBS
#define SEUIL_DIST_OBST Constantes::SEUIL_DIST_OBST
#define NB_ECH_PTS Constantes::NB_ECH_PTS
#define NB_CAMS Constantes::NB_CAMS
#define SEUIL_LISS Constantes::SEUIL_LISS
#define NB_VIEWS Constantes::NB_VIEWS
#define SEUIL_PREDOM Constantes::SEUIL_PREDOM
#define SEUIL_RAIL Constantes::SEUIL_RAIL
#define MIN_DEC_SIZE Constantes::MIN_DEC_SIZE
#define SEUIL_VIS Constantes::SEUIL_VIS

#endif
