#include "Functions.h"

// fonction detectant si collision:
/* renvoit 1 si collision avec obstacle,
		0 si pas de collision,
		-1 si collision avec station */
int Functions::collision(double* point) {

	// test si collision avec le cylindre de la station :
	if (point[0]*point[0]+point[2]*point[2]<RAY_STATION_2) return -1;
	
	// petit test 1 : un seul obstacle de z=400 a 450, x=RAY_STATION+3 a RAY_STATION+5, y=-2 a 2
//	if (point[2]>400 && point[2]<450 && point[0]>RAY_STATION+3
//		&& point[0]<RAY_STATION+5 && point[1]>-2 && point[1]<2) return true; // l'obstacle
	
	// petit test 2 : 4 obstacles, voir rapport 3.1.3
	if (point[1]>50 && point[1]<150 && point[0]>RAY_STATION
		&& point[0]<RAY_STATION+5 && point[2]>-4 && point[2]<-1) return 1;
	if (point[1]>350 && point[1]<450 && point[0]>RAY_STATION+10
		&& point[0]<RAY_STATION+15 && point[2]>-2 && point[2]<2) return 1;
	if (point[1]>550 && point[1]<650 && point[0]>RAY_STATION
		&& point[0]<RAY_STATION+5 && point[2]>1 && point[2]<4) return 1;
	if (point[1]>850 && point[1]<950 && point[0]>RAY_STATION+10
		&& point[0]<RAY_STATION+15 && point[2]>-4 && point[2]<-1) return 1;

	return 0;
	
	// UTILISE PQP :
	
	// setup pqp model with one tiny triangle,
	// to use pqp for distance queries --> construction du pqp_model pr le point test
//	double p[][3]={{0,0,0},{1e-10,0,0},{0,1e-10,0}};
//	PQP_Model pointColl=new PQP_Model();
//	pointColl->BeginModel();
//	pointColl->AddTri(p[0],p[1],p[2],0);
//	pointColl->EndModel();
	
/*
	int 
PQP_Collide(PQP_CollideResult *result,
            PQP_REAL R1[3][3], PQP_REAL T1[3], PQP_Model *o1,
            PQP_REAL R2[3][3], PQP_REAL T2[3], PQP_Model *o2,
            int flag = PQP_ALL_CONTACTS);
*/
//return 0;
	// check de ce point avec tous les autres PQP_model :
	// demain !!
}

// verifie si 3 points sont colineaires :
bool Functions::colineaires(double* point1,double* point2,double* point3) {
	double csteProp;
	if (point2[0]-point1[0]!=0.0)
		csteProp=(point3[0]-point2[0])/(point2[0]-point1[0]);
	else if (point2[1]-point1[1]!=0.0)
		csteProp=(point3[1]-point2[1])/(point2[1]-point1[1]);
	else // au moins une coordonnee non nulle, donc..
		csteProp=(point3[2]-point2[2])/(point2[2]-point1[2]);

	for (int i=2; i>=0; i--)
		if (fabs(csteProp*(point2[i]-point1[i])-point3[i]-point2[i])>SEUIL_TOL) return false;
	return true;
}

// fonction calculant le point de repere courant suivant les parametres :
void Functions::getRepPoint(int indice) {
	if (handPoints[indice]) {
		for (int i=2; i>=0; i--) currPR[i]=coord3D[6+i];
	}
	else if (zoomObstPoints[indice]) {
		currPR[0]=(coordObstacles[indice][0]+coordPointsB[indice][0])/2;
		currPR[1]=(coordObstacles[indice][1]+coordPointsB[indice][1])/2;
		currPR[2]=(coordObstacles[indice][2]+coordPointsB[indice][2])/2;
	}
	else {
		currPR[0]=(COEFF_EP*((coord3D[0]+coord3D[3])/2)+COEFF_CO*((coord3D[3]+coord3D[6])/2)+COEFF_PO*coord3D[6])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
		currPR[1]=(COEFF_EP*((coord3D[1]+coord3D[4])/2)+COEFF_CO*((coord3D[4]+coord3D[7])/2)+COEFF_PO*coord3D[7])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
		currPR[2]=(COEFF_EP*((coord3D[2]+coord3D[5])/2)+COEFF_CO*((coord3D[5]+coord3D[8])/2)+COEFF_PO*coord3D[8])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
	}

	return;
}

// renvoit la matrice de passage en fonction des angles phi et theta :
void Functions::getPassMatrix(double* coeffsX,double* coeffsY,double* coeffsZ,double anglePlan,double angleVert) {
	// premiere tranformation : rotation sur plan xOy
	double tmpSinP=sin(anglePlan),tmpCosP=cos(anglePlan);
	double tmpSinV=sin(angleVert),tmpCosV=cos(angleVert);
	coeffsZ[0]=tmpCosP;
	coeffsZ[1]=tmpSinP;
	// seconde transformation : elevation parallelement a l'axe Oz :
	// on obtient le vecteur (Oz)
	coeffsZ[2]=tmpCosV;
	coeffsZ[0]*=tmpSinV;
	coeffsZ[1]*=tmpSinV;

	// on en deduit les deux autres vecteurs de base
	// (le choix est tel que la base obtenue soit orthonormee DIRECTE)
	coeffsX[0]=tmpCosP*tmpCosV;
	coeffsX[1]=tmpSinP*tmpCosV;
	coeffsX[2]=-tmpSinV;
	coeffsY[0]=-tmpSinP;
	coeffsY[1]=tmpCosP;
	coeffsY[2]=0.0;

	return;
}

// fonction calculant les angles en coordonnees spheriques a partir d'un vecteur :
void Functions::getSphericAngles(double* vecteur,double& phi,double& theta) {
	// transformation en coordonnees spheriques pour obtenir phi et theta :
	
	// preliminaires :
	// SEUIL_TOL permet de passer outre les imprecisions dans les calculs ne donnant pas exactement 0
	if (fabs(vecteur[0])<SEUIL_TOL) vecteur[0]=0.0;
	if (fabs(vecteur[1])<SEUIL_TOL) vecteur[1]=0.0;
	if (fabs(vecteur[2])<SEUIL_TOL) vecteur[2]=0.0;
	
	// calcul :
	if (vecteur[0]!=0.0) {
		if (vecteur[1]!=0.0) {
			phi=atan(vecteur[1]/vecteur[0]);
			// attention phi n'est pas valide dans le cas ou l'on n'est pas dans la partie droite du cadran :
			if (vecteur[0]<0.0) {
				if (vecteur[1]<0.0) phi-=M_PI;
				else phi+=M_PI;
			}
		}
		else {
			if (vecteur[0]<0.0) phi=M_PI;
			else phi=0.0;
		}
	}
	else {
		if (vecteur[1]<0.0) phi=-M_PI_2;
		else phi=M_PI_2; // inclus le cas ou x=y=0 (alors angle arbitraire)
	}
	if (vecteur[2]!=0.0) {
		theta=atan((sqrt(vecteur[0]*vecteur[0]+vecteur[1]*vecteur[1]))/vecteur[2]);
		if (vecteur[2]<0.0) {
			// alors il faut encore ajouter pi pour avoir le resultat souhaite :
			theta+=M_PI;
		}
	}
	else theta=M_PI_2;
	
	return;
}

// fonction effectuant une rotation du point vecteur, d'angle angle autour de l'axe coeffsZ :
double* Functions::rotatePoint(double* coeffsX,double* coeffsY,double* coeffsZ,double angle,double* vecteur) {
	// matrice de la rotation dans base adaptee (M[app,e']) : 
	/*	(cos(angle) -sin(angle) 0 )
		(sin(angle) cos(angle)  0 )
		(     0         0       1 ) */
	// Donc :
	
	double* res=(double*)malloc(3*sizeof(double));
	// a) inv(P(e,e')) X == Interm :
	double* interm=(double*)malloc(3*sizeof(double));
	interm[0]=coeffsX[0]*vecteur[0]+coeffsX[1]*vecteur[1]+coeffsX[2]*vecteur[2];
	interm[1]=coeffsY[0]*vecteur[0]+coeffsY[1]*vecteur[1]+coeffsY[2]*vecteur[2];
	interm[2]=coeffsZ[0]*vecteur[0]+coeffsZ[1]*vecteur[1]+coeffsZ[2]*vecteur[2];
	// b) M[app,e'] Interm == Interm2
	double* interm2=(double*)malloc(3*sizeof(double));
	double cosAngle=cos(angle),sinAngle=sin(angle);
	interm2[0]=cosAngle*interm[0]-sinAngle*interm[1];
	interm2[1]=sinAngle*interm[0]+cosAngle*interm[1];
	interm2[2]=interm[2];
	free(interm);
	// c) P(e,e') Interm2 == resultat final (image du point)
	res[0]=coeffsX[0]*interm2[0]+coeffsY[0]*interm2[1]+coeffsZ[0]*interm2[2];
	res[1]=coeffsX[1]*interm2[0]+coeffsY[1]*interm2[1]+coeffsZ[1]*interm2[2];
	res[2]=coeffsX[2]*interm2[0]+coeffsY[2]*interm2[1]+coeffsZ[2]*interm2[2];
	free(interm2);

	return res;
}

// calcul de la distance de vue :
void Functions::getViewDistance(int indice) {
	if (handPoints[indice]) {
		distCamPR=TAN_ANGLE*(CHAMP_POIGNET*MARGE_VUE)/2.0;
	}
	else if (zoomObstPoints[indice]) {
		distCamPR=TAN_ANGLE*distObstacles[indice]*MARGE_VUE;
	}
	else {
		// calcul de l'ecart maximal entre deux points du bras,
		// pour connaitre l'amplitude que l'on doit filmer :
		double maxEcart=sqrt((coord3D[6]-coord3D[0])*(coord3D[6]-coord3D[0])
							+ (coord3D[7]-coord3D[1])*(coord3D[7]-coord3D[1])
							+ (coord3D[8]-coord3D[2])*(coord3D[8]-coord3D[2]));
		if (LONG_EC>maxEcart) maxEcart=LONG_EC;
		if (LONG_CP>maxEcart) maxEcart=LONG_CP;
		// il faut voir maxEcart*MARGE_VUE, au moins : 
		distCamPR=TAN_ANGLE*(maxEcart*MARGE_VUE)/2.0;
	}
	
	return;
}
