#include "TransfIdioms.h"

#include <iostream>
using namespace std;

/*************************************************************************
* certaines lignes commentees car cette amelioration
* n'est pas encore prise en charge lors de la compilation.
* de plus : cela revient presque au meme, en terme de rendu final;
* c'est juste plus dans l'esprit "idiome" d'effectuer cette amelioration
**************************************************************************/

// transformation de la trajectoire en une suite d'idiomes :
// pas de prise en compte des proximites aux obstacles dans cette version (!)
vector<Idiom> path2idioms(const vector<Configuration>& configurations) {

	vector<Idiom> resIdioms;
	
	// on recupere les points de coupure monotones :
	analyseMonotone(configurations);

	// determination du premier idiome (jamais pris en compte dans l'algo qui va suivre) :
	
	double depctPhi,depctThetaE,depctThetaC,depctRoll,depctRail;
	if (decoupageM.empty()) {
		depctPhi=fabs(configurations[configurations.size()-1][2]-configurations[0][2]);
		depctThetaE=fabs(configurations[configurations.size()-1][1]-configurations[0][1]);
		depctThetaC=fabs(configurations[configurations.size()-1][4]-configurations[0][4]);
		depctRoll=fabs(configurations[configurations.size()-1][3]-configurations[0][3]);
		depctRail=fabs(configurations[configurations.size()-1][0]-configurations[0][0]);
	}
	else {
		depctPhi=fabs(configurations[decoupageM[0]-1][2]-configurations[0][2]);
		depctThetaE=fabs(configurations[decoupageM[0]-1][1]-configurations[0][1]);
		depctThetaC=fabs(configurations[decoupageM[0]-1][4]-configurations[0][4]);
		depctRoll=fabs(configurations[decoupageM[0]-1][3]-configurations[0][3]);
		depctRail=fabs(configurations[decoupageM[0]-1][0]-configurations[0][0]);
	}

	int lastMouv=-1; // 0 : shot i) du memoire (elevation verticale)
					 // 1 : shot ii) du memoire (rotation sur phi)
					 // 2 : shot iii) du memoire (rotation sur dzeta == roll)
					 // 3 : shot iv) du memoire (mouvement de la main)
					 // 4 : shot v) du memoire (translation sur le rail)
					 // -1 : idiome indetermine
	
	// test si mouvement de la main seule :
	if (handPoints[0]) {
		resIdioms.push_back(idiomsBase[0]);
		// initialisation des angles de vue privilegies pour l'idiome :
		resIdioms.back().views.push_back(4);
		resIdioms.back().views.push_back(5);
		lastMouv=3;
	}
	
	else {
		// d'abord test si mouvement sur le rail significatif :
		if (depctRail>SEUIL_RAIL) {
			resIdioms.push_back(idiomsBase[3]);
			// initialisation des angles de vue privilegies pour l'idiome :
			resIdioms.back().views.push_back(4);
			resIdioms.back().views.push_back(5);
			resIdioms.back().views.push_back(0);
			resIdioms.back().views.push_back(1);
			resIdioms.back().views.push_back(6);
			resIdioms.back().views.push_back(7);
			lastMouv=4;
		}

		else {
			// test si predominance d'un mouvement angulaire 
			if (depctPhi>SEUIL_TOL && depctThetaE/depctPhi<SEUIL_PREDOM
								&& depctThetaC/depctPhi<SEUIL_PREDOM
								&& depctRoll/depctPhi<SEUIL_PREDOM) {
				resIdioms.push_back(idiomsBase[1]);
				// initialisation des angles de vue privilegies pour l'idiome :
				resIdioms.back().views.push_back(4);
				resIdioms.back().views.push_back(5);
				resIdioms.back().views.push_back(7);
				resIdioms.back().views.push_back(6);
				resIdioms.back().views.push_back(2);
				resIdioms.back().views.push_back(3);
				lastMouv=1;
			}

			else if (depctRoll>SEUIL_TOL && depctThetaE/depctRoll<SEUIL_PREDOM
									&& depctThetaC/depctRoll<SEUIL_PREDOM
									&& depctPhi/depctRoll<SEUIL_PREDOM) {
				resIdioms.push_back(idiomsBase[0]);
				// initialisation des angles de vue privilegies pour l'idiome :
				resIdioms.back().views.push_back(7);
				resIdioms.back().views.push_back(6);
				resIdioms.back().views.push_back(4);
				resIdioms.back().views.push_back(5);
				lastMouv=2;
			}

			else if ((depctThetaE>SEUIL_TOL && depctPhi/depctThetaE<SEUIL_PREDOM
										&& depctRoll/depctThetaE<SEUIL_PREDOM)
					|| (depctThetaC>SEUIL_TOL && depctPhi/depctThetaC<SEUIL_PREDOM
										&& depctRoll/depctThetaC<SEUIL_PREDOM)) {
				resIdioms.push_back(idiomsBase[2]);
				// initialisation des angles de vue privilegies pour l'idiome :
				resIdioms.back().views.push_back(2);
				resIdioms.back().views.push_back(3);
				resIdioms.back().views.push_back(4);
				resIdioms.back().views.push_back(5);
				resIdioms.back().views.push_back(7);
				lastMouv=0;
			}

			else {
				// mouvement indetermine : on filme simplement en suivant le centre de gravite.
				// on enregistre pas d'idiome particulier :
				resIdioms.push_back(idiomsBase[4]);
				lastMouv=-1;
			}
		}
	}

	resIdioms.back().shots[0].firstInd=0;
	if (decoupageM.empty()) resIdioms.back().shots.back().lastInd=configurations.size()-1;

	// preliminaire : calcul de la moyenne des amplitudes de deplacement le long du rail
	// si pas assez de valeurs : on utilise un seuil (oblige..)
	double moyDepcRail=0.0;
	if ((int)decoupageM.size()>MIN_DEC_SIZE) {
		for (int i=1; i<(int)decoupageM.size(); i++) {
			moyDepcRail+=fabs(configurations[decoupageM[i]-1][0]-configurations[decoupageM[i-1]][0]);
		}
		moyDepcRail/=(decoupageM.size()-1);
		// ainsi un deplacement sera significativement en translation s'il est plus grand que 2*cette moyenne
	}
	else moyDepcRail=SEUIL_RAIL;

	for (int i=0; i<(int)decoupageM.size(); i++) {
		
		// test si mouvement de la main seule :
		if (handPoints[decoupageM[i]]) {
//			if (lastMouv!=3) {
				resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
				resIdioms.push_back(idiomsBase[0]);
				resIdioms.back().shots[0].firstInd=decoupageM[i];
				// initialisation des angles de vue privilegies pour l'idiome :
				resIdioms.back().views.push_back(4);
				resIdioms.back().views.push_back(5);
//			}
			lastMouv=3;
			// sinon on ne fait rien : mouvement de la main qui continue.
		}
		
		else {
			if (i<(int)decoupageM.size()-1) {
				depctPhi=fabs(configurations[decoupageM[i+1]-1][2]-configurations[decoupageM[i]][2]);
				depctThetaE=fabs(configurations[decoupageM[i+1]-1][1]-configurations[decoupageM[i]][1]);
				depctThetaC=fabs(configurations[decoupageM[i+1]-1][4]-configurations[decoupageM[i]][4]);
				depctRoll=fabs(configurations[decoupageM[i+1]-1][3]-configurations[decoupageM[i]][3]);
				depctRail=fabs(configurations[decoupageM[i+1]-1][0]-configurations[decoupageM[i]][0]);
			}
			else {
				depctPhi=fabs(configurations[configurations.size()-1][2]-configurations[decoupageM[i]][2]);
				depctThetaE=fabs(configurations[configurations.size()-1][1]-configurations[decoupageM[i]][1]);
				depctThetaC=fabs(configurations[configurations.size()-1][4]-configurations[decoupageM[i]][4]);
				depctRoll=fabs(configurations[configurations.size()-1][3]-configurations[decoupageM[i]][3]);
				depctRail=fabs(configurations[configurations.size()-1][0]-configurations[decoupageM[i]][0]);
			}

			if (depctRail>moyDepcRail*0.6) { // facteur 0.6 a ajuster......
//				if (lastMouv!=4) {
					resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
					resIdioms.push_back(idiomsBase[3]);
					resIdioms.back().shots[0].firstInd=decoupageM[i];
					// initialisation des angles de vue privilegies pour l'idiome :
					resIdioms.back().views.push_back(4);
					resIdioms.back().views.push_back(5);
					resIdioms.back().views.push_back(0);
					resIdioms.back().views.push_back(1);
					resIdioms.back().views.push_back(6);
					resIdioms.back().views.push_back(7);
//				}
				lastMouv=4;
			}

			// test si predominance d'un mouvement angulaire 
			else if (depctPhi>SEUIL_TOL && depctThetaE/depctPhi<SEUIL_PREDOM
								&& depctThetaC/depctPhi<SEUIL_PREDOM
								&& depctRoll/depctPhi<SEUIL_PREDOM) {
//				if (lastMouv!=1) 
					resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
					resIdioms.push_back(idiomsBase[1]);
					resIdioms.back().shots[0].firstInd=decoupageM[i];
					// initialisation des angles de vue privilegies pour l'idiome :
					resIdioms.back().views.push_back(4);
					resIdioms.back().views.push_back(5);
					resIdioms.back().views.push_back(7);
					resIdioms.back().views.push_back(6);
					resIdioms.back().views.push_back(2);
					resIdioms.back().views.push_back(3);
//				}
				lastMouv=1;
			}

			else if (depctRoll>SEUIL_TOL && depctThetaE/depctRoll<SEUIL_PREDOM
									&& depctThetaC/depctRoll<SEUIL_PREDOM
									&& depctPhi/depctRoll<SEUIL_PREDOM) {
//				if (lastMouv!=2) {
					resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
					resIdioms.push_back(idiomsBase[0]);
					resIdioms.back().shots[0].firstInd=decoupageM[i];
					// initialisation des angles de vue privilegies pour l'idiome :
					resIdioms.back().views.push_back(7);
					resIdioms.back().views.push_back(6);
					resIdioms.back().views.push_back(4);
					resIdioms.back().views.push_back(5);
//				}	
				lastMouv=2;
			}

			else if ((depctThetaE>SEUIL_TOL && depctPhi/depctThetaE<SEUIL_PREDOM
										&& depctRoll/depctThetaE<SEUIL_PREDOM)
					|| (depctThetaC>SEUIL_TOL && depctPhi/depctThetaC<SEUIL_PREDOM
										&& depctRoll/depctThetaC<SEUIL_PREDOM)) {
//				if (lastMouv!=0) {
					resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
					resIdioms.push_back(idiomsBase[2]);
					resIdioms.back().shots[0].firstInd=decoupageM[i];
					// initialisation des angles de vue privilegies pour l'idiome :
					resIdioms.back().views.push_back(2);
					resIdioms.back().views.push_back(3);
					resIdioms.back().views.push_back(4);
					resIdioms.back().views.push_back(5);
					resIdioms.back().views.push_back(7);
//				}	
				lastMouv=0;
			}

			else {
				// mouvement indetermine : on filme simplement en suivant le centre de gravite.
				// on enregistre pas d'idiome particulier.
				if (lastMouv!=-1) {
					resIdioms.back().shots.back().lastInd=decoupageM[i]-1;
					resIdioms.push_back(idiomsBase[4]);
					resIdioms.back().shots[0].firstInd=decoupageM[i];
				}
				lastMouv=-1;
			}
		}
	}

	resIdioms.back().shots.back().lastInd=configurations.size()-1;
	return resIdioms;
}
