#include "View.h"

// renvoit un reel entre 0 et 1 indiquant la visibilite pour cette vue avec cette camera :
// retourne la position calculee dans positionCam
double testView(Camera& cam,int indexView) {
	
	// on commence par placer la camera selon la vue demandee :
	putCamInView(cam,indexView);
	
	// puis on evalue cette vue
	return cam.getVisibilite();
}

// fonction placant la camera dans la vue demandee :
void putCamInView(Camera& cam,int indexView) {

	// coordonnees relatives des joints coude et poignet :
	double cX=coord3D[3]-coord3D[0],cY=coord3D[4]-coord3D[1],cZ=coord3D[5]-coord3D[2],
			cpX=coord3D[6]-coord3D[3],cpY=coord3D[7]-coord3D[4],cpZ=coord3D[8]-coord3D[5];
	
	double dirX=0.0,dirY=1.0,dirZ=0.0; // directions a partir du centre de gravite,
										// pour trouver le placement de la camera.
	
	// initialisation de la position de la camera au point de repere courant
	// (centre de gravite, main ou milieu d'un segment obstacle-bras)
	cam.posX=currPR[0];
	cam.posY=currPR[1];
	cam.posZ=currPR[2];
	
	switch (indexView) {

	/****************************************************
	* code utile a la partie resolution de contraintes :
	*****************************************************/

	case 0 :
	case 1 :
		// on place la camera orthogonalement au plan forme par les 3 joints du robot,
		// a distance correcte du bras par rapport aux obstacles proches ou non,
	
		// translation a partir du centre de gravite, orthogonalement au plan forme par les 3 joints;
		// si les 3 joints sont colineaires, alors..
		if (colineaires(coord3D,coord3D+3,coord3D+6)) {
			// on se place alors parallelement a l'axe de la station (cas rarissime..),
			// du cote ou l'on voit le poignet le plus loin.
			if (coord3D[7]<coord3D[1]) dirY=-1.0;
		}
		else {
			// on suppose maintenant que les 3 joints ne sont pas colineaires :
			// calcul de la direction orthogonale:
			double invDet=1.0/(cZ*cpX-cpZ*cX);
			dirX=invDet*(cZ*cpY-cY*cpZ);
			dirZ=invDet*(cX*cpY-cY*cpX);
			
			// calcul de l'angle avec l'orthogonal de l'horizontale locale de la station :
			// vecteur (0,coord3D[1],0) --> (coord3D[0],coord3D[1],coord3D[2])
			// comme coord3D[2]==0.0 et coord3D[0]>0, le code se reduit a :
			if (fabs(dirX)<SEUIL_TOL) {
				if (indexView==1) {
					dirX=-dirX;
					dirY=-dirY;
					dirZ=-dirZ;
				}
			}
			else if ((indexView==0 && dirX<0) || (indexView==1 && dirX>0)) {
				dirX=-dirX;
				dirY=-dirY;
				dirZ=-dirZ;
			}
		}
		
		{
			double rProp=distCamPR/sqrt(dirX*dirX+1+dirZ*dirZ);
			dirX*=rProp;
			dirY*=rProp;
			dirZ*=rProp;
			// translations le long de l'axe (point de repere-->dir),
			// pour etre a la bonne distance.
			cam.posX+=dirX;
			cam.posY+=dirY;
			cam.posZ+=dirZ;
		}
		break;

	case 2 :
	case 3 :
		// il suffit cette fois de se placer dans le sens de l'axe Oy a partir du centre de gravite,
		// puis d'aller dans la direction telle que le poignet soit plus proche ou plus loin, suivant indexView
		if (fabs(coord3D[7]-coord3D[1])<SEUIL_TOL) {
			if (indexView==3) dirY=-1;
		}
		else if ((indexView==2 && coord3D[7]<coord3D[1]) || (indexView==3 && coord3D[7]>coord3D[1])) dirY=-1;

		// translations le long de l'axe (point de repere-->dir),
		// pour etre a la bonne distance.
		cam.posY+=dirY*distCamPR;
		break;

	case 4 :
	case 5 :
		// vue 3/4 au dessus du bras
		
		{
			// a) elevation du point de repere (souvent centre de gravite) selon l'axe Oz,
			// d'une hauteur "raisonnable" == longueur du bras deplie ;
			// coordonnees de ce point relativement a la base du bras :
			double* point=(double*)malloc(3*sizeof(double));
			point[0]=currPR[0]-coord3D[0];
			point[1]=currPR[1]-coord3D[1];
			point[2]=currPR[2]+LONG_EC+LONG_CP-coord3D[2];
	
			/****************************************************************
			* b) rotation d'axe (Ce, Cg) (voir rapport), d'angle pi/4,
				 du cote defini par indexView
			*****************************************************************/
			
			// on determine et applique la matrice de rotation comme dans Configuration.cpp :
			
			// recuperation des angles des coordonnees spheriques :
			double vecteur[]={currPR[0]-coord3D[0],currPR[1]-coord3D[1],currPR[2]-coord3D[2]};
			double phi,theta;
			getSphericAngles(vecteur,phi,theta);
	
			// on en deduit le vecteur Ce-->Cg en coordonnees normalisees "de base" :
			double *coeffsX=(double*)malloc(3*sizeof(double)),
					*coeffsY=(double*)malloc(3*sizeof(double)),
					*coeffsZ=(double*)malloc(3*sizeof(double));
			getPassMatrix(coeffsX,coeffsY,coeffsZ,phi,theta);
			
			// matrice de la rotation dans cette base (M[app,e']) : 
			/*	(cos(pi/4) -sin(pi/4) 0 )
				(sin(pi/4) cos(pi/4)  0 )
				(   0         0       1 ) */

			// suivant indexView, on se place du ct du plus petit angle entre l'axe Oy et la droite (Ce, Cg),
			// ou du plus grand angle ; calculs faciles car vecteur sur Oy == (0, 1, 0) :
			int sgn=1;
			if (vecteur[1]<0) sgn=-1;
			double* apresRotate;
			if (indexView==4) apresRotate=rotatePoint(coeffsX,coeffsY,coeffsZ,sgn*M_PI_4,point);
			else apresRotate=rotatePoint(coeffsX,coeffsY,coeffsZ,-sgn*M_PI_4,point);
			cam.posX=coord3D[0]+apresRotate[0];
			cam.posY=coord3D[1]+apresRotate[1];
			cam.posZ=coord3D[2]+apresRotate[2];
			free(apresRotate);
			free(coeffsX);
			free(coeffsY);
			free(coeffsZ);
			free(point);
			
			/***********************
			* Fin de la rotation
			***********************/
		}
		
		{
			// ajustement de la distance au centre de gravite :
			double factNorm=distCamPR/(sqrt((cam.posX-currPR[0])*(cam.posX-currPR[0])
								+ (cam.posY-currPR[1])*(cam.posY-currPR[1])
								+ (cam.posZ-currPR[2])*(cam.posZ-currPR[2])));
			cam.posX=currPR[0]+(cam.posX-currPR[0])*factNorm;
			cam.posY=currPR[1]+(cam.posY-currPR[1])*factNorm;
			cam.posZ=currPR[2]+(cam.posZ-currPR[2])*factNorm;
		}
		break;

	case 6 :
		// vue aerienne : on se contente de placer la camera assez haut "au dessus"
		// du centre de gravite du bras (memes coordonnees x et y, z >> z[Cg] )
		cam.posZ+=distCamPR;
		break;

	/***********************************************************
	* fin du code utile a la partie resolution de contraintes ;
	************************************************************/
	
	case 7:
		// vue face au bras : on reste au niveau du point de repere,
		// en changeant l'abscisse x :
		cam.posX+=distCamPR;
		break;

	default :
		return; // si on arrive la c'est que l'appel s'est fait avec de mauvais parametres
	}
	
	return;
}
