// a renommer en FilmRT.cpp dans RT ! 
// + ajouter header et legerement modifier lecture des configurations

#include <cstdlib>
#include <cmath>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "Functions.h"
#include "DecoupMonotone.h"
#include "DecoupObstacles.h"
#include "ZonesToIdioms.h"
#include "Constantes.h"
#include "ConfigCamera.h"
#include "PlaceCamera.h"
#include "IdiomBase.h"
#include "Lissage.h"
#include "ZoneBase.h"
#include <fstream>
#include "TransfIdioms.h"
#include "CompilIdioms.h"
#include "FilmageFixe.h"

using namespace std;

int main (int argc, char** argv) {

	// UTILISATION :
	// FilmRT.exe <fichier_trajectoire> <trajectoire connue ou non (c ou i resp.)> <types de filmage>
	
	// prend en argument une suite de vecteurs  8 dimensions;
	// ceux-ci correspondent aux configurations successives du bras-robot
	ifstream dataFile(argv[1]);
	string line;
	string::iterator strIter1,strIter2;
	int countDim,count=0;
	vector<Configuration> configurations; // les points successifs le long de chaque dimension
	while (getline(dataFile,line,'\n')) { // format d'une ligne: \n<float1>[ ]<float2>[ ]..[ ]<float7>\n
		configurations.push_back(Configuration());
		strIter1=line.begin();
		countDim=0;
		while (true) {
			strIter2=strIter1;
			while (*strIter2!=' ' && strIter2!=line.end()) strIter2++;
			configurations[count][countDim++]=toFloat(string(strIter1,strIter2));
			if (countDim==DIM) break;
			strIter1= ++strIter2;
		}
		count++;
	}
	Constantes cons=Constantes();
	Functions func=Functions();
	Global glob=Global();
	currPR=(double*)malloc(3*sizeof(double));
	
	// calcul de chaque configuration de camera, une par une :
	vector<ConfigCamera> configsCam1,configsCam0,configsCam2;
	if (argv[2][0]=='c') {
		// a) on teste en connaissant la trajectoire a l'avance :
		// deux cas : filmage 'contraintes' ou 'idiomes'
		
		if (argv[3][0]=='c') {
			// decoupage de la trajectoire
			analyseMonotone(configurations);
			analyseObstacles(configurations);
			
			coord3D=configurations[0].conf2coord3D();
			PlaceCamera placeCam=PlaceCamera(); // l'objet servant a (de)placer les cameras

			indiceG=0; // indice courant de parcours de la trajectoire
			
			// determination du point de repere courant :
			getRepPoint(0);

			// calcul de la distance de vue :
			getViewDistance(0);
	
			// camera centrale :
			placeCam.initCamPos(1);
			configsCam1.push_back(placeCam.cameras[1].getConfig());

			// camera de gauche :
			placeCam.initCamPos(0);
			configsCam0.push_back(placeCam.cameras[0].getConfig());

			// camera de droite :
			placeCam.initCamPos(2);
			configsCam2.push_back(placeCam.cameras[2].getConfig());

			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int i=1; i<(int)configurations.size(); i++) {
				indiceG++;
				coord3D=configurations[i].conf2coord3D();
	
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
	
				// calcul du nouveau point de repere :
				getRepPoint(i);

				// calcul de la distance de vue :
				getViewDistance(i);
		
				for (int j=2; j>=0; j--) depctRepPoint[j]=currPR[j]-lastRepPoint[j];

				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam1.push_back(placeCam.cameras[1].getConfig());
			
				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam0.push_back(placeCam.cameras[0].getConfig());
	
				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam2.push_back(placeCam.cameras[2].getConfig());
			}

			// lissage :
			configsCam1=lisse(configsCam1,configurations);
			configsCam0=lisse(configsCam0,configurations);
			configsCam2=lisse(configsCam2,configurations);
			
			// liberations memoire :
			free(lastRepPoint);
			free(depctRepPoint);
		}
		
		else if (argv[3][0]=='i') {
			// approche idiomes :

			// constitution de la base d'idiomes
			buildIdiomsBase();

			vector<ConfigCamera>* configsCam;
			
			if (argv[4][0]=='k') {
				// approche en couloirs suggeree par Khaled --> basee sur l'environnement seulement.
				// NON IMPLEMENTE car decoupage en zones non effectue.
				// completer les fonctions buildZonesBase() et idiomsFromZones(..) des que possible.
			
				// a) constitution de la base de zones definissant les idiomes :
				buildZonesBase();
				
				// b) parcours de la trajectoire du centre de gravite
				// et choix d'un idiome sur chaque portion :
				vector<Idiom> idioms=idiomsFromZones(configurations);
				
				// c) parcours de la liste d'idiomes et gnration de la suite de configurations de (trois!) cameras correspondante
				configsCam=compilation(idioms,configurations);
			}

			else if (argv[4][0]=='t') {
				// approche basee seulement sur la nature de la trajectoire du bras :
			
				// a) parcours de la trajectoire de chaque element du bras,
				// et choix d'un idiome sur chaque portion :
				vector<Idiom> idioms=path2idioms(configurations);
			
				// b) parcours de la liste d'idiomes et generation de la suite de configurations de (trois!) cameras correspondante
				configsCam=compilation(idioms,configurations);
			}
			else {
				cerr << "Options non reconnues" << endl;
				return 1;
			}

			configsCam1=configsCam[1];
			configsCam0=configsCam[0];
			configsCam2=configsCam[2];
		}

		else {
			cerr << "Options non reconnues" << endl;
			return 1;
		}
	}
	
	else if (argv[2][0]=='i') {
		
		if (argv[3][0]=='c') {
			
			// filmage "contraintes" :
		
			// b) on simule une trajectoire imprevisible en la donnant point par point :
			// approche idiomes impossible cette fois.
	
			indiceG=0; // indice courant de parcours de la trajectoire
			coord3D=configurations[0].conf2coord3D();
	
			// decoupage "online" :
			analyseMonotoneImp(configurations[0]);
			analyseObstaclesImp();
	
			PlaceCamera placeCam=PlaceCamera(); // l'objet servant a (de)placer la camera
	
			// calcul du point de repere :
			getRepPoint(0);
			
			// calcul de la distance de vue :
			getViewDistance(0);
	
			// camera centrale :
			placeCam.initCamPos(1);
			configsCam1.push_back(placeCam.cameras[1].getConfig());
	
			// camera de gauche :
			placeCam.initCamPos(0);
			configsCam0.push_back(placeCam.cameras[0].getConfig());
	
			// camera de droite :
			placeCam.initCamPos(2);
			configsCam2.push_back(placeCam.cameras[2].getConfig());
	
			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int i=1; i<(int)configurations.size(); i++) {
				indiceG++;
				coord3D=configurations[i].conf2coord3D();
				
				// decoupage "online" :
				analyseMonotoneImp(configurations[i]);
				analyseObstaclesImp();
				
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
		
				// calcul du nouveau point de repere :
				getRepPoint(i);
				
				// calcul de la distance de vue :
				getViewDistance(i);
	
				for (int j=2; j>=0; j--) depctRepPoint[j]=currPR[j]-lastRepPoint[j];
	
				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam1.push_back(placeCam.cameras[1].getConfig());
		
				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam0.push_back(placeCam.cameras[0].getConfig());
			
				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam2.push_back(placeCam.cameras[2].getConfig());
			}
			
			// liberations memoire :
			free(lastRepPoint);
			free(depctRepPoint);
		}
		
		else if (argv[3][0]=='f') {
			// filmage avec cameras fixes, d'une trajectoire imprevisible (sinon aucun interet) :
			// calcul de la suite "ideale" de configurations de cameras.
		
			// creation des cameras fixes :
			FilmageFixe filmF=FilmageFixe();
			ConfigCamera* configsCam;
			
			indiceG=0;
			for (int i=0; i<(int)configurations.size(); i++) {
				coord3D=configurations[i].conf2coord3D();
				
				// decoupage "online" :
				analyseMonotoneImp(configurations[i]);
				analyseObstaclesImp();

				// calcul du point de repere :
				getRepPoint(i);
				
				// calcul de la distance de vue :
				getViewDistance(i);
				
				// obtention des configurations de camera :
				configsCam=filmF.getCamConfig();
				configsCam1.push_back(configsCam[1]);
				configsCam0.push_back(configsCam[0]);
				configsCam2.push_back(configsCam[2]);
				
				delete [] configsCam;
				
				indiceG++;
			}
		}
		
		else {
			cerr << "Options non reconnues" << endl;
			return 1;
		}
	}
	
	else {
		cerr << "Options non reconnues" << endl;
		return 1;
	}
	
	// affichage temporaire dans un fichier :
	ofstream toto("sortie.txt");
	toto << "[";
	for (int i=0; i<(int)configsCam1.size(); i++) {
		toto << "[" << configsCam1[i].posX << "," << configsCam1[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;
	toto << "[";
	for (int i=0; i<(int)configsCam1.size(); i++) {
		toto << "[" << configsCam1[i].posY << "," << configsCam1[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;

	toto << "[";
	for (int i=0; i<(int)configsCam0.size(); i++) {
		toto << "[" << configsCam0[i].posX << "," << configsCam0[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;
	toto << "[";
	for (int i=0; i<(int)configsCam0.size(); i++) {
		toto << "[" << configsCam0[i].posY << "," << configsCam0[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;

	toto << "[";
	for (int i=0; i<(int)configsCam2.size(); i++) {
		toto << "[" << configsCam2[i].posX << "," << configsCam2[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;
	toto << "[";
	for (int i=0; i<(int)configsCam2.size(); i++) {
		toto << "[" << configsCam2[i].posY << "," << configsCam2[i].posZ << "],";
	}
	toto << "]" << endl << endl << endl;

	// lecture du film : NON IMPLEMENTE
	// ...

/*
	// test 1 :
	double paramBras[]={700,M_PI_4,0.0,0.0,3*M_PI_4,0.0,0.0,0.0};
	Configuration conf=Configuration(paramBras);
	coord3D=conf.conf2coord3D();
	Camera cam=Camera(RAY_STATION+4,0.0,200.0,0.0,0.0,0.0,1.0);
	cout << (cam.getArmIn()) << endl;
*/

/*
 	// test 2 :

	// generation de la trajectoire (offline)
	ofstream traj("trajectoire_test2.txt");
	int i;
	for (i=0; i<300; i++) 
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=300; i<350; i++)
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< ((3+0.02*(i-299))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=350; i<450; i++)
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=450; i<500; i++)
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< (M_PI-0.02*(i-449)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=500; i<550; i++)
		traj << i << " " << ((1+0.02*(i-499))*M_PI_4) << " " << 0.0 << " " << 0.0 << " "
		<< ((3+0.02*(i-499))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=550; i<650; i++)
		traj << i << " " << M_PI_2 << " " << 0.0 << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=650; i<700; i++)
		traj << i << " " << (M_PI_2-0.02*(i-649)*M_PI_4) << " " << 0.0 << " " << 0.0 << " "
		<< (M_PI-0.02*(i-649)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=700; i<800; i++) 
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=800; i<850; i++) 
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< ((3+0.02*(i-799))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=850; i<950; i++)
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=950; i<1000; i++)
		traj << i << " " << M_PI_4 << " " << 0.0 << " " << 0.0 << " "
		<< (M_PI-0.02*(i-949)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;

	// test apres generation de trajectoire :
	ifstream dataFile(argv[1]);
	string line;
	string::iterator strIter1,strIter2;
	int countDim,count=0;
	vector<Configuration> configurations; // les points successifs le long de chaque dimension
	while (getline(dataFile,line,'\n')) { // format d'une ligne: \n<float1>[ ]<float2>[ ]..[ ]<float7>\n
		configurations.push_back(Configuration());
		strIter1=line.begin();
		countDim=0;
		while (true) {
			strIter2=strIter1;
			while (*strIter2!=' ' && strIter2!=line.end()) strIter2++;
			configurations[count][countDim++]=toFloat(string(strIter1,strIter2));
			if (countDim==DIM) break;
			strIter1= ++strIter2;
		}
		count++;
	}
	Functions func=Functions();
	Global glob=Global();

	// decoupage en fonction des obstacles :
	analyseObstacles(configurations);

	// affichage des points de coupure + distances + coordonnees des obstacles proches dans un fichier
	ofstream resTest2("resultats_test2.txt");
	for (int i=0; i<(int)distObstacles.size(); i++) {
		resTest2 << "ligne " << i << " distance a l'obstacle le plus proche : " << distObstacles[i] << endl;
		resTest2 << "	et coordonnees : " << coordObstacles[i][0] << " "
				<< coordObstacles[i][1] << " " << coordObstacles[i][2] << endl;
	}
	for (int i=0; i<(int)decoupageO.size(); i++)
		cout << "point de coupure : " << decoupageO[i] << endl;	
*/

 	// test 3 : trajectoire du test 2, complexifiee pour avoir divers types de shots
/*
	// generation de la trajectoire (offline)
	ofstream traj("trajectoire_test3.txt");
	int i;
	// pan vertical :
	for (i=0; i<100; i++) 
		traj << 0 << " " << (M_PI_4-i*(M_PI_4/100)) << " " << 0.0 << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=100; i<200; i++) 
		traj << 0 << " " << ((i-100)*(M_PI_4/100)) << " " << 0.0 << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=200; i<300; i++) 
		traj << (3*(i-200)) << " " << M_PI_4 << " " << ((i-200)*M_PI/(5*100)) << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=300; i<350; i++)
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< ((3+0.02*(i-299))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=350; i<450; i++)
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=450; i<500; i++)
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< (M_PI-0.02*(i-449)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=500; i<550; i++)
		traj << i << " " << ((1+0.02*(i-499))*M_PI_4) << " " << (M_PI/5) << " " << 0.0 << " "
		<< ((3+0.02*(i-499))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=550; i<650; i++)
		traj << i << " " << M_PI_2 << " " << (M_PI/5) << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=650; i<700; i++)
		traj << i << " " << (M_PI_2-0.02*(i-649)*M_PI_4) << " " << (M_PI/5) << " " << 0.0 << " "
		<< (M_PI-0.02*(i-649)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=700; i<800; i++) 
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=800; i<850; i++) 
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< ((3+0.02*(i-799))*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=850; i<950; i++)
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< M_PI << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=950; i<1000; i++)
		traj << i << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< (M_PI-0.02*(i-949)*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	// pan horizontal :
	for (i=1000; i<1100; i++) 
		traj << 1000 << " " << M_PI_4 << " " << (M_PI/5-(i-1000)*(M_PI_2/100)) << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=1100; i<1200; i++) 
		traj << 1000 << " " << M_PI_4 << " " << (M_PI/5-M_PI_2+(i-1100)*(M_PI_2/100)) << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	// mouvement roll :
	for (i=1200; i<1300; i++) 
		traj << 1000 << " " << M_PI_4 << " " << (M_PI/5) << " " << ((i-1200)*M_PI_2/100) << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	for (i=1300; i<1400; i++) 
		traj << 1000 << " " << M_PI_4 << " " << (M_PI/5) << " " << (M_PI_2-(i-1300)*M_PI_2/100) << " "
		<< (3*M_PI_4) << " " << 0.0 << " " << 0.0 << " " << 0.0 << endl;
	// mouvement de la main seule :
	for (i=1400; i<1500; i++) 
		traj << 1000 << " " << M_PI_4 << " " << (M_PI/5) << " " << 0.0 << " "
		<< (3*M_PI_4) << " " << (M_PI*cos((double)i)) << " " << (M_PI*sin((double)i)) << " " << (M_PI*cos((double)i)) << endl;
*/

/* Test vitesse PQP vs sampling gaussien :
	analyseObstacles(configurations);
	// inutile de mettre un timer si les temps sont suffisament grands
*/
	
	return 0;
}
