#include "Camera.h"

#define PQP_METHOD

#ifdef PQP_METHOD

// methode avec PQP :

// evalue la qualite d'une image ou on doit voir le bras entier :
double Camera::getArmIn() const {
	// on verifie ensuite si le bras est bien inclus dans le cone de vision de la camera:

	// i) P <-- P + vect(CO) ; C == camera, O == origine et P == point sur le bras.
	double* coordTrans3D=(double*)malloc(9*sizeof(double));
	coordTrans3D[0]=coord3D[0]-posX;
	coordTrans3D[1]=coord3D[1]-posY;
	coordTrans3D[2]=coord3D[2]-posZ;
	coordTrans3D[3]=coord3D[3]-posX;
	coordTrans3D[4]=coord3D[4]-posY;
	coordTrans3D[5]=coord3D[5]-posZ;
	coordTrans3D[6]=coord3D[6]-posX;
	coordTrans3D[7]=coord3D[7]-posY;
	coordTrans3D[8]=coord3D[8]-posZ;

	// ii) calcul de la matrice de changement de base "reperes camera / origine"
	double *coeffsX=(double*)malloc(3*sizeof(double)),
			*coeffsY=(double*)malloc(3*sizeof(double)),
			*coeffsZ=(double*)malloc(3*sizeof(double));
	getPassMatrix(coeffsX,coeffsY,coeffsZ,anglePlan,angleVert);

	// on inverse la matrice M obtenue facilement en prenant la transposee : tM;
	// il ne reste plus qu'a calculer tM*coordonnees de chaque point a tester.

	// i) calcul tM*pointEpaule :
	double newX,newY,newZ;
	newX=coeffsX[0]*coordTrans3D[0]+coeffsX[1]*coordTrans3D[1]+coeffsX[2]*coordTrans3D[2];
	newY=coeffsY[0]*coordTrans3D[0]+coeffsY[1]*coordTrans3D[1]+coeffsY[2]*coordTrans3D[2];
	newZ=coeffsZ[0]*coordTrans3D[0]+coeffsZ[1]*coordTrans3D[1]+coeffsZ[2]*coordTrans3D[2];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		// on veut voir (au moins en thorie et dans un premier temps sans optimisations) tout le bras:
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}
	// ii) calculs pour les autres points :
	newX=coeffsX[0]*coordTrans3D[3]+coeffsX[1]*coordTrans3D[4]+coeffsX[2]*coordTrans3D[5];
	newY=coeffsY[0]*coordTrans3D[3]+coeffsY[1]*coordTrans3D[4]+coeffsY[2]*coordTrans3D[5];
	newZ=coeffsZ[0]*coordTrans3D[3]+coeffsZ[1]*coordTrans3D[4]+coeffsZ[2]*coordTrans3D[5];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}
	newX=coeffsX[0]*coordTrans3D[6]+coeffsX[1]*coordTrans3D[7]+coeffsX[2]*coordTrans3D[8];
	newY=coeffsY[0]*coordTrans3D[6]+coeffsY[1]*coordTrans3D[7]+coeffsY[2]*coordTrans3D[8];
	newZ=coeffsZ[0]*coordTrans3D[6]+coeffsZ[1]*coordTrans3D[7]+coeffsZ[2]*coordTrans3D[8];
	if (newX*newX+newY*newY>newZ*newZ*TAN_2_ANGLE) {
		free(coeffsX);
		free(coeffsY);
		free(coeffsZ);
		free(coordTrans3D);
		return 0.0;
	}

	free(coeffsX);
	free(coeffsY);
	free(coeffsZ);
	free(coordTrans3D);

	// enfin, derniere verification avant algorithme principal : on regarde si le bras
	// n'est pas obstrue par lui-meme, en verifiant que les triplets (joint 1, joint 2, camera)
	// ne soient pas colineaires, quels que soient les joints selectionnes
	double* paramCam=(double*)malloc(3*sizeof(double));
	paramCam[0]=posX;
	paramCam[1]=posY;
	paramCam[2]=posZ;
	if (colineaires(coord3D,coord3D+3,paramCam)
			|| colineaires(coord3D,coord3D+6,paramCam)
			|| colineaires(coord3D+3,coord3D+6,paramCam)) {
		free(paramCam);
		return 0.0; // on ne veut pas de telle vue !
	}
	free(paramCam);
	
	// ensuite on echantillonne point par point sur le bras, enregistrant le pourcentage de collisions :
	// E_PTS points entre epaule et coude,
	// C_PTS points entre coude et poignet
	// P_PTS points sur la sphere du poignet
	double* pointEp=(double*)malloc(3*E_PTS*sizeof(double));
	double* pointCo=(double*)malloc(3*C_PTS*sizeof(double));
	double* pointPo=(double*)malloc(3*P_PTS*sizeof(double));

	/*************************************
	*** ajout d'une perturbation radiale :
	*************************************/

	// on calcule deux vecteurs orthogonaux dans le plan de coupe du cylindre epaule-coude,

	double* vect1=(double*)malloc(3*sizeof(double));
	double* vect2=(double*)malloc(3*sizeof(double));
	double* diffVect=(double*)malloc(3*sizeof(double));
	diffVect[0]=coord3D[3]-coord3D[0];
	diffVect[1]=coord3D[4]-coord3D[1];
	diffVect[2]=coord3D[5]-coord3D[2];

	double phi,theta;
	// transformation en coordonnees spheriques pour obtenir phi et theta :
	getSphericAngles(diffVect,phi,theta);

	double tmpSinP=sin(phi),tmpCosP=cos(phi),tmpCosV=cos(theta);
	vect1[0]=tmpSinP;
	vect1[1]=-tmpCosP;
	vect1[2]=0.0;
	vect2[0]=-tmpCosP*tmpCosV;
	vect2[1]=-tmpSinP*tmpCosV;
	vect2[2]=sin(theta);
	// on normalise ensuite les vecteurs se basant sur RAY_C (rayon d'un cylindre de bras)
	for (int i=2; i>=0; i--) {
		vect1[i]*=RAY_C;
		vect2[i]*=RAY_C;
	}
	
	// puis on echantillonne :
	double alpha;
	int rdB1,rdB2;
	for (int i=E_PTS-1; i>=0; i--) {
		alpha=genAlea();
		pointEp[3*i]=alpha*coord3D[0]+(1-alpha)*coord3D[3];
		pointEp[3*i+1]=alpha*coord3D[1]+(1-alpha)*coord3D[4];
		pointEp[3*i+2]=alpha*coord3D[2]+(1-alpha)*coord3D[5];
		alpha=genAlea();
		rdB1=randBit();
		rdB2=randBit();
		pointEp[3*i]+=rdB1*alpha*vect1[0]+rdB2*(1-alpha)*vect2[0];
		pointEp[3*i+1]+=rdB1*alpha*vect1[1]+rdB2*(1-alpha)*vect2[1];
		pointEp[3*i+2]+=rdB1*alpha*vect1[2]+rdB2*(1-alpha)*vect2[2];
	}

	/***************************************************
	***	meme travail pour la partie du coude au poignet :
	***************************************************/

	diffVect[0]=coord3D[6]-coord3D[3];
	diffVect[1]=coord3D[7]-coord3D[4];
	diffVect[2]=coord3D[8]-coord3D[5];

	// transformation en coordonnees spheriques pour obtenir phi et theta :
	getSphericAngles(diffVect,phi,theta);
	free(diffVect);

	tmpSinP=sin(phi);
	tmpCosP=cos(phi);
	tmpCosV=cos(theta);
	vect1[0]=tmpSinP;
	vect1[1]=-tmpCosP;
	vect1[2]=0.0;
	vect2[0]=-tmpCosP*tmpCosV;
	vect2[1]=-tmpSinP*tmpCosV;
	vect2[2]=sin(theta);
	// on normalise ensuite les vecteurs se basant sur RAY_C (rayon d'un cylindre de bras)
	for (int i=2; i>=0; i--) {
		vect1[i]*=RAY_C;
		vect2[i]*=RAY_C;
	}
	
	// puis on echantillonne :
	for (int i=C_PTS-1; i>=0; i--) {
		alpha=genAlea();
		pointCo[3*i]=alpha*coord3D[3]+(1-alpha)*coord3D[6];
		pointCo[3*i+1]=alpha*coord3D[4]+(1-alpha)*coord3D[7];
		pointCo[3*i+2]=alpha*coord3D[5]+(1-alpha)*coord3D[8];
		alpha=genAlea();
		rdB1=randBit();
		rdB2=randBit();
		pointCo[3*i]+=rdB1*alpha*vect1[0]+rdB2*(1-alpha)*vect2[0];
		pointCo[3*i+1]+=rdB1*alpha*vect1[1]+rdB2*(1-alpha)*vect2[1];
		pointCo[3*i+2]+=rdB1*alpha*vect1[2]+rdB2*(1-alpha)*vect2[2];
	}
	free(vect1);
	free(vect2);

	/*********************************
	**** puis enfin pour le poignet :
	*********************************/

	// on connait le centre ainsi que le rayon :
	// on utilise donc une loi uniforme dans la sphere.
	double tmpSinT;
	for (int i=P_PTS-1; i>=0; i--) {
		// tirage aleatoire de l'angle phi puis theta :
		phi=2*M_PI*genAlea();
		theta=2*M_PI*genAlea();
		
		// calcul des coordonnees x,y,z en fonction des coordonnees spheriques r,phi,theta:
		tmpSinT=sin(theta);
		pointPo[i*3]=coord3D[6]+RAY_P*cos(phi)*tmpSinT;
		pointPo[i*3+1]=coord3D[7]+RAY_P*sin(phi)*tmpSinT;
		pointPo[i*3+2]=coord3D[8]+RAY_P*cos(theta);
	}

	// on arrive au moment ou l'on doit tester les obstructions le long de chaque segment :
	int countObstE=0,countObstC=0,countObstP=0; // compteurs d'obstruction

	// variables utiles pr PQP_Collide :
	PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}}; // tranformation identite
	PQP_CollideResult* result=new PQP_CollideResult();

	// points epaule-coude :
	for (int i=E_PTS-1; i>=0; i--) {
		// UTILISE PQP (comme dans Functions::collisionBen) :
		double p[][3]={{pointEp[0],pointEp[1],pointEp[2]},{pointEp[0]+1e-8,pointEp[1],pointEp[2]},{posX,posY,posZ}};
		PQP_Model* pointColl=new PQP_Model();
		pointColl->BeginModel();
		pointColl->AddTri(p[0],p[1],p[2],0);
		pointColl->EndModel();
		PQP_REAL position[3]={(posX+pointEp[0])/2.0,(posY+pointEp[1])/2.0,(posZ+pointEp[2])/2.0}; // la position du triangle

		// boucle sur tous les obstacles :
		for (int i=obstaclesBen->num_obstacles()-1; i>=0; i--) {
			PQP_Collide(result,rotateId,position,pointColl,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i]);
			if (result->Colliding()) {
				countObstE++;
				break;
			}
		}

		delete pointColl;
	}
	// points coude-poignet :
	for (int i=C_PTS-1; i>=0; i--) {
		// UTILISE PQP (comme dans Functions::collisionBen) :
		double p[][3]={{pointCo[0],pointCo[1],pointCo[2]},{pointCo[0]+1e-8,pointCo[1],pointCo[2]},{posX,posY,posZ}};
		PQP_Model* pointColl=new PQP_Model();
		pointColl->BeginModel();
		pointColl->AddTri(p[0],p[1],p[2],0);
		pointColl->EndModel();
		PQP_REAL position[3]={(posX+pointCo[0])/2.0,(posY+pointCo[1])/2.0,(posZ+pointCo[2])/2.0}; // la position du triangle

		// boucle sur tous les obstacles :
		for (int i=obstaclesBen->num_obstacles()-1; i>=0; i--) {
			PQP_Collide(result,rotateId,position,pointColl,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i]);
			if (result->Colliding()) {
				countObstC++;
				break;
			}
		}

		delete pointColl;
	}
	// points main (poignet) :
	for (int i=P_PTS-1; i>=0; i--) {
		// UTILISE PQP (comme dans Functions::collisionBen) :
		double p[][3]={{pointPo[0],pointPo[1],pointPo[2]},{pointPo[0]+1e-8,pointPo[1],pointPo[2]},{posX,posY,posZ}};
		PQP_Model* pointColl=new PQP_Model();
		pointColl->BeginModel();
		pointColl->AddTri(p[0],p[1],p[2],0);
		pointColl->EndModel();
		PQP_REAL position[3]={(posX+pointPo[0])/2.0,(posY+pointPo[1])/2.0,(posZ+pointPo[2])/2.0}; // la position du triangle

		// boucle sur tous les obstacles :
		for (int i=obstaclesBen->num_obstacles()-1; i>=0; i--) {
			PQP_Collide(result,rotateId,position,pointColl,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i]);
			if (result->Colliding()) {
				countObstP++;
				break;
			}
		}

		delete pointColl;
	}

	// liberations memoire
	free(pointEp);
	free(pointCo);
	free(pointPo);
	delete result;

	// resultat : ratio non_obstrue / total
	return 1.0-(double)(countObstE+countObstC+countObstP)/(E_PTS+C_PTS+P_PTS);
}

// meme boulot pour un unique point (bcp plus simple,
// cas de la main ou du filmage avec zoom si obstacle proche) :
double Camera::getPointIn() const {
	// echantillonnage au voisinage du point :
	// on utilise une loi uniforme dans la sphere, avec P_PTS points et de rayon RAY_P si main,
	// avec floor(20*distObstacles[indiceG]) points sinon, pour un rayon de distObstacles[indiceG]
	double tmpSinT,rayon=0.0;
	int nbPoints=0;
	if (handPoints[indiceG]) {
		nbPoints=P_PTS;
		rayon=RAY_P;
	}
	else {
		nbPoints=(int)floor(20*distObstacles[indiceG]);
		rayon=distObstacles[indiceG];
	}
	double* pointsEch=(double*)malloc(nbPoints*3*sizeof(double));

	double phi,theta;
	for (int i=nbPoints-1; i>=0; i--) {
		// tirage aleatoire de l'angle phi puis theta :
		phi=2*M_PI*genAlea();
		theta=2*M_PI*genAlea();
		
		// calcul des coordonnees x,y,z en fonction des coordonnees spheriques r,phi,theta:
		tmpSinT=sin(theta);
		pointsEch[i*3]=currPR[0]+rayon*cos(phi)*tmpSinT;
		pointsEch[i*3+1]=currPR[1]+rayon*sin(phi)*tmpSinT;
		pointsEch[i*3+2]=currPR[2]+rayon*cos(theta);
	}
	
	// recherche de collisions :
	int countObst=0;

	// variables utiles pr PQP_Collide :
	PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}}; // tranformation identite
	PQP_CollideResult* result=new PQP_CollideResult();

	for (int i=nbPoints-1; i>=0; i--) {
		// UTILISE PQP (comme dans Functions::collisionBen) :
		double p[][3]={{pointsEch[0],pointsEch[1],pointsEch[2]},{pointsEch[0]+1e-8,pointsEch[1],pointsEch[2]},{posX,posY,posZ}};
		PQP_Model* pointColl=new PQP_Model();
		pointColl->BeginModel();
		pointColl->AddTri(p[0],p[1],p[2],0);
		pointColl->EndModel();
		PQP_REAL position[3]={(posX+pointsEch[0])/2.0,(posY+pointsEch[1])/2.0,(posZ+pointsEch[2])/2.0}; // la position du triangle

		// boucle sur tous les obstacles :
		for (int i=obstaclesBen->num_obstacles()-1; i>=0; i--) {
			PQP_Collide(result,rotateId,position,pointColl,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i]);
			if (result->Colliding()) {
				countObst++;
				break;
			}
		}

		delete pointColl;
	}
	free(pointsEch);
	delete result;
	
	return 1-(double)countObst/nbPoints;
}

#else

// methode plus rapide via les buffers d'image :

// evalue la qualite d'une image ou on doit voir le bras entier :
double Camera::getArmIn() const {
	// positionnement des deux cameras, avec et sans les obstacles :
	mpkGUI::rootCamera2->position=SbVec3f(posX,posY,posZ);
	mpkGUI::rootCamera2->pointAt(SbVec3f(posX+distCamPR*cos(anglePlan)*sin(angleVert),
										posY+distCamPR*sin(anglePlan)*sin(angleVert),
										posZ+distCamPR*cos(angleVert)),
								SbVec3f(0,-1,0));

	mpkGUI::rootCamera2Ben->position=SbVec3f(posX,posY,posZ);
	mpkGUI::rootCamera2Ben->pointAt(SbVec3f(posX+distCamPR*cos(anglePlan)*sin(angleVert),
										posY+distCamPR*sin(anglePlan)*sin(angleVert),
										posZ+distCamPR*cos(angleVert)),
								SbVec3f(0,-1,0));

	return visibilityCheckBen(true); // appel avec true si l'on filme le bras entier
}

// evalue la qualite d'une image ou on doit voir le bras entier :
double Camera::getPointIn() const {
	// positionnement des deux cameras, avec et sans les obstacles :
	mpkGUI::rootCamera2->position=SbVec3f(posX,posY,posZ);
	mpkGUI::rootCamera2->pointAt(SbVec3f(posX+distCamPR*cos(anglePlan)*sin(angleVert),
										posY+distCamPR*sin(anglePlan)*sin(angleVert),
										posZ+distCamPR*cos(angleVert)),
								SbVec3f(0,-1,0));

	mpkGUI::rootCamera2Ben->position=SbVec3f(posX,posY,posZ);
	mpkGUI::rootCamera2Ben->pointAt(SbVec3f(posX+distCamPR*cos(anglePlan)*sin(angleVert),
										posY+distCamPR*sin(anglePlan)*sin(angleVert),
										posZ+distCamPR*cos(angleVert)),
								SbVec3f(0,-1,0));

	return visibilityCheckBen(false); // appel avec false si l'on zoome sur un point (currPR)
}

#endif

// fonction un niveau au dessus, qui met a jour les angles (sauf pr la main) 
// avant d'appeler une des fonctions precedente :
double Camera::getVisibilite() {
	// on regarde simplement si la camera est en collision :
	double params[]={posX,posY,posZ};
	if (collisionBen(params)) return 0.0;

	// mise a jour des angles :
	double vecteur[]={currPR[0]-posX,currPR[1]-posY,currPR[2]-posZ};
	getSphericAngles(vecteur,anglePlan,angleVert);

	if (handPoints[indiceG] || zoomObstPoints[indiceG]) return getPointIn();

	return getArmIn();
}
