#include "CompilIdioms.h"

#include <iostream>

using namespace std;

// compile une suite d'idiomes en placements de cameras :
vector<ConfigCamera>* compilation(vector<Idiom>& idiomes,const vector<mpkConfig>& configurations) {

	// deux cas : compilation d'un idiome simple (un seul shot),
	// et compilation d'un idiome compose.
	
	// second cas nettement plus delicat car on doit regler les instants de changement de shots.
	// on n'implemente que le premier. Notons tout de meme que le reglage des timeStamps est la seule difficulte,
	// la compilation s'effectuant sur la base des shots seuls.
	
	// IMPORTANT :
	// on initialise zoomObstPoints a false (on ne s'en sert pas dans le cas des idiomes,
	// mais cela pourrait interferer) :
	for (int i=configurations.size()-1; i>=0; i--) zoomObstPoints.push_back(false);
	
	// recuperation/evaluation des angles de vues pr le film :
	for (int i=0; i<(int)idiomes.size(); i++)
		evalViews(idiomes[i],configurations);
	
	PlaceCamera placeCam=PlaceCamera(); // objet utile pr creer et placer les 3 cameras
	ConfigCamera configCam1=ConfigCamera(),configCam0=ConfigCamera(),configCam2=ConfigCamera();
	vector<ConfigCamera>* resConfigs=new vector<ConfigCamera>[3]; // vecteur des configurations resultat
	double* vecteur=(double*)malloc(3*sizeof(double));
	double* initPR=(double*)malloc(3*sizeof(double));

	// placement des cameras selon ces vues :
	for (int i=0; i<(int)idiomes.size(); i++) { 

// DEBUG :
cout << idiomes[i].name <<"  " << idiomes[i].shots[0].firstInd << " " << idiomes[i].shots.back().lastInd << endl;
	
		// premier cas simpliste : on n'a pas su dire quel type d'idiome c'est,
		// donc on filme comme en resolution de contraintes, image par image,
		// sans oublier de lisser le resultat obtenu :
		if (strcmp(idiomes[i].name,"unknown")==0) { // on espere que ce cas n'arrive pas..

			vector<ConfigCamera> configsCam1,configsCam0,configsCam2;
			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots.back().lastInd; j++) {
				cout << "iterCam " << j << endl;

				indiceG=j;
				coord3D=conf2coord3D(configurations[j]);
	
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
	
				// calcul du nouveau point de repere :
				getRepPoint(j);

				// calcul de la distance de vue :
				getViewDistance(j);
		
				for (int k=2; k>=0; k--) depctRepPoint[k]=currPR[k]-lastRepPoint[k];

				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam1.push_back(placeCam.cameras[1].getConfig());
			
				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam0.push_back(placeCam.cameras[0].getConfig());
	
				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam2.push_back(placeCam.cameras[2].getConfig());
			}

			// lissage :
			vector<mpkConfig> configs;
			for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots[0].lastInd; j++)
				configs.push_back(configurations[j]);
			configsCam1=lisse(configsCam1,configs);
			configsCam0=lisse(configsCam0,configs);
			configsCam2=lisse(configsCam2,configs);
			
			// ajout au resultat :
			for (int j=0; j<=idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd; j++) {
				resConfigs[1].push_back(configsCam1[j]);
				resConfigs[0].push_back(configsCam0[j]);
				resConfigs[2].push_back(configsCam2[j]);
			}
			
			free(lastRepPoint);
			free(depctRepPoint);
			continue;
		}

		/********************
		* Initialisations :
		********************/

		// determination du point de repere initial :
		coord3D=conf2coord3D(configurations[idiomes[i].shots[0].firstInd]);
		getRepPoint(idiomes[i].shots[0].firstInd);
		
		// calcul de la distance de vue : maximum des distances de vues pr les images de l'idiome !
		double distance=0.0;
		for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots.back().lastInd; j++) {
			getViewDistance(j);
			distance+=distCamPR;
		}
		distance/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);

		// ********** CAMERA 1 (centrale) : ****************
		if (idiomes[i].views[0]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(1);
		}
		else {
			putCamInView(placeCam.cameras[1],idiomes[i].views[0]);
		}

		// ********** CAMERA 0 (gauche) : *****************
		if (idiomes[i].views[1]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(0);
		}
		else {
			putCamInView(placeCam.cameras[0],idiomes[i].views[1]);
		}
		
		// *********** CAMERA 2 (droite) : *****************
		if (idiomes[i].views[2]<0) {
			// alors vue initiale aleatoire : (donnerait surement de mauvais resultats, esperons que ca n'arrive pas)
			placeCam.getAleaView(2);
		}
		else {
			putCamInView(placeCam.cameras[2],idiomes[i].views[2]);
		}

		// calcul des angles initiaux de chaque camera,
		// et initialisations des configurations :
		
		// ********** CAMERA 1 (centrale) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[1].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[1].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[1].posZ;
		getSphericAngles(vecteur,placeCam.cameras[1].anglePlan,placeCam.cameras[1].angleVert);
		placeCam.cameras[1].roll=0.0; // a calculer ulterieurement
		placeCam.cameras[1].zoom=1.0; // toujours 1 dans cette section
		configCam1=placeCam.cameras[1].getConfig();

		// ********** CAMERA 0 (gauche) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[0].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[0].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[0].posZ;
		getSphericAngles(vecteur,placeCam.cameras[0].anglePlan,placeCam.cameras[0].angleVert);
		placeCam.cameras[0].roll=0.0;
		placeCam.cameras[0].zoom=1.0;
		configCam0=placeCam.cameras[0].getConfig();
		
		// ********** CAMERA 2 (droite) : ****************
		vecteur[0]=currPR[0]-placeCam.cameras[2].posX;
		vecteur[1]=currPR[1]-placeCam.cameras[2].posY;
		vecteur[2]=currPR[2]-placeCam.cameras[2].posZ;
		getSphericAngles(vecteur,placeCam.cameras[2].anglePlan,placeCam.cameras[2].angleVert);
		placeCam.cameras[2].roll=0.0;
		placeCam.cameras[2].zoom=1.0;
		configCam2=placeCam.cameras[2].getConfig();

		// sauvegarde de la valeur du point de repere courant (==initial),
		for (int j=2; j>=0; j--) initPR[j]=currPR[j];
		// calcul du point de repere dans la configuration finale sur cet idiome :
		coord3D=conf2coord3D(configurations[idiomes[i].shots.back().lastInd]);
		getRepPoint(idiomes[i].shots.back().lastInd);
		
		// preliminaire : calcul des pas angulaires (ou de translation) elementaires :
		// tout depend de la position relative de la camera par rapport au bras;
		// et aussi des types d'idiomes.
		double pasX1=0.0,pasY1=0.0,pasZ1=0.0,pasPhi1=0.0,pasTheta1=0.0,
				pasX0=0.0,pasY0=0.0,pasZ0=0.0,pasPhi0=0.0,pasTheta0=0.0,
				pasX2=0.0,pasY2=0.0,pasZ2=0.0,pasPhi2=0.0,pasTheta2=0.0;
		// si static : pas de pas == rien a faire.

		if (strcmp(idiomes[i].name,"panHori")==0) {
			// seul pasPhi va etre affecte :

			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine grossierement la position de la camera pour connaitre le signe :
			int sgn;
			if (configCam1.posX<(initPR[0]+currPR[0])/2.0) {
				if (currPR[2]-initPR[2]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[2]-initPR[2]<0) sgn=-1;
				else sgn=1;
			}
	
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasPhi1=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/

			// on determine grossierement la position de la camera pour connaitre le signe :
			if (configCam0.posX<(initPR[0]+currPR[0])/2.0) {
				if (currPR[2]-initPR[2]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[2]-initPR[2]<0) sgn=-1;
				else sgn=1;
			}
			
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasPhi0=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi0/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 2 : droite ************
			******************************************/
			
			// on determine grossierement la position de la camera pour connaitre le signe :
			if (configCam2.posX<(initPR[0]+currPR[0])/2.0) {
				if (currPR[2]-initPR[2]<0)
					// deplacement "de la droite vers la gauche", donc etant donne le
					// demi-espace dans lequel se situe la camera, signe positif.
					sgn=1;
				else sgn=-1;
			}
			else {
				// le contraire :
				if (currPR[2]-initPR[2]<0) sgn=-1;
				else sgn=1;
			}
			
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasPhi2=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasPhi2/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
		}
		
		else if (strcmp(idiomes[i].name,"panVert")==0) {
			// seul pasTheta va etre affecte : meme boulot

			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine le signe (le meme pr les 3 cameras !) :
			int sgn;
			if (currPR[1]-initPR[1]<0)
				// deplacement "du haut vers le bas", donc signe positif.
				sgn=1;
			else sgn=-1;
			
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasTheta1=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/
			
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasTheta0=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta0/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 2 : droite ************
			******************************************/
			
			// calcul de l'angle global en fonction des 3 cotes |currPR[1]-initPR[1]|, distCamPR, distCamPR :
			pasTheta2=sgn*2.0*asin(fabs(currPR[1]-initPR[1])/(2.0*distCamPR));
			// on normalise pour obtenir le pas :
			pasTheta2/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
		}
		
		else if (strcmp(idiomes[i].name,"track")==0) {
			// seul pasX,Y,Z va etre affecte : bcp plus facile car pas d'angles a prendre en compte.
			
			/******************************************
			// ********* camera 1 : centrale ************
			******************************************/

			// on determine le signe :
			int sgn;
			if (initPR[2]-currPR[2]<0)
				// deplacement "de la gauche vers la droite", donc signe positif.
				sgn=1;
			else sgn=-1;
			
			// coefficients obtenus par somme des facteurs au carres pour normalisation, voir View.cpp
			pasX1=0.267394*sgn*fabs(currPR[0]-initPR[0]);
			pasY1=0.126899*sgn*fabs(currPR[1]-initPR[1]);
			pasZ1=0.955195*sgn*fabs(currPR[2]-initPR[2]);
			// on normalise pour obtenir le pas :
			pasX1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			pasY1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			pasZ1/=(idiomes[i].shots.back().lastInd-idiomes[i].shots[0].firstInd+1);
			
			/******************************************
			// ********* camera 0 : gauche ************
			******************************************/
			
			pasX0=pasX1;
			pasY0=pasY1;
			pasZ0=pasZ1; // logique..
		
			/******************************************
			// ********* camera 1 : droite ************
			******************************************/
			
			pasX2=pasX1;
			pasY2=pasY1;
			pasZ2=pasZ1;
		}

		/***********************
		* Chemin des cameras :
		***********************/

		for (int j=idiomes[i].shots[0].firstInd; j<=idiomes[i].shots.back().lastInd; j++) {
			resConfigs[1].push_back(configCam1);
			resConfigs[0].push_back(configCam0);
			resConfigs[2].push_back(configCam2);
	
			placeCam.cameras[1].posX+=pasX1;
			placeCam.cameras[1].posY+=pasY1;
			placeCam.cameras[1].posZ+=pasZ1;
			placeCam.cameras[1].anglePlan+=pasPhi1;
			placeCam.cameras[1].angleVert+=pasTheta1;
			configCam1=placeCam.cameras[1].getConfig();
			
			placeCam.cameras[0].posX+=pasX0;
			placeCam.cameras[0].posY+=pasY0;
			placeCam.cameras[0].posZ+=pasZ0;
			placeCam.cameras[0].anglePlan+=pasPhi0;
			placeCam.cameras[0].angleVert+=pasTheta0;
			configCam0=placeCam.cameras[0].getConfig();
			
			placeCam.cameras[2].posX+=pasX2;
			placeCam.cameras[2].posY+=pasY2;
			placeCam.cameras[2].posZ+=pasZ2;
			placeCam.cameras[2].anglePlan+=pasPhi2;
			placeCam.cameras[2].angleVert+=pasTheta2;
			configCam2=placeCam.cameras[2].getConfig();
		}
	}

	free(vecteur);
	free(initPR);

	return resConfigs;
}
