#include "DecoupMonotone.h"

// on marque tous les points terminaux de segments sur lesquels main bouge, ET pas le reste.
// ensuite on analyse ces points, pour sortir des segments sur lesquels on devra
// filmer en zoomant sur la main seule.
// ensuite, au cours du filmage : on ajoute un switch aux fonctions adjustCamPos 
// (qui ne fait que mettre  jour les angles dans le cas de la main seule),
// initCamPos : pas de changements (inutile), mais switch dans getArmIn() (attention !!)
// randomSelectOnSphere : pareil (appele qu'en cas extreme de ttes faons)
// moveCamera : pareil (elle ne bouge alors pas.. possibilite de ne pas appeler cette fonction a ce stade donc)
void analyseMonotone(const vector<mpkConfig>& configurations) {
	vector<bool> hPoints(configurations.size());

	// analyse des variations le long de chaque dimension:
	// on decoupe de maniere a ce que les variations soient uniformes
	// sur chaque segment; ensuite, on pourra eventuellement recoller les segments
	// variations partout sauf main, pr des raisons de filmage..
	vector<int> decoupageM_tmp; // stockage temporaire des points de coupures
	int variations[DIM];
	int nbPoints=configurations.size(),i,j;
	// variations initialisees avec les deux premiers points:
	for (i=DIM-1; i>=0; i--) {
		if (fabs(configurations[1][i]-configurations[0][i])<SEUIL_JOINT) variations[i]=0;
		else if (configurations[1][i]-configurations[0][i]>=SEUIL_JOINT) variations[i]=1;
		else variations[i]=-1;
	}
	
	hPoints[0]=false;
	hPoints[1]=true;
	for (j=0; j<5; j++) {
		if (variations[j]!=0) {
			hPoints[1]=false;
			break;
		}
	}
	if (hPoints[1]) {
		for (j=5; j<8; j++)
			if (variations[j]!=0) break;
		if (j>=8) hPoints[1]=false;
	}
	
	i=2;
	while (i<nbPoints) {
		for (j=0; j<DIM; j++) {
			if ((fabs(configurations[i][j]-configurations[i-1][j])<SEUIL_JOINT && variations[j]!=0)
				|| (configurations[i][j]-configurations[i-1][j]>=SEUIL_JOINT && variations[j]!=1)
				|| (configurations[i][j]-configurations[i-1][j]<=-SEUIL_JOINT && variations[j]!=-1)) break;
		}
		if (j<DIM) {
			// on recalcule les variations et ajoute un point de coupure:
			for (int k=DIM-1; k>=0; k--) {
				if (fabs(configurations[i][k]-configurations[i-1][k])<SEUIL_JOINT) variations[k]=0;
				else if (configurations[i][k]-configurations[i-1][k]>=SEUIL_JOINT) variations[k]=1;
				else variations[k]=-1;
			}
			// seulement si coupure pour autre joint que le poignet :
			if (j<DIM-3) decoupageM_tmp.push_back(i);
		}

		// mise a jour de hPoints
		hPoints[i]=true;
		for (j=0; j<5; j++) {
			if (variations[j]!=0) {
				hPoints[i]=false;
				break;
			}
		}
		if (hPoints[i]) {
			for (j=5; j<DIM; j++)
				if (variations[j]!=0) break;
			if (j>=DIM) hPoints[i]=false;
		}
		
		// passage au point suivant :
		i++;
	}

	// si aucun point de coupure : on sort tout de suite
	if (decoupageM_tmp.empty()) {
		// appel de l'analyseur des segments de film avec zoom sur main :
		handPoints=analysePoints_Densite(hPoints);
		return;
	}

	// une fois le decoupage effectue, on regarde si on ne peut pas fusionner des segments,
	// dans le cas ou la trajectoire serait legerement bruitee:
	// On regarde si certaines periodes uniformes (==segments) sont trop courts
	// relativement aux autres segments. Si oui, on les fusionne avec le segment
	// de nature le plus proche, adjacent.
	// a) calcul de la moyenne des ecarts entre coupures:
	double mean=(double)nbPoints/(decoupageM_tmp.size()+1);
	// b) comparaison de cette moyenne a chaque ecart, qui est alors juge trop petit eventuellement:
	vector<int> toDelete; // points de coupure a supprimer dans "decoupageM_tmp"
	if ((nbPoints-decoupageM_tmp[decoupageM_tmp.size()-1])/mean<EPSILON_BEN) toDelete.push_back(decoupageM_tmp.size()-1);
	for (i=decoupageM_tmp.size()-1; i>=1; i--) {
		if ((decoupageM_tmp[i]-decoupageM_tmp[i-1])/mean<EPSILON_BEN) {
			// ici, il faut fusionner avec le segment aux caracteristiques les plus proches,
			// a gauche ou a droite; en cas d'ex-aequo, choix aleatoire
			int similG=0;
			if (i>1) {
				for (j=DIM-1; j>=0; j--) {
					if ((fabs(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])
							< (decoupageM_tmp[i]-decoupageM_tmp[i-1])*SEUIL_JOINT
						&& fabs(configurations[decoupageM_tmp[i-1]][j]-configurations[decoupageM_tmp[i-2]][j])
							< (decoupageM_tmp[i-1]-decoupageM_tmp[i-2])*SEUIL_JOINT)
						|| (configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])
							*(configurations[decoupageM_tmp[i-1]][j]-configurations[decoupageM_tmp[i-2]][j])>0) similG++;
				}
			}
			else {
				for (j=DIM-1; j>=0; j--) {
					if ((fabs(configurations[decoupageM_tmp[1]][j]-configurations[decoupageM_tmp[0]][j])
							< (decoupageM_tmp[1]-decoupageM_tmp[0])*SEUIL_JOINT
						&& fabs(configurations[decoupageM_tmp[0]][j]-configurations[0][j])
							< decoupageM_tmp[0]*SEUIL_JOINT)
						|| (configurations[decoupageM_tmp[1]][j]-configurations[decoupageM_tmp[0]][j])
							*(configurations[decoupageM_tmp[0]][j]-configurations[0][j])>0) similG++;
				}
			}
			// Important: il faut regarder si le segment a droite a deja ete fusionne, 
			// pour eviter de fusionner sur la base d'un segment trop court et bruite.
			int similD=0;
			if (i<(int)decoupageM_tmp.size()-1) {
				for (j=DIM-1; j>=0; j--) {
					int k=i+1;
					while (k<(int)decoupageM_tmp.size() && (decoupageM_tmp[k]-decoupageM_tmp[k-1])/mean<EPSILON_BEN) k++;
					if (k<(int)decoupageM_tmp.size()) {
						if ((fabs(configurations[decoupageM_tmp[k]][j]-configurations[decoupageM_tmp[i]][j])
								< (decoupageM_tmp[k]-decoupageM_tmp[i])*SEUIL_JOINT
							&& fabs(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])
								< (decoupageM_tmp[i]-decoupageM_tmp[i-1])*SEUIL_JOINT)
							|| (configurations[decoupageM_tmp[k]][j]-configurations[decoupageM_tmp[i]][j])
								*(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])>0) similD++;
					}
					else {
						if ((fabs(configurations[nbPoints-1][j]-configurations[decoupageM_tmp[i]][j])<SEUIL_JOINT
							&& fabs(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])<SEUIL_JOINT)
							|| (configurations[nbPoints-1][j]-configurations[decoupageM_tmp[i]][j])
								*(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])>0) similD++;
					}
				}
			}
			else {
				for (j=DIM-1; j>=0; j--) {
					if ((fabs(configurations[nbPoints-1][j]-configurations[decoupageM_tmp[i]][j])
							< (nbPoints-1-decoupageM_tmp[i])*SEUIL_JOINT
						&& fabs(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])
							< (decoupageM_tmp[i]-decoupageM_tmp[i-1])*SEUIL_JOINT)
						|| (configurations[nbPoints-1][j]-configurations[decoupageM_tmp[i]][j])
							*(configurations[decoupageM_tmp[i]][j]-configurations[decoupageM_tmp[i-1]][j])>0) similD++;
				}
			}
			if (similG<similD) toDelete.push_back(i);
			else if (similG>similD) toDelete.push_back(i-1);
			else toDelete.push_back(i-(randBit()+1)/2);
		}
	}

	// finalement, suppression des points superflus :
	if (!toDelete.empty()) {
		i=0;
		j=0;
		while (i<(int)decoupageM_tmp.size()) {
			while (i<toDelete[toDelete.size()-j-1]) {
				decoupageM.push_back(decoupageM_tmp[i]);
				i++;
			}
			i++;
			j++;
			if (j==(int)toDelete.size()) { // cas ou il n'y a plus de points marques pr suppression
				for (; i<(int)decoupageM_tmp.size(); i++) decoupageM.push_back(decoupageM_tmp[i]);
				break;
			}
		}
	}
	else decoupageM=decoupageM_tmp;

	// appel de l'analyseur des segments de film avec zoom sur main :
	handPoints=analysePoints_Densite(hPoints);
	
	return;
}

// version avec trajectoire inconnue :
void analyseMonotoneImp(const mpkConfig& configC) {
	static bool nothingRead=true;
	static double* lastPoint=(double*)malloc(DIM*sizeof(double));
	int i;
	if (nothingRead) {
		for (i=DIM-1; i>=0; i--) lastPoint[i]=configC[i];
		nothingRead=false;
		handPoints.push_back(false); // vital pr pas planter..
		return;
	}
	
	int j;
	// on sait qu'on a lu au moins un point;
	// sens de deplacement 2==positif, 3==negatif, 1==neutre
	static int* sensDepct=(int*)calloc(DIM,sizeof(int));
	if (sensDepct[0]==0) {
		// tous premiers deplacements du bras :
		// on se contente de mettre a jour les valeurs SANS ajouter un pt de coupure.
		for (i=DIM-1; i>=0; i--) {
			if (fabs(configC[i]-lastPoint[i])<SEUIL_JOINT) sensDepct[i]=1;
			else if (configC[i]-lastPoint[i]>0.0) sensDepct[i]=2;
			else sensDepct[i]=3;
			lastPoint[i]=configC[i]; // passage au point suivant
		}
		
		// mise a jour de handPoints
		handPoints.push_back(true);
		for (j=0; j<5; j++) {
			if (sensDepct[j]>1) {
				handPoints[indiceG]=false;
				break;
			}
		}
		if (handPoints[indiceG]) {
			for (j=5; j<8; j++)
				if (sensDepct[j]>1) break;
			if (j>=8) handPoints[indiceG]=false;
		}
		return;
	}
	
	// un "mouvement" a deja ete amorce :
	// note : dans ce cas on ne peut pas fusionner les intervalles contrairement au cas precedent,
	// compte-tenu du caractere "on-line" de l'algorithme.
	bool cut=false;
	for (i=DIM-1; i>=0; i--) {
		if (fabs(configC[i]-lastPoint[i])<SEUIL_JOINT) {
			if (sensDepct[i]!=1) cut=true;
			sensDepct[i]=1;
		}
		else if (configC[i]-lastPoint[i]>0.0) {
			if (sensDepct[i]!=2) cut=true;
			sensDepct[i]=2;
		}
		else {
			if (sensDepct[i]!=3) cut=true;
			sensDepct[i]=3;
		}
	}
	if (cut) decoupageM.push_back(indiceG);
	
	// mise a jour de handPoints
	handPoints.push_back(true);
	for (j=0; j<5; j++) {
		if (sensDepct[j]>1) {
			handPoints[indiceG]=false;
			break;
		}
	}
	if (handPoints[indiceG]) {
		for (j=5; j<8; j++)
			if (sensDepct[j]>1) break;
		if (j>=8) handPoints[indiceG]=false;
	}

	return;
}
