#include "DecoupObstacles.h"

// generateur aleatoire gaussien simple :
// (Algorithm is from Numerical Recipes in C, Second Edition)
double gaussRand(double mean,double stdDev) {
	static int iset=0;
	static double gset;
	double fac,rsq,v1,v2;

	if (iset==0) {
		do {
			v1=2.0*genAlea()-1.0;
			v2=2.0*genAlea()-1.0;
			rsq=v1*v1+v2*v2;
		}
		while (rsq>=1.0 || rsq<=0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return stdDev*v2*fac+mean;
	}
	else {
		iset=0;
		return stdDev*gset+mean;
	}
}

void gauss_sample(double sigma,double* pointTest) {
	static int compteurPassages=0;
	if (compteurPassages<E_PTS) {
		// calculs preliminaires : base adaptee pr l'echantillonnage
		// calcul de la matrice de changement de base "reperes bras / origine";
		// on transforme d'abord les coordonnes cartesiennes en angles spheriques :
		double* coudeTrans=(double*)malloc(3*sizeof(double));
		coudeTrans[0]=coord3D[3]-coord3D[0];
		coudeTrans[1]=coord3D[4]-coord3D[1];
		coudeTrans[2]=coord3D[5]-coord3D[2];
		double phi,theta;
		getSphericAngles(coudeTrans,phi,theta);
		free(coudeTrans);

		// on a juste besoin de X et Y; voir Functions.cpp pour la construction de Z
		double* coeffsX=(double*)malloc(3*sizeof(double));
		double* coeffsY=(double*)malloc(3*sizeof(double));
		double tmpSinP=sin(phi),tmpCosP=cos(phi);
		double tmpCosV=cos(theta);
		coeffsX[0]=tmpCosP*tmpCosV;
		coeffsX[1]=tmpSinP*tmpCosV;
		coeffsX[2]=-sin(theta);
		coeffsY[0]=-tmpSinP;
		coeffsY[1]=tmpCosP;
		coeffsY[2]=0.0;

		// echantillonnage au voisinage du segment E-C
		double alea=genAlea();
		// 1ere assignation au point, avant perturbation gaussienne radiale
		pointTest[0]=alea*coord3D[0]+(1-alea)*coord3D[3];
		pointTest[1]=alea*coord3D[1]+(1-alea)*coord3D[4];
		pointTest[2]=alea*coord3D[2]+(1-alea)*coord3D[5];
		// perturbation radiale :
		alea=genAlea();
		int rdb1=randBit(),rdb2=randBit();
		double distRay=RAY_C+gaussRand(0,sigma);
		pointTest[0]+=(rdb1*alea*coeffsX[0]+rdb2*(1-alea)*coeffsY[0])*distRay;
		pointTest[1]+=(rdb1*alea*coeffsX[1]+rdb2*(1-alea)*coeffsY[1])*distRay;
		pointTest[2]+=(rdb1*alea*coeffsX[2]+rdb2*(1-alea)*coeffsY[2])*distRay;
		free(coeffsX);
		free(coeffsY);
	}
	else if (compteurPassages<E_PTS+C_PTS) {
		// calculs preliminaires : base adaptee pr l'echantillonnage
		// calcul de la matrice de changement de base "reperes bras / origine";
		// on transforme d'abord les coordonnes cartesiennes en angles spheriques :
		double* poignetTrans=(double*)malloc(3*sizeof(double));
		poignetTrans[0]=coord3D[6]-coord3D[3];
		poignetTrans[1]=coord3D[7]-coord3D[4];
		poignetTrans[2]=coord3D[8]-coord3D[5];
		double phi,theta;
		getSphericAngles(poignetTrans,phi,theta);
		free(poignetTrans);

		// on a juste besoin de X et Y; voir Camera.cpp pour la construction de Z
		double* coeffsX=(double*)malloc(3*sizeof(double));
		double* coeffsY=(double*)malloc(3*sizeof(double));
		double tmpCosP=cos(phi),tmpSinP=sin(phi);
		double tmpCosV=cos(theta);
		coeffsX[0]=tmpCosP*tmpCosV;
		coeffsX[1]=tmpSinP*tmpCosV;
		coeffsX[2]=-sin(theta);
		coeffsY[0]=-tmpSinP;
		coeffsY[1]=tmpCosP;
		coeffsY[2]=0.0;

		// echantillonnage au voisinage du segment C-P
		double alea=genAlea();
		// 1ere assignation au point, avant perturbation gaussienne radiale
		pointTest[0]=alea*coord3D[3]+(1-alea)*coord3D[6];
		pointTest[1]=alea*coord3D[4]+(1-alea)*coord3D[7];
		pointTest[2]=alea*coord3D[5]+(1-alea)*coord3D[8];
		// perturbation radiale :
		alea=genAlea();
		int rdb1=randBit(),rdb2=randBit();
		double distRay=RAY_C+gaussRand(0,sigma);
		pointTest[0]+=(rdb1*alea*coeffsX[0]+rdb2*(1-alea)*coeffsY[0])*distRay;
		pointTest[1]+=(rdb1*alea*coeffsX[1]+rdb2*(1-alea)*coeffsY[1])*distRay;
		pointTest[2]+=(rdb1*alea*coeffsX[2]+rdb2*(1-alea)*coeffsY[2])*distRay;
		free(coeffsX);
		free(coeffsY);
	}
	else {
		// echantillonnage au voisinage du poignet;
		// 1er tirage gaussien pour obtenir le rayon:
		double rad=maxF(0.0,gaussRand(0,sigma)+RAY_P);
		double aleaX=randBit()*genAlea();
		double aleaY=randBit()*genAlea();
		double aleaZ=randBit()*genAlea();
		double coeffMult=rad/sqrt(aleaX*aleaX+aleaY*aleaY+aleaZ*aleaZ); // constante de proportionalite
		pointTest[0]=coord3D[6]+coeffMult*aleaX;
		pointTest[1]=coord3D[7]+coeffMult*aleaY;
		pointTest[2]=coord3D[8]+coeffMult*aleaZ;
	}
	compteurPassages=(compteurPassages+1)%(E_PTS+C_PTS+P_PTS);
	return;
}

// calcule la distance (presque..) exacte d'un point 3D  une configuration 8D du bras robot
// + memorise le point le plus proche sur le bras
double distance(double* point) {
	// methode : calcul des projetes orthogonaux sur chaque droite.
	// puis 2 cas : projete appartient au(x) segment(s), alors distance facilement calculee;
	// projete en dehors : on prend le joint le plus proche.
	double cX=coord3D[3]-coord3D[0],
			cY=coord3D[4]-coord3D[1],
			cZ=coord3D[5]-coord3D[2],
			cpX=coord3D[6]-coord3D[3],
			cpY=coord3D[7]-coord3D[4],
			cpZ=coord3D[8]-coord3D[5];
	double k=(cX*(point[0]-coord3D[0])+cY*(point[1]-coord3D[1])+cZ*(point[2]-coord3D[2]))/(cX*cX+cY*cY+cZ*cZ);
	double minDist=MAX_DIST;
	if (coord3D[0]+k*cX<=coord3D[3]) {
		// projete 1 appartient au segment :
		minDist=sqrt((coord3D[0]+k*cX-point[0])*(coord3D[0]+k*cX-point[0])
					+ (coord3D[1]+k*cY-point[1])*(coord3D[1]+k*cY-point[1])
					+ (coord3D[2]+k*cZ-point[2])*(coord3D[2]+k*cZ-point[2]));
		coordPointsB.back()[0]=coord3D[0]+k*cX;
		coordPointsB.back()[1]=coord3D[1]+k*cY;
		coordPointsB.back()[2]=coord3D[2]+k*cZ;
	}
	// on calcule de meme avec le second segment :
	k=(cpX*(point[0]-coord3D[3])+cpY*(point[1]-coord3D[4])+cpZ*(point[2]-coord3D[5]))/(cpX*cpX+cpY*cpY+cpZ*cpZ);
	double tmpDist;
	if (coord3D[3]+k*cpX<=coord3D[6]) {
		// projete 2 appartient au segment :
		tmpDist=sqrt((coord3D[3]+k*cpX-point[0])*(coord3D[3]+k*cpX-point[0])
					+ (coord3D[4]+k*cpY-point[1])*(coord3D[4]+k*cpY-point[1])
					+ (coord3D[5]+k*cpZ-point[2])*(coord3D[5]+k*cpZ-point[2]));
		if (tmpDist<minDist) {
			minDist=tmpDist;
			coordPointsB.back()[0]=coord3D[0]+k*cpX;
			coordPointsB.back()[1]=coord3D[1]+k*cpY;
			coordPointsB.back()[2]=coord3D[2]+k*cpZ;
		}
	}
	// check des distances aux joints :
	tmpDist=sqrt((point[0]-coord3D[0])*(point[0]-coord3D[0])
				+ (point[1]-coord3D[1])*(point[1]-coord3D[1])
				+ (point[2]-coord3D[2])*(point[2]-coord3D[2]));
	if (tmpDist<minDist) {
		minDist=tmpDist;
		coordPointsB.back()[0]=coord3D[0];
		coordPointsB.back()[1]=coord3D[1];
		coordPointsB.back()[2]=coord3D[2];
	}
	tmpDist=sqrt((point[0]-coord3D[3])*(point[0]-coord3D[3])
				+ (point[1]-coord3D[4])*(point[1]-coord3D[4])
				+ (point[2]-coord3D[5])*(point[2]-coord3D[5]));
	if (tmpDist<minDist) {
		minDist=tmpDist;
		coordPointsB.back()[0]=coord3D[3];
		coordPointsB.back()[1]=coord3D[4];
		coordPointsB.back()[2]=coord3D[5];
	}
	tmpDist=sqrt((point[0]-coord3D[6])*(point[0]-coord3D[6])
				+ (point[1]-coord3D[7])*(point[1]-coord3D[7])
				+ (point[2]-coord3D[8])*(point[2]-coord3D[8]));
	if (tmpDist<minDist) {
		minDist=tmpDist;
		coordPointsB.back()[0]=coord3D[6];
		coordPointsB.back()[1]=coord3D[7];
		coordPointsB.back()[2]=coord3D[8];
	}

	return minDist;
}

// mode pour calculer les distances via PQP (semble tres lent !)
#define PQP_MODE

// cas ou la trajectoire est connue a l'avance : (utilise eventuellement PQP)
void analyseObstacles(vector<mpkConfig>& configurations) {
	double min_dist_obstacle;
	vector<double> valeursDist; // distances a prendre en compte dans la boucle courante de l'algorithme
	vector<double> outsiders; // distances supposes dans un premier temps tre absurdes

	#ifndef PQP_MODE

		vector<bool> zoPoints(configurations.size());
		double sigma,dist_obstacle;
		double* pointTest=(double*)malloc(3*sizeof(double));
		double* pointBras=(double*)malloc(3*sizeof(double));

		for (int i=0; i<(int)configurations.size(); i++) {
			// code de recherche de la "plus courte distance" :
			coordObstacles.push_back((double*)calloc(3,sizeof(double)));
			coordPointsB.push_back((double*)calloc(3,sizeof(double)));
			for (int l=2; l>=0; l--) pointBras[l]=0.0;

			coord3D=conf2coord3D(configurations[i]);
			min_dist_obstacle=MAX_DIST;

			sigma=S_MIN;
			while (sigma<=S_MAX) {
				for (int k=E_PTS+C_PTS+P_PTS-1; k>=0; k--) {
					gauss_sample(sigma,pointTest);


/////////////////////////////////////////////////////
/*
					pointTest[0]=0.0;
					pointTest[1]=0.0;
					pointTest[2]=0.0;
PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}};

		double p0[][3]={{coord3D[0],coord3D[1],coord3D[2]},
							{coord3D[0],coord3D[1]+1e-8,coord3D[2]},
							{coord3D[3],coord3D[4],coord3D[5]}};
			
			PQP_Model* pointColl0=new PQP_Model();
			pointColl0->BeginModel();
			pointColl0->AddTri(p0[0],p0[1],p0[2],0);
			pointColl0->EndModel();
			

			PQP_REAL position0[3]={(coord3D[0]+coord3D[3])/2.0,
									(coord3D[1]+coord3D[4])/2.0,
									(coord3D[2]+coord3D[5])/2.0}; // la position du triangle0
			


PQP_DistanceResult* result=new PQP_DistanceResult();
for (int j=1; j<obstaclesBen->num_obstacles(); j++) {
PQP_Distance(result,rotateId,position0,pointColl0,
						obstaclesBen->Tr[j].R,obstaclesBen->Tr[j].T,obstaclesBen->pqp[j],
						0.5,0.5);
cout << "distance 000 :    " << result->Distance() << "       " << collisionBen(pointTest) << endl;
}
*/


//////////////////////////////////////////////////////


					if (collisionBen(pointTest)) {
						dist_obstacle=distance(pointTest);
						if (dist_obstacle<min_dist_obstacle) {
							min_dist_obstacle=dist_obstacle;
							for (int l=2; l>=0; l--) {
								coordObstacles.back()[l]=pointTest[l];
								pointBras[l]=coordPointsB.back()[l];
							}
						}
						break;
					}
				}
				sigma+=S_INC;
			}
			for (int l=2; l>=0; l--) coordPointsB.back()[l]=pointBras[l];

			cout << "passage probabiliste dans analyseObstacles()  " << i
					<< "  avec distance de  " << min_dist_obstacle << endl;
		}
		
		free(pointTest);
		free(pointBras);

		// appel de l'analyseur des segments de film avec zoom sur point proche obstacle :
		zoomObstPoints=analysePoints_Densite(zoPoints);

	#else

/*
		// METHODE LOURDE :
		// (ne renvoyant de surcroit pas les memes results.. koikaibon ?? =/ on se base sur l'autre en attendant)

		// variables PQP :
		PQP_DistanceResult* result=new PQP_DistanceResult();

		// parcours des configurations pour trouver les distances,
		// recherche simultanee des points de coupure :
		for (int i=0; i<(int)configurations.size(); i++) {
			
			// code de recherche de la plus courte distance :
			coordObstacles.push_back((double*)calloc(3,sizeof(double)));
			coordPointsB.push_back((double*)calloc(3,sizeof(double)));

			result=config_checkBen->distanceObstacles(&(configurations[i]));
			min_dist_obstacle=sqrt((result->p2[0]-result->p1[0])*(result->p2[0]-result->p1[0])
									+ (result->p2[1]-result->p1[1])*(result->p2[1]-result->p1[1])
									+ (result->p2[2]-result->p1[2])*(result->p2[2]-result->p1[2]));;
			for (int l=2; l>=0; l--) {
				coordObstacles.back()[l]=result->p2[l];
				coordPointsB.back()[l]=result->p1[l];
			}

			distObstacles.push_back(min_dist_obstacle);
			
			if (min_dist_obstacle<SEUIL_DIST_OBST) zoomObstPoints.push_back(true);
			else zoomObstPoints.push_back(false);

			cout << "passage PQP dans analyseObstacles()  " << i
					<< "  avec distance de  " << min_dist_obstacle << endl;
		}
		delete result;
*/

		// METHODE LEGERE (... et correcte ? en tt k, coherente) :

		PQP_DistanceResult* result=new PQP_DistanceResult();
		PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}}; // tranformation identite
		double distBrasObst;

		// parcours des configurations pour trouver les distances,
		// recherche simultanee des points de coupure :
		for (int i=0; i<(int)configurations.size(); i++) {
			coord3D=conf2coord3D(configurations[i]);
			min_dist_obstacle=MAX_DIST;

			// code de recherche de la plus courte distance :
			coordObstacles.push_back((double*)calloc(3,sizeof(double)));
			coordPointsB.push_back((double*)calloc(3,sizeof(double)));

			// setup pqp model with one triangle == modelisation de chaque lien
			double p0[][3]={{coord3D[0],coord3D[1],coord3D[2]},
							{coord3D[0],coord3D[1]+1e-8,coord3D[2]},
							{coord3D[3],coord3D[4],coord3D[5]}};
			double p1[][3]={{coord3D[3],coord3D[4],coord3D[5]},
							{coord3D[3],coord3D[4]+1e-8,coord3D[5]},
							{coord3D[6],coord3D[7],coord3D[8]}};
			PQP_Model* pointColl0=new PQP_Model();
			pointColl0->BeginModel();
			pointColl0->AddTri(p0[0],p0[1],p0[2],0);
			pointColl0->EndModel();
			PQP_Model* pointColl1=new PQP_Model();
			pointColl1->BeginModel();
			pointColl1->AddTri(p1[0],p1[1],p1[2],0);
			pointColl1->EndModel();

			PQP_REAL position0[3]={(coord3D[0]+coord3D[3])/2.0,
									(coord3D[1]+coord3D[4])/2.0,
									(coord3D[2]+coord3D[5])/2.0}; // la position du triangle0
			PQP_REAL position1[3]={(coord3D[3]+coord3D[6])/2.0,
									(coord3D[4]+coord3D[7])/2.0,
									(coord3D[5]+coord3D[8])/2.0}; // la position du triangle1
		
			// boucle sur tous les obstacles sauf Terre :
			for (int j=1; j<obstaclesBen->num_obstacles(); j++) {
				PQP_Distance(result,rotateId,position0,pointColl0,
						obstaclesBen->Tr[j].R,obstaclesBen->Tr[j].T,obstaclesBen->pqp[j],
						0.5,0.5); // 0.5 a ajuster ...
				distBrasObst=sqrt((result->p2[0]-result->p1[0])*(result->p2[0]-result->p1[0])
									+ (result->p2[1]-result->p1[1])*(result->p2[1]-result->p1[1])
									+ (result->p2[2]-result->p1[2])*(result->p2[2]-result->p1[2]));
				if (distBrasObst<min_dist_obstacle) {
					// alors on actualise nos variables :
					min_dist_obstacle=distBrasObst;
					for (int l=2; l>=0; l--) {
						coordObstacles.back()[l]=result->p2[l];
						coordPointsB.back()[l]=result->p1[l];
					}
				}
				PQP_Distance(result,rotateId,position1,pointColl1,
						obstaclesBen->Tr[j].R,obstaclesBen->Tr[j].T,obstaclesBen->pqp[j],
						0.5,0.5); // 0.5 a ajuster ...
				distBrasObst=sqrt((result->p2[0]-result->p1[0])*(result->p2[0]-result->p1[0])
									+ (result->p2[1]-result->p1[1])*(result->p2[1]-result->p1[1])
									+ (result->p2[2]-result->p1[2])*(result->p2[2]-result->p1[2]));
				if (distBrasObst<min_dist_obstacle) {
					// alors on actualise nos variables :
					min_dist_obstacle=distBrasObst;
					for (int l=2; l>=0; l--) {
						coordObstacles.back()[l]=result->p2[l];
						coordPointsB.back()[l]=result->p1[l];
					}
				}
			}

			// liberations memoire :
			delete pointColl0;
			delete pointColl1;

			distObstacles.push_back(min_dist_obstacle);
			
			if (min_dist_obstacle<SEUIL_DIST_OBST) zoomObstPoints.push_back(true);
			else zoomObstPoints.push_back(false);

			cout << "passage PQP dans analyseObstacles()  " << i
					<< "  avec distance de  " << min_dist_obstacle << endl;
		}
		delete result;

	#endif

	// code de recherche des points de coupure base sur le vecteur zoomObstPoints :
	bool gTrue=false,firstTGroup=true;
	for (int i=0; i<(int)zoomObstPoints.size(); i++) {
		if (zoomObstPoints[i]==true) { // on rencontre un groupe de valeurs "true"
			if (!gTrue) {
				gTrue=true;
				if (firstTGroup && i>0) decoupageO.push_back(i);
				firstTGroup=false;
			}
		}
		else {
			if (gTrue) {
				// i est un point de coupure :
				decoupageO.push_back(i);
				gTrue=false;
			}
		}
	}

	#ifndef PQP_MODE
	
		// smoothing des distances : lorsqu'un point est marque comme MAX_DIST,
		// on lui assigne la distance moyenne des deux valeurs coherentes de part et d'autre :
		double leftVal,rightVal;
		int leftInd,rightInd;
		for (int i=zoomObstPoints.size()-1; i>=0; i--) {
			if (zoomObstPoints[i] && distObstacles[i]==MAX_DIST) {
				leftVal=MAX_DIST;
				rightVal=MAX_DIST;
				leftInd=0;
				rightInd=0;

				for (int j=i-1; j>=0; j--) {
					if (distObstacles[j]<MAX_DIST) {
						leftVal=distObstacles[j];
						leftInd=j;
						break;
					}
				}
				for (int j=i+1; j<(int)distObstacles.size(); j++) {
					if (distObstacles[j]<MAX_DIST) {
						rightVal=distObstacles[j];
						rightInd=j;
						break;
					}
				}

				if (leftVal==MAX_DIST) {
					distObstacles[i]=rightVal;
					coordObstacles[i]=coordObstacles[rightInd];
					coordPointsB[i]=coordPointsB[rightInd];
				}
				else if (rightVal==MAX_DIST) {
					distObstacles[i]=leftVal;
					coordObstacles[i]=coordObstacles[leftInd];
					coordPointsB[i]=coordPointsB[leftInd];
				}
				else {
					distObstacles[i]=(leftVal+rightVal)/2.0;
					// on ne peut pas effectuer une moyenne sur les coordonnes (aucun sens);
					// on choisit donc au hasard :
					if (randBit()>0) {
						coordObstacles[i]=coordObstacles[leftInd];
						coordPointsB[i]=coordPointsB[leftInd];
					}
					else {
						coordObstacles[i]=coordObstacles[rightInd];
						coordPointsB[i]=coordPointsB[rightInd];
					}
				}
			}
		}

	#endif

	return;
}

// cas ou la trajectoire est imprevisible :
void analyseObstaclesImp(mpkConfig& configC) {

	#ifndef PQP_MODE

		double sigma,dist_obstacle;
		double* pointTest=(double*)malloc(3*sizeof(double));
		double* pointBras=(double*)calloc(3,sizeof(double));

	#endif

	double min_dist_obstacle=MAX_DIST;
	coordObstacles.push_back((double*)calloc(3,sizeof(double)));
	coordPointsB.push_back((double*)calloc(3,sizeof(double)));
	// variables static car doivent etre memorisees d'un appel a l'autre :
	static vector<double> valeursDist; // distances a prendre en compte dans la boucle courante de l'algorithme
	static double moy=0.0; // moyenne de ces distances
	static vector<double> outsiders; // distances supposes dans un premier temps tre absurdes
	static int compteur=0;
	
	#ifndef PQP_MODE

		sigma=S_MIN;
		while (sigma<=S_MAX) {
			for (int k=E_PTS+C_PTS+P_PTS-1; k>=0; k--) {
				gauss_sample(sigma,pointTest);
				if (collisionBen(pointTest)) {
					dist_obstacle=distance(pointTest);
					if (dist_obstacle<min_dist_obstacle) {
						min_dist_obstacle=dist_obstacle;
						for (int l=2; l>=0; l--) {
							coordObstacles.back()[l]=pointTest[l];
							pointBras[l]=coordPointsB.back()[l];
						}
					}
					break;
				}
			}
			sigma+=S_INC;
		}
		for (int l=2; l>=0; l--) coordPointsB.back()[l]=pointBras[l];
		free(pointTest);
		free(pointBras);

		cout << "passage probabiliste dans analyseObstaclesImp()  " << indiceG
				<< "  avec distance de  " << min_dist_obstacle << endl;

	#else

		// code avec PQP : comme dans la version avec trajectoire connue

/*
		// METHODE LOURDE : (et correcte ??)

		PQP_DistanceResult* result=new PQP_DistanceResult();

		result=config_checkBen->distanceObstacles(&configC);
		min_dist_obstacle=result->Distance();
		for (int l=2; l>=0; l--) {
			coordObstacles.back()[l]=result->p2[l];
			coordPointsB.back()[l]=result->p1[l];
		}
		delete result;

		cout << "passage PQP dans analyseObstaclesImp()  " << indiceG
				<< "  avec distance de  " << min_dist_obstacle << endl;
*/

		// METHODE LEGERE (... et correcte ? en tt k, coherente) :

		double distBrasObst;
	
		// variables PQP :
		PQP_DistanceResult* result=new PQP_DistanceResult();
		PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}}; // tranformation identite
		
		// setup pqp model with one triangle == modelisation de chaque lien
		double p0[][3]={{coord3D[0],coord3D[1],coord3D[2]},
						{coord3D[0],coord3D[1]+1e-8,coord3D[2]},
						{coord3D[3],coord3D[4],coord3D[5]}};
		double p1[][3]={{coord3D[3],coord3D[4],coord3D[5]},
						{coord3D[3],coord3D[4]+1e-8,coord3D[5]},
						{coord3D[6],coord3D[7],coord3D[8]}};
		PQP_Model* pointColl0=new PQP_Model();
		pointColl0->BeginModel();
		pointColl0->AddTri(p0[0],p0[1],p0[2],0);
		pointColl0->EndModel();
		PQP_Model* pointColl1=new PQP_Model();
		pointColl1->BeginModel();
		pointColl1->AddTri(p1[0],p1[1],p1[2],0);
		pointColl1->EndModel();

		PQP_REAL position0[3]={(coord3D[0]+coord3D[3])/2.0,
								(coord3D[1]+coord3D[4])/2.0,
								(coord3D[2]+coord3D[5])/2.0}; // la position du triangle0
		PQP_REAL position1[3]={(coord3D[3]+coord3D[6])/2.0,
								(coord3D[4]+coord3D[7])/2.0,
								(coord3D[5]+coord3D[8])/2.0}; // la position du triangle1

		// boucle sur tous les obstacles sauf Terre :
		for (int i=1; i<obstaclesBen->num_obstacles(); i++) {
			PQP_Distance(result,rotateId,position0,pointColl0,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i],
					0.5,0.5);
			distBrasObst=sqrt((result->p2[0]-result->p1[0])*(result->p2[0]-result->p1[0])
								+ (result->p2[1]-result->p1[1])*(result->p2[1]-result->p1[1])
								+ (result->p2[2]-result->p1[2])*(result->p2[2]-result->p1[2]));
			if (distBrasObst<min_dist_obstacle) {
				// alors on actualise nos variables :
				min_dist_obstacle=distBrasObst;
				for (int l=2; l>=0; l--) {
					coordObstacles.back()[l]=result->p2[l];
					coordPointsB.back()[l]=result->p1[l];
				}
			}
			PQP_Distance(result,rotateId,position1,pointColl1,
					obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i],
					0.5,0.5); // 0.5 a ajuster ...
			distBrasObst=sqrt((result->p2[0]-result->p1[0])*(result->p2[0]-result->p1[0])
								+ (result->p2[1]-result->p1[1])*(result->p2[1]-result->p1[1])
								+ (result->p2[2]-result->p1[2])*(result->p2[2]-result->p1[2]));
			if (distBrasObst<min_dist_obstacle) {
				// alors on actualise nos variables :
				min_dist_obstacle=distBrasObst;
				for (int l=2; l>=0; l--) {
					coordObstacles.back()[l]=result->p2[l];
					coordPointsB.back()[l]=result->p1[l];
				}
			}
		}

		// liberations memoire :
		delete pointColl0;
		delete pointColl1;
		delete result;

		cout << "passage PQP dans analyseObstaclesImp()  " << indiceG
				<< "  avec distance de  " << min_dist_obstacle << endl;

	#endif

	distObstacles.push_back(min_dist_obstacle);
	
	if (min_dist_obstacle<SEUIL_DIST_OBST) zoomObstPoints.push_back(true);
	else zoomObstPoints.push_back(false);
	
	// code de recherche des points de coupure :
	if (distObstacles.size()==1) {
		// demarrage :
		valeursDist.push_back(min_dist_obstacle);
		moy=min_dist_obstacle;
		compteur++;
	}
	else {
		if (fabs(min_dist_obstacle-moy)<SEUIL_OBST) {
			valeursDist.push_back(min_dist_obstacle);
			compteur+=(outsiders.size()+1);
			outsiders.clear(); // important : effacage des outsiders si valeur coherente
			moy=(moy+min_dist_obstacle)/2;
		}
		else {
			outsiders.push_back(min_dist_obstacle);
			if (outsiders.size()==4) {
				if (compteur>0) decoupageO.push_back(compteur);
				compteur+=4;
				valeursDist=outsiders;
				outsiders.clear();
				// recalcul de la moyenne :
				moy=0.0;
				for (int k=3; k>=0; k--) moy+=valeursDist[k];
				moy/=4;
			}
		}
	}

	return;
}
