#include "EvalIdiomViews.h"

// fonction evaluant la qualit d'un idiome en terme de visibilite de l'objectif;
// on prend en parametre la liste de configurations du bras pour pouvoir realiser l'evaluation.
// + on utilise les parametres globaux donnant le type d'objectif.
void evalViews(Idiom& idiom,const vector<mpkConfig>& configurations) {
	
	Camera cam=Camera(); // objet servant a evaluer chaque image
	// moyennes des vues sur plusieurs points du film :
	vector<double> moyVis(idiom.views.size());
	for (int i=idiom.views.size()-1; i>=0; i--) moyVis[i]=0.0;
	
	// 3 cas : film seulement la main, tout sauf track, et track le long du rail.
	// dans le premier cas une seule evaluation suffit, dans le second on en effectue 3,
	// et dans le dernier il faudra evaluer en plusieurs points du parcours.
	
	if (handPoints[idiom.shots[0].firstInd]) {
		// determination du point de repere :
		coord3D=conf2coord3D(configurations[idiom.shots[0].firstInd]);
		getRepPoint(idiom.shots[0].firstInd);
		
		// calcul de la distance de vue :
		getViewDistance(idiom.shots[0].firstInd);
		
		// on evalue alors les vues :
		for (int i=idiom.views.size()-1; i>=0; i--)
			moyVis[i]+=testView(cam,idiom.views[i]);
	}

	else if (strcmp(idiom.name,"track")!=0) {
		// determination du point de repere initial :
		coord3D=conf2coord3D(configurations[idiom.shots[0].firstInd]);
		getRepPoint(idiom.shots[0].firstInd);
		
		// calcul de la distance de vue :
		getViewDistance(idiom.shots[0].firstInd);
		
		// on evalue alors les vues :
		for (int i=idiom.views.size()-1; i>=0; i--)
			moyVis[i]+=testView(cam,idiom.views[i]);
		
		// determination du point de repere intermediaire :
		coord3D=conf2coord3D(configurations[(idiom.shots[0].firstInd+idiom.shots[0].lastInd)/2]);
		getRepPoint((idiom.shots[0].firstInd+idiom.shots[0].lastInd)/2);
		
		// calcul de la distance de vue :
		getViewDistance((idiom.shots[0].firstInd+idiom.shots[0].lastInd)/2);
		
		// on evalue alors les vues :
		for (int i=idiom.views.size()-1; i>=0; i--)
			moyVis[i]+=testView(cam,idiom.views[i]);
		
		// determination du point de repere final :
		coord3D=conf2coord3D(configurations[idiom.shots[0].lastInd]);
		getRepPoint(idiom.shots[0].lastInd);
		
		// calcul de la distance de vue :
		getViewDistance(idiom.shots[0].lastInd);
		
		// on evalue alors les vues :
		for (int i=idiom.views.size()-1; i>=0; i--)
			moyVis[i]+=testView(cam,idiom.views[i]);
	}
	
	else {
		// cas le plus delicat : evaluation du shot track.
		// on selectionne alors plusieurs point le long du parcours selon une loi uniforme,
		// plus on prend de points et meilleure est la precision..
		
		int indice;
		for (int i=NB_ECH_PTS-1; i>=0; i--) {
			// tirage aleatoire de l'indice :
			indice=genAleaInt(idiom.shots[0].lastInd-idiom.shots[0].firstInd+1);
			
			// determination du point de repere :
			coord3D=conf2coord3D(configurations[indice]);
			getRepPoint(indice);
			
			// calcul de la distance de vue :
			getViewDistance(indice);
			
			// on evalue alors les vues :
			for (int i=idiom.views.size()-1; i>=0; i--)
				moyVis[i]+=testView(cam,idiom.views[i]);
		}
	}

	// on garde dans views[0] la meilleure vue si celle-ci
	// a une visibilite plus grande que 0.7 (sinon vue aleatoire; ne devrait pas arriver) :
	// (on trie en fait les valeurs, ce sera utile par la suite).
	sort(moyVis.begin(),moyVis.end());
	// elimination des angles de vue trop mauvais :
	for (int i=0; i<(int)moyVis.size(); i++) {
		if (moyVis[i]<0.7) { // 0.7 --> a ajuster......
			for (int j=i; j<(int)moyVis.size(); j++) idiom.views[j]=-1;
			break;
		}
	}
	
	return;
}
