#include "FilmRT.h"

// initialise avec les objets obstacles et robot :
void initialiseFilmRT(mpkObstacleCollection* obsts,mpkRobotCollection* robts) {
	Global glob=Global();
	obstaclesBen=obsts;
	robotsBen=robts;

	Functions func=Functions();
	currPR=(double*)malloc(3*sizeof(double));

	mpkCollPairSet* collcheck_pairs = new mpkCollPairSet(robotsBen,obstaclesBen,SEUIL_DIST_OBST);
	config_checkBen = new mpkConfigChecker(&collcheck_pairs->test_pairs, robotsBen);

	idiomsMode=false;

	return;
}

// fonction principale effectuant la planification :
// prend en argument une suite de vecteurs  8 dimensions;
// ceux-ci correspondent aux configurations successives du bras-robot
vector<ConfigCamera>* mainFilmRT (vector<mpkConfig>& configurations,char* argv) {
	// UTILISATION :
	// FilmRT.exe <fichier_trajectoire> <trajectoire connue ou non (c ou i resp.)> <types de filmage>

	// calcul de chaque configuration de camera, une par une :
	vector<ConfigCamera>* configsCam=new vector<ConfigCamera>[3];
	if (argv[0]=='c') {
		// a) on teste en connaissant la trajectoire a l'avance :
		// deux cas : filmage 'contraintes' ou 'idiomes'
		
		if (argv[1]=='c') {
			cout << "iterCam 0" << endl;

			// decoupage de la trajectoire
			analyseMonotone(configurations);
			analyseObstacles(configurations);
			
			coord3D=conf2coord3D(configurations[0]);
			PlaceCamera placeCam=PlaceCamera(); // l'objet servant a (de)placer les cameras

			indiceG=0; // indice courant de parcours de la trajectoire

			// determination du point de repere courant :
			getRepPoint(0);

			// calcul de la distance de vue :
			getViewDistance(0);

			// camera centrale :
			placeCam.initCamPos(1);
			configsCam[1].push_back(placeCam.cameras[1].getConfig());

			// camera de gauche :
			placeCam.initCamPos(0);
			configsCam[0].push_back(placeCam.cameras[0].getConfig());

			// camera de droite :
			placeCam.initCamPos(2);
			configsCam[2].push_back(placeCam.cameras[2].getConfig());

			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int i=1; i<(int)configurations.size(); i++) {
				cout << "iterCam " << i << endl;

				indiceG++;
				coord3D=conf2coord3D(configurations[i]);
	
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
	
				// calcul du nouveau point de repere :
				getRepPoint(i);

				// calcul de la distance de vue :
				getViewDistance(i);
		
				for (int j=2; j>=0; j--) depctRepPoint[j]=currPR[j]-lastRepPoint[j];

				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam[1].push_back(placeCam.cameras[1].getConfig());

				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam[0].push_back(placeCam.cameras[0].getConfig());

				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam[2].push_back(placeCam.cameras[2].getConfig());
			}

			// lissage :
			configsCam[1]=lisse(configsCam[1],configurations);
			configsCam[0]=lisse(configsCam[0],configurations);
			configsCam[2]=lisse(configsCam[2],configurations);
			
			// liberations memoire :
			free(lastRepPoint);
			free(depctRepPoint);
		}
		
		else if (argv[1]=='i') {
			// approche idiomes :
			idiomsMode=true;

			// constitution de la base d'idiomes
			buildIdiomsBase();

			if (argv[2]=='k') {
				// approche en couloirs suggeree par Khaled --> basee sur l'environnement seulement.
				// NON IMPLEMENTE car decoupage en zones non effectue.
				// completer les fonctions buildZonesBase() et idiomsFromZones(..) des que possible.
			
				// a) constitution de la base de zones definissant les idiomes :
				cout << "build idioms base...." << endl;
				buildZonesBase();
				
				// b) parcours de la trajectoire du centre de gravite
				// et choix d'un idiome sur chaque portion :
				cout << "get idioms from path...." << endl;
				vector<Idiom> idioms=idiomsFromZones(configurations);
				
				// c) parcours de la liste d'idiomes et gnration de la suite de configurations de (trois!) cameras correspondante
				cout << "get pathCam from idioms...." << endl;
				configsCam=compilation(idioms,configurations);
			}

			else if (argv[2]=='t') {
				// approche basee seulement sur la nature de la trajectoire du bras :
			
				// a) parcours de la trajectoire de chaque element du bras,
				// et choix d'un idiome sur chaque portion :
				cout << "get idioms from path...." << endl;
				vector<Idiom> idioms=path2idioms(configurations);
			
				// b) parcours de la liste d'idiomes et generation de la suite de configurations de (trois!) cameras correspondante
				cout << "get pathCam from idioms...." << endl;
				configsCam=compilation(idioms,configurations);
			}
			else {
				cerr << "Options non reconnues" << endl;
			}
		}

		else {
			cerr << "Options non reconnues" << endl;
		}
	}
	
	else if (argv[0]=='i') {
		
		if (argv[1]=='c') {
			
			// filmage "contraintes" :
		
			// b) on simule une trajectoire imprevisible en la donnant point par point :
			// approche idiomes impossible cette fois.
	
			cout << "iterCam 0" << endl;

			indiceG=0; // indice courant de parcours de la trajectoire
			coord3D=conf2coord3D(configurations[0]);
	
			// decoupage "online" :
			analyseMonotoneImp(configurations[0]);
			analyseObstaclesImp(configurations[0]);
	
			PlaceCamera placeCam=PlaceCamera(); // l'objet servant a (de)placer la camera
	
			// calcul du point de repere :
			getRepPoint(0);
			
			// calcul de la distance de vue :
			getViewDistance(0);
	
			// camera centrale :
			placeCam.initCamPos(1);
			configsCam[1].push_back(placeCam.cameras[1].getConfig());
	
			// camera de gauche :
			placeCam.initCamPos(0);
			configsCam[0].push_back(placeCam.cameras[0].getConfig());
	
			// camera de droite :
			placeCam.initCamPos(2);
			configsCam[2].push_back(placeCam.cameras[2].getConfig());
	
			double *lastRepPoint=(double*)malloc(3*sizeof(double)),
					*depctRepPoint=(double*)malloc(3*sizeof(double));
			for (int i=1; i<(int)configurations.size(); i++) {
				cout << "iterCam " << i << endl;
				
				indiceG++;
				coord3D=conf2coord3D(configurations[i]);
				
				// decoupage "online" :
				analyseMonotoneImp(configurations[i]);
				analyseObstaclesImp(configurations[i]);
				
				// memorisation du dernier point de repere :
				lastRepPoint[0]=currPR[0];
				lastRepPoint[1]=currPR[1];
				lastRepPoint[2]=currPR[2];
		
				// calcul du nouveau point de repere :
				getRepPoint(i);
				
				// calcul de la distance de vue :
				getViewDistance(i);
	
				for (int j=2; j>=0; j--) depctRepPoint[j]=currPR[j]-lastRepPoint[j];
	
				// camera centrale :
				placeCam.moveCamera(depctRepPoint,1);
				placeCam.adjustCamPos(1);
				configsCam[1].push_back(placeCam.cameras[1].getConfig());
		
				// camera de gauche :
				placeCam.moveCamera(depctRepPoint,0);
				placeCam.adjustCamPos(0);
				configsCam[0].push_back(placeCam.cameras[0].getConfig());
			
				// camera de droite :
				placeCam.moveCamera(depctRepPoint,2);
				placeCam.adjustCamPos(2);
				configsCam[2].push_back(placeCam.cameras[2].getConfig());
			}
			
			// liberations memoire :
			free(lastRepPoint);
			free(depctRepPoint);
		}
		
		else if (argv[1]=='f') {
			// filmage avec cameras fixes, d'une trajectoire imprevisible (sinon aucun interet) :
			// calcul de la suite "ideale" de configurations de cameras.
		
			// creation des cameras fixes :
			FilmageFixe filmF=FilmageFixe();
			
			ConfigCamera* configCamsFixes=new ConfigCamera[3];
			indiceG=0;
			for (int i=0; i<(int)configurations.size(); i++) {
				cout << "iterCam " << i << endl;

				coord3D=conf2coord3D(configurations[i]);
				
				// decoupage "online" :
				analyseMonotoneImp(configurations[i]);
				analyseObstaclesImp(configurations[i]);

				// calcul du point de repere :
				getRepPoint(i);
				
				// calcul de la distance de vue :
				getViewDistance(i);
				
				// obtention des configurations de camera :
				filmF.getCamConfig(configCamsFixes);
				configsCam[1].push_back(configCamsFixes[1]);
				configsCam[0].push_back(configCamsFixes[0]);
				configsCam[2].push_back(configCamsFixes[2]);
				
				indiceG++;
			}
			delete [] configCamsFixes;
		}
		
		else {
			cerr << "Options non reconnues" << endl;
		}
	}
	
	else {
		cerr << "Options non reconnues" << endl;
	}

/*
// Test vitesse PQP vs sampling gaussien :
ofstream test("pqpVSsGauss.txt");
for (int i=0; i<(int)configurations.size(); i++) {
	coord3D=conf2coord3D(configurations[i]);
	analyseObstaclesImp(configurations[i]);
	test << distObstacles[i] << endl;
}
*/

	return configsCam;
}
