#include "FilmageFixe.h"

#include <iostream>

using namespace std;

// initialisation des cameras (a corriger avec les vraies valeurs) :
FilmageFixe::FilmageFixe () {
	camerasF=new Camera[NB_CAMS];

	camerasF[0].posX=6.20204;
	camerasF[0].posY=12.573;
	camerasF[0].posZ=357.522;
	camerasF[0].roll=0.0;

	camerasF[1].posX=-12.6783;
	camerasF[1].posY=-4.55809;
	camerasF[1].posZ=281.89;
	camerasF[1].roll=0.0;
	
	camerasF[2].posX=-50.7536;
	camerasF[2].posY=63.9251;
	camerasF[2].posZ=269.873;
	camerasF[2].roll=0.0;
	
	camerasF[3].posX=-38.4679;
	camerasF[3].posY=-20.5531;
	camerasF[3].posZ=147.948;
	camerasF[3].roll=0.0;
	
	camerasF[4].posX=-82.3542;
	camerasF[4].posY=69.8355;
	camerasF[4].posZ=157.182;
	camerasF[4].roll=0.0;
	
	camerasF[5].posX=-120.182;
	camerasF[5].posY=-65.6899;
	camerasF[5].posZ=-116.157;
	camerasF[5].roll=0.0;

	camerasF[6].posX=-162.531;
	camerasF[6].posY=14.6718;
	camerasF[6].posZ=-128.422;
	camerasF[6].roll=0.0;
	
	camerasF[7].posX=-189.516;
	camerasF[7].posY=-81.2736;
	camerasF[7].posZ=-307.9;
	camerasF[7].roll=0.0;
	
	camerasF[8].posX=-197.092;
	camerasF[8].posY=-1.96433;
	camerasF[8].posZ=-251.416;
	camerasF[8].roll=0.0;
	
	camerasF[9].posX=-177.013;
	camerasF[9].posY=-85.2833;
	camerasF[9].posZ=-327.652;
	camerasF[9].roll=0.0;
	
	camerasF[10].posX=-109.058;
	camerasF[10].posY=121.11;
	camerasF[10].posZ=6.13425;
	camerasF[10].roll=0.0;
}

// obtention des configurations de cameras pour l'image courante :
void FilmageFixe::getCamConfig(ConfigCamera* resConfigs) {
	static int indice1=-1,indice0=-1,indice2=-1; // les precedents indices des cameras, que l'on teste en premier
	
	// on ordonne les cameras pour le filmage de cette configuration :
	double bestVis1=0.0,bestVis0=0.0,bestVis2=0.0;
	double* visibilite=(double*)calloc(NB_CAMS,sizeof(double));
	int bestInd1=-1,bestInd0=-1,bestInd2=-1;
	for (int j=NB_CAMS-1; j>=0; j--) {
		visibilite[j]=camerasF[j].getVisibilite(); // initialise aussi les angles

		if (visibilite[j]>bestVis1) {
			bestVis2=bestVis0;
			bestVis0=bestVis1;
			bestVis1=visibilite[j];
			
			bestInd2=bestInd0;
			bestInd0=bestInd1;
			bestInd1=j;
		}
		
		else if (visibilite[j]>bestVis0) {
			bestVis2=bestVis0;
			bestVis0=visibilite[j];
			
			bestInd2=bestInd0;
			bestInd0=j;
		}
		
		else if (visibilite[j]>bestVis2) {
			bestVis2=visibilite[j];
			bestInd1=j;
		}
	}
	
	// sauvegarde de la distance de vue "normale" :
	double normDist=distCamPR;
	
	// ********** camera 1 (centrale) : ***************
	if (indice1>=0 && visibilite[indice1]>=SEUIL_VIS_FIXE) {cout << "recup ancienne vue !! " << endl;
		// on garde l'ancienne vue :
		// distance a l'objectif :
		distCamPR=sqrt((currPR[0]-camerasF[indice1].posX)*(currPR[0]-camerasF[indice1].posX)
							+ (currPR[1]-camerasF[indice1].posY)*(currPR[1]-camerasF[indice1].posY)
							+ (currPR[2]-camerasF[indice1].posZ)*(currPR[2]-camerasF[indice1].posZ));
		camerasF[indice1].zoom=maxF(distCamPR/normDist,1.0);
		resConfigs[1]=camerasF[indice1].getConfig();
	}
	else {
		if (bestInd1>=0) {
			// distance a l'objectif :
			distCamPR=sqrt((currPR[0]-camerasF[bestInd1].posX)*(currPR[0]-camerasF[bestInd1].posX)
								+ (currPR[1]-camerasF[bestInd1].posY)*(currPR[1]-camerasF[bestInd1].posY)
								+ (currPR[2]-camerasF[bestInd1].posZ)*(currPR[2]-camerasF[bestInd1].posZ));
			camerasF[bestInd1].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[1]=camerasF[bestInd1].getConfig();
			indice1=bestInd1;
		}
		else {
			// choix de la camera 5 (par defaut) :
			distCamPR=sqrt((currPR[0]-camerasF[5].posX)*(currPR[0]-camerasF[5].posX)
								+ (currPR[1]-camerasF[5].posY)*(currPR[1]-camerasF[5].posY)
								+ (currPR[2]-camerasF[5].posZ)*(currPR[2]-camerasF[5].posZ));
			camerasF[5].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[1]=camerasF[5].getConfig();
		}
	}

	// ********** camera 0 (gauche) : ***************
	if (indice0>=0 && visibilite[indice0]>=SEUIL_VIS_FIXE) {
		// on garde l'ancienne vue :
		// distance a l'objectif :
		distCamPR=sqrt((currPR[0]-camerasF[indice0].posX)*(currPR[0]-camerasF[indice0].posX)
							+ (currPR[1]-camerasF[indice0].posY)*(currPR[1]-camerasF[indice0].posY)
							+ (currPR[2]-camerasF[indice0].posZ)*(currPR[2]-camerasF[indice0].posZ));
		camerasF[indice0].zoom=maxF(distCamPR/normDist,1.0);
		resConfigs[0]=camerasF[indice0].getConfig();
	}
	else {
		if (bestInd0>=0 && bestInd0!=indice1) {
			// distance a l'objectif :
			distCamPR=sqrt((currPR[0]-camerasF[bestInd0].posX)*(currPR[0]-camerasF[bestInd0].posX)
								+ (currPR[1]-camerasF[bestInd0].posY)*(currPR[1]-camerasF[bestInd0].posY)
								+ (currPR[2]-camerasF[bestInd0].posZ)*(currPR[2]-camerasF[bestInd0].posZ));
			camerasF[bestInd0].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[0]=camerasF[bestInd0].getConfig();
			indice0=bestInd0;
		}
		else {
			// choix de la camera 1 (par defaut) :
			distCamPR=sqrt((currPR[0]-camerasF[1].posX)*(currPR[0]-camerasF[1].posX)
								+ (currPR[1]-camerasF[1].posY)*(currPR[1]-camerasF[1].posY)
								+ (currPR[2]-camerasF[1].posZ)*(currPR[2]-camerasF[1].posZ));
			camerasF[1].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[0]=camerasF[1].getConfig();
		}
	}
	
	// ********** camera 2 (droite) : ***************
	if (indice2>=0 && visibilite[indice2]>=SEUIL_VIS_FIXE) {
		// on garde l'ancienne vue :
		// distance a l'objectif :
		distCamPR=sqrt((currPR[0]-camerasF[indice2].posX)*(currPR[0]-camerasF[indice2].posX)
							+ (currPR[1]-camerasF[indice2].posY)*(currPR[1]-camerasF[indice2].posY)
							+ (currPR[2]-camerasF[indice2].posZ)*(currPR[2]-camerasF[indice2].posZ));
		camerasF[indice2].zoom=maxF(distCamPR/normDist,1.0);
		resConfigs[2]=camerasF[indice2].getConfig();
	}
	else {
		if (bestInd2>=0 && bestInd2!=indice1 && bestInd2!=indice0) {
			// distance a l'objectif :
			distCamPR=sqrt((currPR[0]-camerasF[bestInd2].posX)*(currPR[0]-camerasF[bestInd2].posX)
								+ (currPR[1]-camerasF[bestInd2].posY)*(currPR[1]-camerasF[bestInd2].posY)
								+ (currPR[2]-camerasF[bestInd2].posZ)*(currPR[2]-camerasF[bestInd2].posZ));
			camerasF[bestInd2].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[2]=camerasF[bestInd2].getConfig();
		}
		else {
			// choix de la camera 9 (par defaut) :
			distCamPR=sqrt((currPR[0]-camerasF[9].posX)*(currPR[0]-camerasF[9].posX)
								+ (currPR[1]-camerasF[9].posY)*(currPR[1]-camerasF[9].posY)
								+ (currPR[2]-camerasF[9].posZ)*(currPR[2]-camerasF[9].posZ));
			camerasF[9].zoom=maxF(distCamPR/normDist,1.0);
			resConfigs[2]=camerasF[9].getConfig();
		}
	}
	
	return;
}
