#include "Functions.h"

// fonction detectant si collision:
/* renvoit 1 si collision avec obstacle,
			0 si pas de collision,
			-1 si collision avec station dans ancienne version ;
	ici simplement true ou false.. */
bool collisionBen(double* point) {
	// UTILISE PQP :
	
	// setup pqp model with one tiny triangle,
	// to use pqp for distance queries --> construction du pqp_model pr le point test
	double p[][3]={{point[0],point[1],point[2]},{point[0]+1e-8,point[1],point[2]},{point[0],point[1]+1e-8,point[2]}};
	PQP_Model* pointColl=new PQP_Model();
	pointColl->BeginModel();
	pointColl->AddTri(p[0],p[1],p[2],0);
	pointColl->EndModel();
	PQP_REAL rotateId[][3]={{1,0,0},{0,1,0},{0,0,1}}; // tranformation identite, car un seul point
	PQP_REAL position[3]={point[0],point[1],point[2]}; // la position du triangle

	PQP_CollideResult* result=new PQP_CollideResult();

	// boucle sur tous les obstacles sauf Terre :
	for (int i=1; i<obstaclesBen->num_obstacles(); i++) {
		PQP_Collide(result,rotateId,position,pointColl,
				obstaclesBen->Tr[i].R,obstaclesBen->Tr[i].T,obstaclesBen->pqp[i]);
		if (result->Colliding()) {
			delete pointColl;
			delete result;
			return true;
		}
	}

	delete pointColl;
	delete result;
	return false;
}

// verifie si 3 points sont colineaires :
bool colineaires(double* point1,double* point2,double* point3) {
	double csteProp;
	if (point2[0]-point1[0]!=0.0)
		csteProp=(point3[0]-point2[0])/(point2[0]-point1[0]);
	else if (point2[1]-point1[1]!=0.0)
		csteProp=(point3[1]-point2[1])/(point2[1]-point1[1]);
	else // au moins une coordonnee non nulle, donc..
		csteProp=(point3[2]-point2[2])/(point2[2]-point1[2]);

	for (int i=2; i>=0; i--)
		if (fabs(csteProp*(point2[i]-point1[i])-point3[i]-point2[i])>SEUIL_TOL) return false;
	return true;
}

// fonction calculant le point de repere courant suivant les parametres :
void getRepPoint(int indice) {
	if (handPoints[indice]) {
		for (int i=2; i>=0; i--) currPR[i]=coord3D[6+i];
	}
	else if (zoomObstPoints[indice]) {
		currPR[0]=(coordObstacles[indice][0]+coordPointsB[indice][0])/2;
		currPR[1]=(coordObstacles[indice][1]+coordPointsB[indice][1])/2;
		currPR[2]=(coordObstacles[indice][2]+coordPointsB[indice][2])/2;
	}
	else {
		currPR[0]=(COEFF_EP*((coord3D[0]+coord3D[3])/2)+COEFF_CO*((coord3D[3]+coord3D[6])/2)+COEFF_PO*coord3D[6])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
		currPR[1]=(COEFF_EP*((coord3D[1]+coord3D[4])/2)+COEFF_CO*((coord3D[4]+coord3D[7])/2)+COEFF_PO*coord3D[7])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
		currPR[2]=(COEFF_EP*((coord3D[2]+coord3D[5])/2)+COEFF_CO*((coord3D[5]+coord3D[8])/2)+COEFF_PO*coord3D[8])
					/(COEFF_EP+COEFF_CO+COEFF_PO);
	}

	return;
}

// renvoit la matrice de passage en fonction des angles phi et theta :
void getPassMatrix(double* coeffsX,double* coeffsY,double* coeffsZ,double anglePlan,double angleVert) {
	// premiere tranformation : rotation sur plan xOy
	double tmpSinP=sin(anglePlan),tmpCosP=cos(anglePlan);
	double tmpSinV=sin(angleVert),tmpCosV=cos(angleVert);
	coeffsZ[0]=tmpCosP;
	coeffsZ[1]=tmpSinP;
	// seconde transformation : elevation parallelement a l'axe Oz :
	// on obtient le vecteur (Oz)
	coeffsZ[2]=tmpCosV;
	coeffsZ[0]*=tmpSinV;
	coeffsZ[1]*=tmpSinV;

	// on en deduit les deux autres vecteurs de base
	// (le choix est tel que la base obtenue soit orthonormee DIRECTE)
	coeffsX[0]=tmpCosP*tmpCosV;
	coeffsX[1]=tmpSinP*tmpCosV;
	coeffsX[2]=-tmpSinV;
	coeffsY[0]=-tmpSinP;
	coeffsY[1]=tmpCosP;
	coeffsY[2]=0.0;

	return;
}

// fonction calculant les angles en coordonnees spheriques a partir d'un vecteur :
void getSphericAngles(double* vecteur,double& phi,double& theta) {
	// transformation en coordonnees spheriques pour obtenir phi et theta :
	
	// preliminaires :
	// SEUIL_TOL permet de passer outre les imprecisions dans les calculs ne donnant pas exactement 0
	if (fabs(vecteur[0])<SEUIL_TOL) vecteur[0]=0.0;
	if (fabs(vecteur[1])<SEUIL_TOL) vecteur[1]=0.0;
	if (fabs(vecteur[2])<SEUIL_TOL) vecteur[2]=0.0;
	
	// calcul :
	if (vecteur[0]!=0.0) {
		if (vecteur[1]!=0.0) {
			phi=atan(vecteur[1]/vecteur[0]);
			// attention phi n'est pas valide dans le cas ou l'on n'est pas dans la partie droite du cadran :
			if (vecteur[0]<0.0) {
				if (vecteur[1]<0.0) phi-=M_PI;
				else phi+=M_PI;
			}
		}
		else {
			if (vecteur[0]<0.0) phi=M_PI;
			else phi=0.0;
		}
	}
	else {
		if (vecteur[1]<0.0) phi=-M_PI_2;
		else phi=M_PI_2; // inclus le cas ou x=y=0 (alors angle arbitraire)
	}
	if (vecteur[2]!=0.0) {
		theta=atan((sqrt(vecteur[0]*vecteur[0]+vecteur[1]*vecteur[1]))/vecteur[2]);
		if (vecteur[2]<0.0) {
			// alors il faut encore ajouter pi pour avoir le resultat souhaite :
			theta+=M_PI;
		}
	}
	else theta=M_PI_2;
	
	return;
}

// fonction effectuant une rotation du point vecteur, d'angle angle autour de l'axe coeffsZ :
double* rotatePoint(double* coeffsX,double* coeffsY,double* coeffsZ,double angle,double* vecteur) {
	// matrice de la rotation dans base adaptee (M[app,e']) : 
	/*	(cos(angle) -sin(angle) 0 )
		(sin(angle) cos(angle)  0 )
		(     0         0       1 ) */
	// Donc :
	
	double* res=(double*)malloc(3*sizeof(double));
	// a) inv(P(e,e')) X == Interm :
	double* interm=(double*)malloc(3*sizeof(double));
	interm[0]=coeffsX[0]*vecteur[0]+coeffsX[1]*vecteur[1]+coeffsX[2]*vecteur[2];
	interm[1]=coeffsY[0]*vecteur[0]+coeffsY[1]*vecteur[1]+coeffsY[2]*vecteur[2];
	interm[2]=coeffsZ[0]*vecteur[0]+coeffsZ[1]*vecteur[1]+coeffsZ[2]*vecteur[2];
	// b) M[app,e'] Interm == Interm2
	double* interm2=(double*)malloc(3*sizeof(double));
	double cosAngle=cos(angle),sinAngle=sin(angle);
	interm2[0]=cosAngle*interm[0]-sinAngle*interm[1];
	interm2[1]=sinAngle*interm[0]+cosAngle*interm[1];
	interm2[2]=interm[2];
	free(interm);
	// c) P(e,e') Interm2 == resultat final (image du point)
	res[0]=coeffsX[0]*interm2[0]+coeffsY[0]*interm2[1]+coeffsZ[0]*interm2[2];
	res[1]=coeffsX[1]*interm2[0]+coeffsY[1]*interm2[1]+coeffsZ[1]*interm2[2];
	res[2]=coeffsX[2]*interm2[0]+coeffsY[2]*interm2[1]+coeffsZ[2]*interm2[2];
	free(interm2);

	return res;
}

// calcul de la distance de vue :
void getViewDistance(int indice) {
	double marge;
	if (idiomsMode) marge=MARGE_VUE_IDIOMS;
	else marge=MARGE_VUE;

	if (handPoints[indice]) {
		distCamPR=TAN_ANGLE*(CHAMP_POIGNET*marge)/2.0;
	}
	else if (zoomObstPoints[indice]) {
		distCamPR=TAN_ANGLE*2.0*distObstacles[indice]*marge; // 2.0 a ajuster......
	}
	else {
		// calcul de l'ecart maximal entre deux points du bras,
		// pour connaitre l'amplitude que l'on doit filmer :
		double maxEcart=sqrt((coord3D[6]-coord3D[0])*(coord3D[6]-coord3D[0])
							+ (coord3D[7]-coord3D[1])*(coord3D[7]-coord3D[1])
							+ (coord3D[8]-coord3D[2])*(coord3D[8]-coord3D[2]));
		if (LONG_EC>maxEcart) maxEcart=LONG_EC;
		if (LONG_CP>maxEcart) maxEcart=LONG_CP;
		// il faut voir maxEcart*marge, au moins : 
		distCamPR=TAN_ANGLE*(maxEcart*marge)/2.0;
	}
	
	return;
}

// transforme une configuration en triplet de points 3D :
double* conf2coord3D(const mpkConfig& config) {
	// code issu de RT :
	robotsBen->set_config(config); 
	robotsBen->compute_forward_kinematics();
	robotsBen->update_iv_transf();

	double* res=(double*)malloc(9*sizeof(double));
	double baseRep[]={0.0,0.0,0.0};
	// obtention des coordonnees de la base mobile (==epaule) :
	robotsBen->rob[0]->joints[0].Taccu.apply(res,baseRep);
	// puis du coude :
	robotsBen->rob[0]->joints[4].Taccu.apply(res+3,baseRep);
	// enfin, le poignet :
	robotsBen->rob[0]->joints[9].Taccu.apply(res+6,baseRep);

	return res;
}

// fonction renvoyant la visibilite par comparaison de 2 images
double visibilityCheckBen() {
	// etape 1 : on recupere l'image avec tous les obstacles :

	// en fait on se sert pas de VisibilityTest.cpp, on reimplemente ici le code adapte..


	// en particulier : 1) changer couleur des pixels, couleurObst==couleurFond et couleurBras.
	// 2) compter nbPixelsBras / nbPixelBras sur autre image
	// 3) hop...
/*
	render(0);
	unsigned char* buffer0 = renderer->getBuffer();


	char** images=visibiliteCheck->parseRootBen(mpkGUI->root2);


*/

	// etape 2 : l'image sans les obstacles, puis comparaison pixel par pixel..


	// NON IMPLEMENTE
	return 0.0;
}
