#include "GroupDensite.h"

// analyse un ensemble des points et les regroupe en valeurs successives,
// eliminant les outsiders efficacement (complexite lineaire).
vector<bool> analysePoints_Densite(const vector<bool>& points) {
	// on procede par evaluations de densite des sous-ensembles de points :

	// copie de l'argument : (l'instruction res=points est correcte mais trompeuse..)
	vector<bool> res(points.size());
	for (int i=points.size()-1; i>=0; i--) res[i]=points[i];

	bool kGroup,gTrue;
	int leftSide=0,taille,j;

	while (true) {

		// operation d'isolement : groupe (a1, .., an) isol si tous les points a1-i et an+i avec i<=n+1 sont a false
		// on repete l'operation tant que l'on trouve des groupes isoles.
		while (true) {
			kGroup=false;
			gTrue=false;
			for (int i=0; i<(int)points.size(); i++) {
				if (res[i]==true) { // on rencontre un groupe de valeurs "true"
					if (!gTrue) {
						gTrue=true;
						leftSide=i;
					}
				}
				else {
					if (gTrue) {
						taille=i-leftSide; // taille du groupe repere
						// on laisse ce groupe tranquille si sa taille est suffisante :
						if (taille<IM_SEC) {
							for (j=maxi(0,leftSide-taille-1); j<leftSide; j++)
								if (res[j]) break;
							if (j==leftSide) {
								// rien a gauche..
								// on continue voir s'il n'y a pas des valeurs true a droite :
								int borneRight=mini((int)points.size(),i+taille);
								for (j=i; j<=borneRight; j++)
									if (res[j]) break;
								if (j>borneRight) {
									// rien a droite non plus, donc..
									// on supprime le groupe :
									for (int k=leftSide; k<i; k++) res[k]=false;
									kGroup=true;
								}
							}
						}
						gTrue=false;
					}
				}
			}
			
			// traitement du dernier groupe dans le cas ou on finit sur des valeurs true :
			if (gTrue) {
				taille=(int)points.size()-leftSide; // taille du groupe repere
				// on laisse ce groupe tranquille si sa taille est suffisante :
				if (taille<IM_SEC) {
					for (j=maxi(0,leftSide-taille-1); j<leftSide; j++)
						if (res[j]) break;
					if (j==leftSide) {
						// rien a gauche.. donc on supprime car on sait deja qu'il n'y a rien a droite non plus (!) :
						for (int k=leftSide; k<(int)points.size(); k++) res[k]=false;
						kGroup=true;
					}
				}
			}
			
			if (!kGroup) break;
		}
		
		// puis de fusion : on relie les groupes non isoles entre eux, puis on recommence l'operation precedente,
		// jusqu'a ne plus trouver de groupes de taille<IM_SEC;
		// deux cas : fusion de petits groupe sur la base de la taille du groupe,
		// et fusion de grands groupes sur la base de IM_SEC :
		gTrue=false;
		for (int i=0; i<(int)points.size(); i++) {
			if (res[i]==true) { // on rencontre un groupe de valeurs "true"
				if (!gTrue) {
					gTrue=true;
					leftSide=i;
				}
			}
			else {
				if (gTrue) {
					taille=i-leftSide; // taille du groupe repere
					if (taille>IM_SEC) taille=IM_SEC; // pour ne pas trop fusionner (!)
					// fusion a gauche :
					int borneLeft=maxi(0,leftSide-taille-1);
					for (j=leftSide-1; j>=borneLeft; j--) {
						if (res[j]) {
							// alors on relie
							for (int k=j+1; k<leftSide; k++) res[k]=true;
							break;
						}
					}
					// fusion a droite :
					int borneRight=mini((int)points.size(),i+taille);
					for (j=i; j<=borneRight; j++) {
						if (res[j]) {
							// alors on relie
							for (int k=i; k<j; k++) res[k]=true;
							break;
						}
					}
					gTrue=false;
				}
			}
		}
		
		// traitement du dernier groupe dans le cas ou on finit sur des valeurs true :
		if (gTrue) {
			taille=(int)points.size()-leftSide; // taille du groupe repere
			if (taille>IM_SEC) taille=IM_SEC; // pour ne pas trop fusionner (!)
			// fusion a gauche :
			int borneLeft=maxi(0,leftSide-taille-1);
			for (j=leftSide-1; j>=borneLeft; j--) {
				if (res[j]) {
					// alors on relie
					for (int k=j+1; k<leftSide; k++) res[k]=true;
					break;
				}
			}
		}
		
		// recherche si un groupe est de taille<IM_SEC apres fusion :
		int minTaille=IM_SEC;
		gTrue=false;
		for (int i=0; i<(int)points.size(); i++) {
			if (res[i]==true) { // on rencontre un groupe de valeurs "true"
				if (!gTrue) {
					gTrue=true;
					leftSide=i;
				}
			}
			else {
				if (gTrue) {
					taille=i-leftSide; // taille du groupe repere
					if (taille<minTaille) {
						minTaille=taille;
						break;
					}
					gTrue=false;
				}
			}
		}
		
		// ..enfin le dernier groupe eventuellement si l'on finit par des valeurs true :
		if (gTrue) {
			taille=(int)points.size()-leftSide; // taille du groupe repere
			if (taille<minTaille) minTaille=taille;
		}

		// cas d'arret : tous les groupes sont de taille suffisante ou ont disparu :
		if (minTaille>=IM_SEC) break;

	}

	return res;
}
