#include "visibilitytest.h"

CVisibilityTest::CVisibilityTest(int largeur, int hauteur)
:initialise(false)
{
	debugListBox = NULL;
	init(largeur,hauteur);
	obstaclesPresents = NULL;
}

CVisibilityTest::~CVisibilityTest()
{
	initialise = false;
	if (obstaclesPresents) delete obstaclesPresents;
	// Il faudrait faire la destruction ici...

	// tentative (?) 
	/*
	for (int i=listescene.size()-1; i>=0; i--) delete listescene[i];
	listescene.clear();
	for (int i=listeswitch.size()-1; i>=0; i--) delete listeswitch[i];
	listeswitch.clear(); */
	if (renderer!=NULL) delete renderer;
}

int CVisibilityTest::addScene(SoSeparator* sep)
{
	listescene.push_back(sep);
	return listescene.size()-1;
}

void CVisibilityTest::init(int largeur, int hauteur)
{
    vpr.setWindowSize(largeur,hauteur);

	renderer = new SoOffscreenRenderer(vpr);
    renderer->setBackgroundColor(SbColor(0.0,0.0,0.0));

	initialise = true;
}

bool CVisibilityTest::isInit()
{
	return initialise;
}

void CVisibilityTest::render(int numview)
{
	SetSwitchState(1);
	renderer->render(listescene[numview]);
	SetSwitchState(0);
}

int CVisibilityTest::debugWriteToRGB(char * fichier)
{
	return renderer->writeToRGB(fichier);
}

int CVisibilityTest::ParseRoot(SoSeparator* sep)
{
	ObstacleId oid;
	int i;
	SoSearchAction searchobst;
	searchobst.setType(mpkObstacle::getClassTypeId());
	searchobst.setInterest(SoSearchAction::ALL);
	searchobst.setSearchingAll(TRUE);
	searchobst.apply(sep);
	SoPathList obstpathlist = searchobst.getPaths();
	i = obstpathlist.getLength();

	if (obstaclesPresents) delete obstaclesPresents;
	ViderListeObst();
	for (i=0;i<obstpathlist.getLength();i++)
	{
		oid.obstacle = (mpkObstacle*)obstpathlist[i]->getTail();
		oid.obstacle->ref();
		oid.numcol = i+1;		// 0 est la couleur de fond. -> ne pas l'assigner.
		listeobst.push_back(oid);
		SetColorSwitches((SoNode*)oid.obstacle,oid.numcol,&listeswitch,SoNode::getClassTypeId());
	}

	// Patch pour le robot, qui n'est pas un mpkobstacle...
	// Ce n'est pas tres beau mais ca marche.
	oid.numcol = i+1;
	oid.obstacle = new mpkObstacle();
	oid.obstacle->setName(SbName("Robot"));
	oid.obstacle->ref();
	listeobst.push_back(oid);
	SetColorSwitches(sep,oid.numcol,&listeswitch,SoSeparator::getClassTypeId());

	obstaclesPresents = new const char*[listeobst.size()+1];
	return 0;
}

void CVisibilityTest::ViderListeObst()
{
	for (int i=0;i<listeobst.size();i++) listeobst[i].obstacle->unref();
	listeobst.clear();
}

char** CVisibilityTest::VisibilityCheck()
{
	short maxx, maxy;
	int x,y;
	float proportion;
	char strbuf[250];
	vpr.getWindowSize().getValue(maxx,maxy);
	RtlZeroMemory(tableResultats,256*sizeof(int));
	unsigned char* buffer = renderer->getBuffer();

	// Compter le nombre de pixels de la meme couleur
	for (x=0;x<maxx*maxy;x++) tableResultats[buffer[x*3]]++;

	// Effacer la liste
	if (debugListBox) debugSendToListBox(NULL);

	// Recuperer les noms de tous les obstacles presents dans l'image
	y=0;
	for(x=1;x<listeobst.size()+1;x++) if (tableResultats[x] > 0)
	{
		obstaclesPresents[y]=getObstacleByColor(x)->getName().getString();
		proportion = (float)tableResultats[x]/(float)(maxx*maxy)*100.0;
		sprintf(strbuf,"%s, %3.1f%%",obstaclesPresents[y],proportion);
		if (debugListBox) debugSendToListBox(strbuf);
		y++;
	}
	obstaclesPresents[y] = NULL; // Marquer la fin de la chaine de char*

	return 0;
}

// Creer une listbox. Si un parametre non nul est fourni,
// La fenetre ne sera pas cree. Dans ce cas, "listbox" devra
// correspondre a une listbox cree par le programme appelant.
void CVisibilityTest::debugCreateListBox(HWND listbox)
{
	debugListBox = listbox ? listbox:CreateWindowEx(WS_EX_TOPMOST | WS_THICKFRAME, "LISTBOX", "Objets visibles", WS_VISIBLE, 500, 600, 250, 350, NULL, (HMENU)NULL, NULL, 0);
	ShowWindow(debugListBox,SW_SHOWNORMAL);
}

// Detruire la lisbox cree par la classe.
void CVisibilityTest::debugDestroyListBox()
{
	DestroyWindow(debugListBox);
	debugListBox = NULL;
}

// Ajoute une entree a la liste.
// Si str == NULL, la liste sera videe
void CVisibilityTest::debugSendToListBox(const char* str)
{
	if (str) SendMessage(debugListBox,LB_ADDSTRING,-1,(LPARAM)str);
	else SendMessage(debugListBox,LB_RESETCONTENT,0,0);
}

// Fonction pas tres efficace... A revoir
mpkObstacle* CVisibilityTest::getObstacleByColor(int col)
{
	int i;
	for (i=0;i<listeobst.size();i++)
	{
		if (col == listeobst[i].numcol) return listeobst[i].obstacle;
	}
	return NULL;
}

// Cette fonction remplace les SoMaterial par des SoSwitch contenant 2
// SoMaterial: l'original (position 0) et le nouveau (position 1)
int CVisibilityTest::SetColorSwitches(SoNode * sep, int col, vector<SoSwitch*>* liste,SoType &arret)
{
	int i;
	SoChildList *cl;
	SoNode *ntemp;
	SoSwitch *sw;
	SoMaterial *mat;
	float fcolor = (float)col/(float)255;

	cl=sep->getChildren();
	if (cl) for (i=0;i<cl->getLength();i++)
	{
		ntemp = (*cl)[i];
		if (ntemp->isOfType(SoMaterial::getClassTypeId()))
		{
			sw = new SoSwitch;
			mat = new SoMaterial;
			mat->diffuseColor.setValue(0.0,0.0,0.0);
			mat->ambientColor.setValue(0.0,0.0,0.0);
			mat->emissiveColor.setValue(fcolor,0.0,0.0);
			mat->specularColor.setValue(0.0,0.0,0.0);
			mat->shininess.setValue(0.0);
			sw->addChild(ntemp);
			sw->addChild(mat);
			cl->insert(sw,i);
			cl->remove(i+1);
			liste->push_back(sw);
			return 1;
		}
		else if (ntemp->isOfType(arret)) SetColorSwitches(ntemp,col,liste,arret);
	}
	return 0;
}

// Decider si on affiche la couleur originale (0) ou la couleur speciale (1)
void CVisibilityTest::SetSwitchState(int state)
{
	int i;
	for (i=0;i<listeswitch.size();i++)
	{
		listeswitch[i]->whichChild = state;
	}

}
