/*************************************************************
Test de visibilite des obstacles.

Cette classe cree un bitmap dans un buffer offscreen pour savoir
quels objets sont reellement visibles. Utilisation:

1) ParseRoot(sep)
Ou "sep" est un root contenant tous les obstacles.
Cette fonction recherche tous les objets de type mpkObstacle et leur
associe un SoMaterial unique pour le rendering offscreen.

2) addScene(sep)
Ou sep est une scene contenant les memes obstacles que ParseRoot a
recus, et contenant aussi la camera active pour cette scene.
On peut appeler addScene pour plusieurs scenes ayant des cameras differentes, tant
qu'elles contiennent toutes les memes obstacles.
addScene() retourne le numero assigne a la scene.

3) render() & VisibilityCheck()
Render cree le bitmap.
VisibilityCheck retourne un tableau de char* contenant les noms des objets visibles. Le tableau
est termine par NULL.

Cette classe a ete implemente comme une classe 'a tout faire' pour qu'elle soit
facile a integrer aux differentes versions du tuteur.

//--------------------------------------------------
//Exemple:
	case IDC_TESTVINCENT:
		if (!visib)
		{
			visib = new CVisibilityTest(100,130);
			visib->debugCreateListBox();
			visib->ParseRoot(mpkGUI::root1);
			visib->addScene(mpkGUI::root1);
			visib->addScene(mpkGUI::root2);
			visib->addScene(mpkGUI::root3);
		}
		visib->render(0);
		//visib->debugWriteToRGB("c:\\test.rgb");
		visib->VisibilityCheck();
		break;
//--------------------------------------------------

A corrier/modifier:
* destruction de la classe
* ParseRoot laisse les SoSwitch dans l'arbre. Plusieurs appels de cette fonction transformeraient
  l'arbre en gros spaghetti. Il faudrait faire une fonction pour detruire ces switches et
  les couleurs speciales.

Vincent Gournay - 27/06/2006
/************************************************************/
#pragma once
#include "c:\Coin3d\include\inventor\nodes\soseparator.h"
#include "c:\Coin3d\include\inventor\sbviewportregion.h"
#include "c:\Coin3d\include\inventor\sooffscreenrenderer.h"
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoMaterial.h>
#include "mpkObstacle.H"
#include "C:\Coin3D\include\Inventor\misc\SoChildList.h"
#include "mpkGUI.H"

#include <vector>
using std::vector;

class CVisibilityTest
{
public:
	CVisibilityTest() {}
	CVisibilityTest(int largeur, int hauteur);
	~CVisibilityTest(void);
	int addScene(SoSeparator* sep);
	int ParseRoot(SoSeparator* sep);
	bool isInit(void);
	void render(int numview);
	char** VisibilityCheck(void);

	/*****************************************************************************
	* @progBenji : fonction renvoyant la visibilite par comparaison de 2 images */
	double visibilityCheckBen();
	/****************************************************************************/

	int debugWriteToRGB(char * fichier);
	void debugCreateListBox(HWND parent=NULL);
	void debugDestroyListBox();

protected:

	struct ObstacleId
	{
		int numcol;
		mpkObstacle * obstacle;
	};

	void debugSendToListBox(const char* str);
	void init(int largeur, int hauteur);
	void ViderListeObst(void);
	mpkObstacle* getObstacleByColor(int col);
	int SetColorSwitches(SoNode * sep, int col, vector<SoSwitch*>* liste, SoType &arret);
	void SetSwitchState(int state);


	const char **obstaclesPresents;
	vector<SoSeparator*> listescene;
	vector<ObstacleId> listeobst;
	vector<SoSwitch*> listeswitch;
	bool initialise;
	SoOffscreenRenderer *renderer;
	SbViewportRegion vpr;
	HWND debugListBox;
	int tableResultats[256];	// 256 pour les 256 teintes de couleur.

};
