#include "camera.h"

void camera::set_camParam(SoPerspectiveCamera* cam, float xPos, float yPos, float zPos, float xOr, float yOr, float zOr, float angle,
				 float aR, float nD, float faD, float foD, float hA)
{
  cam->position.setValue(SbVec3f(xPos, yPos, zPos));
  cam->orientation.setValue(SbRotation(SbVec3f(xOr, yOr, zOr), angle));
  cam->aspectRatio.setValue(aR);
  cam->nearDistance.setValue(nD); 
  cam->farDistance.setValue(faD); 
  cam->focalDistance.setValue(foD); 
  cam->heightAngle.setValue(hA); 
}

void camera::which_cam(int fenetre, float xPos, float yPos, float zPos, float xOr, float yOr, float zOr, float angle,
			   float aR, float nD, float faD, float foD, float hA){
				 switch(fenetre){
					 case 1:
						 set_camParam(mpkGUI::rootCamera1, xPos, yPos, zPos, xOr, yOr, zOr, angle, aR, nD, faD, foD, hA);
						 break;
					 case 2:
						 set_camParam(mpkGUI::rootCamera2, xPos, yPos, zPos, xOr, yOr, zOr, angle, aR, nD, faD, foD, hA);
						 break;
					 case 3:
						 set_camParam(mpkGUI::rootCamera3, xPos, yPos, zPos, xOr, yOr, zOr, angle, aR, nD, faD, foD, hA);
						 break;
					 case 4:
						 set_camParam(mpkGUI::rootCameraSpatial, xPos, yPos, zPos, xOr, yOr, zOr, angle, aR, nD, faD, foD, hA);
						 break;
					 default:
						 printf("aucune fenetre active\n");
				 }
}

void camera::get_camera(SoPerspectiveCamera* cam, float xP, float yP, float zP, float xO, float yO, float zO, float angle){
  SbVec3f pos, axis; 
  SbRotation or; 
  
  pos = cam->position.getValue();
  pos.getValue(xP,yP,zP);
  
  or = cam->orientation.getValue(); 
  or.getValue(axis, angle); 
  axis.getValue(xO,yO,zO);

}

void camera::print_camera(SoPerspectiveCamera* cam, char* fname)
{
  SbVec3f pos, axis; 
  float xP, yP, zP, xO, yO, zO, angle, aspectRat,nearDist, farDist, focalDist, heightAng; 
  SbRotation or; 
  
  pos = cam->position.getValue();
  pos.getValue(xP,yP,zP);
  
  or = cam->orientation.getValue(); 
  or.getValue(axis, angle); 
  axis.getValue(xO,yO,zO);
      
  aspectRat = cam->aspectRatio.getValue();
  nearDist = cam->nearDistance.getValue(); 
  farDist = cam->farDistance.getValue(); 
  focalDist = cam->focalDistance.getValue(); 
  heightAng = cam->heightAngle.getValue(); 

  ofstream f(fname);
  if (!f) return;
  f << xP << ",";
  f << yP << ",";
  f << zP << ","; 
  f << xO << ",";
  f << yO << ",";
  f << zO << ",";
  f<< angle << ","; 
  f<< aspectRat << ","; 
  f<< nearDist << ","; 
  f<< farDist << ","; 
  f<< focalDist <<","; 
  f<< heightAng << "\n"; 
  f.close();

}

void camera::print_camera(SoPerspectiveCamera* cam)
{
  SbVec3f pos, axis; 
  float xP, yP, zP, xO, yO, zO, angle, aspectRat,nearDist, farDist, focalDist, heightAng; 
  SbRotation or; 
  
  pos = cam->position.getValue();
  pos.getValue(xP,yP,zP);
  
  or = cam->orientation.getValue(); 
  or.getValue(axis, angle); 
  axis.getValue(xO,yO,zO);
      
  aspectRat = cam->aspectRatio.getValue();
  nearDist = cam->nearDistance.getValue(); 
  farDist = cam->farDistance.getValue(); 
  focalDist = cam->focalDistance.getValue(); 
  heightAng = cam->heightAngle.getValue();

  printf("    Position:       (%f, %f, %f)\n", xP, yP, zP); 
  printf("    Orientation:    ((%f, %f, %f), %f)\n", xO, yO, zO, angle); 
  printf("    Aspect Ratio:   %f\n", aspectRat); 
  printf("    Near Distance:  %f\n", nearDist); 
  printf("    Far Distance:   %f\n", farDist); 
  printf("    Focal Distance: %f\n", focalDist); 
  printf("    Height Angle:   %f\n", heightAng);  
}

void camera::set_cam(SoPerspectiveCamera* cam, char *fname){


	SbVec3f pos, axis; 
    float xP, yP, zP, xO, yO, zO, angle, aspectRat,nearDist, farDist, focalDist, heightAng; 
    SbRotation or; 
  
    pos = cam->position.getValue();
    pos.getValue(xP,yP,zP);
  
    or = cam->orientation.getValue(); 
    or.getValue(axis, angle); 
    axis.getValue(xO,yO,zO);

	aspectRat = cam->aspectRatio.getValue();
	nearDist = cam->nearDistance.getValue(); 
	farDist = cam->farDistance.getValue(); 
	focalDist = cam->focalDistance.getValue(); 
	heightAng = cam->heightAngle.getValue(); 

	ifstream Fichier (fname, ios::in ); 
	if(!Fichier){ 
	cerr<<"Cannot Open File "<< fname <<endl; 
	return; 
	} 
	int i=0;
	while (!Fichier.eof()){ 
        Fichier>> xP;
        Fichier>> yP;
        Fichier>> zP;
        Fichier>> xO;
		Fichier>> yO;
		Fichier>> zO;
		Fichier>> angle;
		Fichier>> aspectRat; 
		Fichier>> nearDist; 
		Fichier>> farDist; 
		Fichier>> focalDist; 
		Fichier>> heightAng;
   }
	set_camParam(cam, xP, yP, zP, xO, yO, zO, angle, aspectRat, nearDist, farDist, focalDist, heightAng);
	Fichier.close();

}
