#ifndef MPK_CONFIG_CHECKER_H
#define MPK_CONFIG_CHECKER_H

#include <queue>
#include "mpkConfig.H"
#include "mpkCollPair.H"
#include "mpkRobotCollection.H"
#include "mpkCollDistAlgo.H"

/**************************
* @progBenji : include.. */
#include "Constantes.h"
/*************************/

///
class mpkConfigChecker {

public:

  ///
  mpkConfigChecker( vector<mpkCollPair>* test_pairs,
		   mpkRobotCollection* robots);
  
  /// tells if the configuration q is colliding
  bool collision( mpkConfig* q, mpk_idx model_id=0);

  /** returns lower bound on clearance (0 if colliding or separation <
      delta for some pair) */
  bool clearance(const mpkConfig* q, mpk_idx model_id=0,
		   bool store_dist=true);

  /*********************************************************************************************************
  * @progBenji : fonction legerement modifiee de la precedente, pour calculer les distances aux obstacles */
  PQP_DistanceResult* distanceObstacles(mpkConfig* q, mpk_idx model_id=0);
  /********************************************************************************************************/

  /** returns 0 for clear, 1 for proximity and 2 for collision */
  int status(const mpkConfig* q, mpk_idx model_id=0,
		   bool store_dist=true);

  /** the index (in test_pairs) of the colliding (or too
      close) pair (undef if no collision). */
  mpk_idx collpair_idx() {return coll_idx;};
  mpk_idx proxpair_idx() {return prox_idx;};
  double proxpair_distance() {return prox_distance;};

  static int num_tri_tests;
  static int num_bv_tests;
  static int num_obj_tests;
  //vector<mpkCollPair> tooclose_pairs;
private:
  
  // general information
   mpkRobotCollection* robots;
   vector<mpkCollPair>* test_pairs;
   vector<mpkCollPair>* prox_pairs;


  // status corresponding to most recent operation
  mpk_idx model_id;
  const mpkConfig* q;
  vector<double> lbdist; // lower distance bounds (after clearance() only)
  double min_sep;  // minimum of lbdist (after clearance() only)
  mpk_idx coll_idx; // only valid if collision / too close
  mpk_idx prox_idx; 
  double prox_distance;
  //
  friend class mpkSimpleSegmentChecker;
  friend class mpkDiscrSegmentChecker;
  friend class mpkAdaptSegmentChecker;

};

#endif
