#include "mpkConfigChecker.H"

#define CHECK_SPHERES_FIRST

int mpkConfigChecker::num_tri_tests;
int mpkConfigChecker::num_bv_tests;
int mpkConfigChecker::num_obj_tests;

mpkConfigChecker::mpkConfigChecker(vector<mpkCollPair>* test_pairs,
		  mpkRobotCollection* robots)
{
  this->robots = robots;
  this->test_pairs = test_pairs;
   this->prox_pairs = test_pairs;
  this->q = 0;
  this->model_id = 0;
}

bool mpkConfigChecker::collision(mpkConfig* q, mpk_idx model_id)
{
  assert(model_id==0 || model_id==1);

  num_tri_tests = 0;
  num_bv_tests = 0;
  num_obj_tests = 0;
  this->model_id = model_id;
  this->q = 0;
  this->min_sep = -1; // disable: collision test does not determine min_sep

  robots->set_config(*q);
  robots->compute_forward_kinematics();

  for ( int i=0; i < test_pairs->size(); i++ ) {

    num_obj_tests++;

    const mpkCollPair& curr_pair = (*test_pairs)[i]; 

#ifdef CHECK_SPHERES_FIRST
    // perform simple sphere overlap test first for top-level bounding spheres
    if ( curr_pair.o2.rob ) {
      num_bv_tests++;
      double c1[3], c2[3];
      mpkBaseRobot::Joint& j1 =curr_pair.o1.rob->joints[curr_pair.o1.joint_idx];
      mpkBaseRobot::Joint& j2 =curr_pair.o2.rob->joints[curr_pair.o2.joint_idx];
      curr_pair.o1.Tr->apply(c1, j1.sphere_c);
      curr_pair.o2.Tr->apply(c2, j2.sphere_c);
      if (mpkCollDistAlgo::SphereDist(c1, c2,j1.sphere_r, j2.sphere_r) > 0)
	continue; // spheres do not overlap --> continue with next pair
    }
#endif

    bool coll = mpkCollDistAlgo::Collision(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
				 curr_pair.o1.pqp[model_id], 
				 curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
				 curr_pair.o2.pqp[model_id]);

    num_tri_tests += mpkCollDistAlgo::coll_res.num_tri_tests;
    num_bv_tests  += mpkCollDistAlgo::coll_res.num_bv_tests;

    if ( coll ) {
      coll_idx = i;
      return true;
    }

  }
  return false;
}

bool mpkConfigChecker::clearance(const mpkConfig* q, mpk_idx model_id, bool store_dist)
{
  assert(model_id==0 || model_id==1);

  num_tri_tests = 0;
  num_bv_tests = 0;
  num_obj_tests = 0;
  this->model_id = model_id;

  //this->tooclose_pairs.clear();   // initialiser les tooclose_pairs.

  this->q = q;
  this->min_sep = 10;    // distance a garder avec les zones

  if ( store_dist && lbdist.size() != prox_pairs->size() )
    lbdist.resize(prox_pairs->size());

  robots->set_config(*q);
  robots->compute_forward_kinematics();
  

  for ( int i=0; i < prox_pairs->size(); i++ ) {

    num_obj_tests++;

    const mpkCollPair& curr_pair = (*prox_pairs)[i]; 

    double dist ;

//#ifdef CHECK_SPHERES_FIRST
    // perform simple sphere distance computation first but only if we
    // do not store distances (greedy lower bounds from spheres tend
    // to be very bad) and only for pairs of two robot links (spheres
    // of obstacles and robot links are most likely overlapping)
  //  if ( !store_dist && curr_pair.o2.rob ) {
    //  double c1[3], c2[3];
  //    mpkBaseRobot::Joint& j1 =curr_pair.o1.rob->joints[curr_pair.o1.joint_idx];
  //    mpkBaseRobot::Joint& j2 =curr_pair.o2.rob->joints[curr_pair.o2.joint_idx];
  //    curr_pair.o1.Tr->apply(c1, j1.sphere_c);
  //    curr_pair.o2.Tr->apply(c2, j2.sphere_c);
  //    dist =mpkCollDistAlgo::SphereDist(c1, c2, j1.sphere_r, j2.sphere_r)- curr_pair.delta;
  //  }
//#endif

  //  if ( dist <= 0 ) {
      dist =
		  mpkCollDistAlgo::Distance(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
					curr_pair.o1.pqp[model_id], 
					curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
					curr_pair.o2.pqp[model_id],
					curr_pair.delta);
      num_tri_tests += mpkCollDistAlgo::dist_res.num_tri_tests;
      num_bv_tests  += mpkCollDistAlgo::dist_res.num_bv_tests;
   // }
    
    if ( dist <= min_sep ) {
      prox_idx = i;
	  prox_distance = dist;
	  // remplir les tooclose_pairs (avec les zones)
	  //this->tooclose_pairs.push_back(curr_pair);  
     //   cout << "("
	 //<< curr_pair.o1.name 
	 //<< ","
	 //<< curr_pair.o2.name 
	 //<< ")" << endl;
    return true;
	}


  }

  return false;
}

	/*********************************************************************************************************
	* @progBenji : fonction legerement modifiee de la precedente, pour calculer les distances aux obstacles */
	PQP_DistanceResult* mpkConfigChecker::distanceObstacles(mpkConfig* q, mpk_idx model_id) {
		num_tri_tests = 0;
		num_bv_tests = 0;
		num_obj_tests = 0;
		this->model_id = model_id;
		this->q = q;
		PQP_DistanceResult* result;
		double minDist=MAX_DIST;

		robots->set_config(*q);
		robots->compute_forward_kinematics();

		for ( int i=0; i < prox_pairs->size(); i++ ) {

			num_obj_tests++;

			const mpkCollPair& curr_pair = (*prox_pairs)[i]; 

			PQP_DistanceResult* resultDistPoints=
				mpkCollDistAlgo::getDistancePoints(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
													curr_pair.o1.pqp[model_id], 
													curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
													curr_pair.o2.pqp[model_id],
													curr_pair.delta);
				num_tri_tests += mpkCollDistAlgo::dist_res.num_tri_tests;
				num_bv_tests  += mpkCollDistAlgo::dist_res.num_bv_tests;

			if ( resultDistPoints->Distance() < minDist ) {
				minDist=resultDistPoints->Distance();
				result=resultDistPoints;
			}
			else delete resultDistPoints;
		}

		return result;
	}
	/********************************************************************************************************/

// Idea to calculate collisions and proximity the same time ... doesnt improve performance,
// therefor abandonned.
int mpkConfigChecker::status(const mpkConfig* q, mpk_idx model_id, bool store_dist)
{
  assert(model_id==0 || model_id==1);

  num_tri_tests = 0;
  num_bv_tests = 0;
  num_obj_tests = 0;
  this->model_id = model_id;

  //this->tooclose_pairs.clear();   // initialiser les tooclose_pairs.

  this->q = q;
  this->min_sep = 10;    // distance a garder avec les zones

  if ( store_dist && lbdist.size() != prox_pairs->size() )
    lbdist.resize(prox_pairs->size());

  robots->set_config(*q);
  robots->compute_forward_kinematics();
  
  int worst = 0;

  for ( int i=0; i < prox_pairs->size(); i++ ) {

    num_obj_tests++;

    const mpkCollPair& curr_pair = (*prox_pairs)[i]; 

    double dist ;

  //  if ( dist <= 0 ) {
      dist =
		  mpkCollDistAlgo::Distance(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
					curr_pair.o1.pqp[model_id], 
					curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
					curr_pair.o2.pqp[model_id],
					curr_pair.delta);
      num_tri_tests += mpkCollDistAlgo::dist_res.num_tri_tests;
      num_bv_tests  += mpkCollDistAlgo::dist_res.num_bv_tests;
   // }
    
	  if ( dist <= 0 ) {
		  coll_idx = i;
		  return 2;
	  }

    if (( worst == 0 ) && (dist <= min_sep )) {
      prox_idx = i;
	  prox_distance = dist;
	  worst = 1;
	}


  }

  return worst;
}
