#ifndef MPK_ZONE_CHECKER_H
#define MPK_ZONE_CHECKER_H

#include <queue>
#include "mpkConfig.H"
#include "mpkCollPair.H"
#include "mpkRobotCollection.H"
#include "mpkZoneCollection.H"
#include "mpkCollDistAlgo.H"

///
class mpkZoneChecker {

public:

  ///
  mpkZoneChecker( vector<mpkCollPair>* test_pairs,
		   mpkRobotCollection* robots, mpkZoneCollection* zones);
  
  /// tells if the configuration q is colliding
  bool collision( mpkConfig* q, mpk_idx model_id=0);

  /** returns lower bound on clearance (0 if colliding or separation <
      delta for some pair) */
  double get_dd(const mpkConfig* q, mpk_idx model_id=0);

  /** the index (in test_pairs) of the colliding (or too
      close) pair (undef if no collision). */
  mpk_idx collpair_idx() {return coll_idx;};

    static int num_tri_tests;
    static int num_bv_tests;
    static int num_obj_tests;
  //vector<mpkCollPair> tooclose_pairs;

   vector<double> membership;
  double dd; // the degree of desirability of the config


private:
  
  // general information
   mpkRobotCollection* robots;
   mpkZoneCollection* zones;
   vector<mpkCollPair>* test_pairs;


  // status corresponding to most recent operation
  mpk_idx model_id;
  const mpkConfig* q;
  vector<double> lbdist; // lower distance bounds (after clearance() only)
  double dmax;  // minimum of lbdist (after clearance() only)
  mpk_idx coll_idx; // only valid if collision / too close
  



  //
  friend class mpkSimpleSegmentChecker;
  friend class mpkDiscrSegmentChecker;
  friend class mpkAdaptSegmentChecker;

};

#endif
