#include "mpkZoneChecker.H"

#define CHECK_SPHERES_FIRST

int mpkZoneChecker::num_tri_tests;

int mpkZoneChecker::num_bv_tests;
int mpkZoneChecker::num_obj_tests;

mpkZoneChecker::mpkZoneChecker(vector<mpkCollPair>* test_pairs,
		  mpkRobotCollection* robots, mpkZoneCollection* zones)
{
  this->robots = robots;
  this->zones = zones;
  this->test_pairs = test_pairs;
  this->q = 0;
  this->dd = 0;
  this->model_id = 0;
  this->membership.clear();
}

bool mpkZoneChecker::collision(mpkConfig* q, mpk_idx model_id)
{
  assert(model_id==0 || model_id==1);

  num_tri_tests = 0;
  num_bv_tests = 0;
  num_obj_tests = 0;
  this->model_id = model_id;
  this->q = 0;
  this->dmax = -1; // disable: collision test does not determine min_sep

  robots->set_config(*q);
  robots->compute_forward_kinematics();

  for ( int i=0; i < test_pairs->size(); i++ ) {

    num_obj_tests++;

    const mpkCollPair& curr_pair = (*test_pairs)[i]; 

#ifdef CHECK_SPHERES_FIRST
    // perform simple sphere overlap test first for top-level bounding spheres
    if ( curr_pair.o2.rob ) {
      num_bv_tests++;
      double c1[3], c2[3];
      mpkBaseRobot::Joint& j1 =curr_pair.o1.rob->joints[curr_pair.o1.joint_idx];
      mpkBaseRobot::Joint& j2 =curr_pair.o2.rob->joints[curr_pair.o2.joint_idx];
      curr_pair.o1.Tr->apply(c1, j1.sphere_c);
      curr_pair.o2.Tr->apply(c2, j2.sphere_c);
      if (mpkCollDistAlgo::SphereDist(c1, c2,j1.sphere_r, j2.sphere_r) > 0)
	continue; // spheres do not overlap --> continue with next pair
    }
#endif

    bool coll = mpkCollDistAlgo::Collision(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
				 curr_pair.o1.pqp[model_id], 
				 curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
				 curr_pair.o2.pqp[model_id]);

    num_tri_tests += mpkCollDistAlgo::coll_res.num_tri_tests;
    num_bv_tests  += mpkCollDistAlgo::coll_res.num_bv_tests;

    if ( coll ) {
      coll_idx = i;
      return true;
    }

  }
  return false;
}

double mpkZoneChecker::get_dd(const mpkConfig* q, mpk_idx model_id)
{
  assert(model_id==0 || model_id==1);

  num_tri_tests = 0;
  num_bv_tests = 0;
  num_obj_tests = 0;
//  this->model_id = model_id;
  this->dd = 0;

  //this->tooclose_pairs.clear();   // initialiser les tooclose_pairs.

  this->q = q;
  this->dmax = 150;    // Etendu de la Zone

//  if ( store_dist && lbdist.size() != test_pairs->size() )
   // lbdist.resize(test_pairs->size());

  robots->set_config(*q);
  robots->compute_forward_kinematics();
  membership.clear(); 
  
  double val;  

  
for ( int k=0; k<zones->num_zones(); k++ )
{	
  int kk=0;

  for ( int i=0; i < test_pairs->size(); i++ ) {

//cout << zones->names[k];
//cout << (*test_pairs)[i].o2.name<<endl;


   if(strcoll(zones->names[k],(*test_pairs)[i].o2.name)==0)
	  {
     //num_obj_tests++;

    const mpkCollPair& curr_pair = (*test_pairs)[i]; 
		
	val	= (*test_pairs)[i].o2.desirability;
	//cout << val<<endl;

    double dist;

      dist =
	mpkCollDistAlgo::GreedyDistance(curr_pair.o1.Tr->R, curr_pair.o1.Tr->T,
					curr_pair.o1.pqp[model_id], 
					curr_pair.o2.Tr->R, curr_pair.o2.Tr->T,
					curr_pair.o2.pqp[model_id],
					curr_pair.delta);
      //num_tri_tests += mpkCollDistAlgo::dist_res.num_tri_tests;
      //num_bv_tests  += mpkCollDistAlgo::dist_res.num_bv_tests;
 
    
    if ( dist <= dmax )
	{kk++;}
  }  
	// fin du if


  }  // fin du for (i....
	
    membership.push_back(((double)kk)/((double)robots->num_params()-1));
	dd = dd+ ((membership[k]*val)/(double)(zones->num_zones())); 


 }  // fin du for( k....

return dd;

}