#include "mpkZoneCollection.H"
#include "mpkZone.H"
#include "string.H"

mpkZoneCollection::mpkZoneCollection(SoGroup* scenegraph)
{
  // setup search action for mpkZones
  SoSearchAction searchzone;
  searchzone.setType(mpkZone::getClassTypeId());
  searchzone.setInterest(SoSearchAction::ALL);
  searchzone.setSearchingAll(TRUE);
  searchzone.apply(scenegraph);
  SoPathList zonepathlist = searchzone.getPaths();

  // process mpkZones
  for ( int k=0; k < zonepathlist.getLength(); k++ ) {

    SoNode *zonenode = zonepathlist[k]->getNodeFromTail(0);

    SbVec3f translation, zoneScale, collScale;
    SbRotation rotation, scaleOrientation;

    // get transform of zones
    SbViewportRegion dummy;
    SoGetMatrixAction get_matrix(dummy);
    get_matrix.apply(zonepathlist[k]);
    SbMatrix matrix = get_matrix.getMatrix();
    matrix.getTransform(translation, rotation, zoneScale, scaleOrientation);    
    double tx = translation.getValue()[0];
    double ty = translation.getValue()[1];
    double tz = translation.getValue()[2];

    mpkTransform zone_transf(rotation, SbVec3f(tx,ty,tz));
    Tr.push_back(zone_transf);

    // store name of Zone
    //char* s2;
	//const char* l;
	//strcpy (s2,zonenode->getName().getString());
	
	//cout << s2<<endl;//l = strtok(s,"_");
    names.push_back(zonenode->getName().getString());

    // find nodes named "__triangulate__" in mpkZone (zonenode) and triangulate them
    SoSearchAction searchcoll;
    searchcoll.setName("__triangulate__");
    searchcoll.setInterest(SoSearchAction::ALL);
    searchcoll.setSearchingAll(TRUE);
    searchcoll.apply(zonenode);
    SoPathList collpathlist = searchcoll.getPaths();

    if ( collpathlist.getLength() == 0 ) {
      cerr << "Error: mpkZone " << zonenode->getName().getString()
	   << " has no nodes declared for triangulation " << endl
	   << "Hint: use \"DEF __triangulate__ <node-type>\"" << endl;
#ifdef WIN32
      getchar();
#endif
      exit(1);
    }
    else {

      PQP_Model* pqp_model = new PQP_Model;
      pqp_model->BeginModel();
      
      int zone_num_tri=0;
      for ( int i=0; i < collpathlist.getLength(); i++ ) {
	
	get_matrix.apply(collpathlist[i]);
	matrix = get_matrix.getMatrix();
	matrix.getTransform(translation, rotation, collScale, scaleOrientation);    
	double tx = translation.getValue()[0]*zoneScale[0];
	double ty = translation.getValue()[1]*zoneScale[1];
	double tz = translation.getValue()[2]*zoneScale[2]; 
	mpkTransform coll_transf(rotation, SbVec3f(tx,ty,tz));
	
	SoCallbackAction triAction;
	
	collScale[0] *= zoneScale[0];
	collScale[1] *= zoneScale[1];
	collScale[2] *= zoneScale[2];
	
	tri_info info(pqp_model,coll_transf,collScale);
	triAction.addTriangleCallback(SoNode::getClassTypeId(),
				      triang_CB, (void*)&info);
	triAction.apply(collpathlist[i]);
	zone_num_tri += info.tri_cnt;
      }
      pqp_model->EndModel();
      
      pqp.push_back(pqp_model);
      cerr << "  mpkZone with " << zone_num_tri << " triangles." << endl;
    }
  }
}


mpkZoneCollection::~mpkZoneCollection()
{
  for ( int i=0; i<pqp.size(); i++ )
    delete pqp[i];
}

void mpkZoneCollection::triang_CB(void *data, SoCallbackAction *,const SoPrimitiveVertex *vertex1,const SoPrimitiveVertex *vertex2,const SoPrimitiveVertex *vertex3)
{
  tri_info* info = (tri_info*)data;
  if ( !info->pqp_model ) return;

  SbVec3f points[] =
    { vertex1->getPoint(), vertex2->getPoint(), vertex3->getPoint() };
  double verts[3][3];

  for (int i = 0; i < 3; i++) {
    double vert[3];
    for (int j = 0; j < 3; j++) {
      vert[j] = points[i][j] * info->scalef[j];
      //cout << points[i][j] << " ";
    }
    info->Tr.apply(verts[i], vert);
    //cout << " * ";
  }
  info->pqp_model->AddTri(verts[0], verts[1], verts[2], info->tri_cnt++);

  //cout << endl;
}