#ifndef MPK_ZONE_COLLECTION_H
#define MPK_ZONE_COLLECTION_H

#include <vector>
#include "PQP.h"
#include "mpkTransform.H"

using namespace std;

/**@memo Extracts all zones (nodes of type {@link mpkZone mpkZone})
   from an Inventor scene graph and builds PQP collision models for
   them.
   
   @see mpkZone
 */
class mpkZoneCollection {

public:

  /**@memo {\bf scenegraph} is the inventor scene graph which will be
     parsed for zone nodes.
  */
  mpkZoneCollection(SoGroup* scenegraph);

  ~mpkZoneCollection();

  ///@memo Returns the number of zones
  int num_zones() {return pqp.size();};

  ///@memo PQP models of all zones.
  vector<PQP_Model*> pqp;

  ///@memo Corresponding transforms of all zones.
  vector<mpkTransform> Tr;

  /**@memo Corresponding names of all zones (may be prepended by a
     file name if the zone was read from an included file).
  */
  vector<const char*> names;


private:

//   information for triangle callback
  struct tri_info {
    tri_info(PQP_Model* model, const mpkTransform& transf,
 	     const SbVec3f& scaleFactor, int cnt=0) {
      pqp_model = model; Tr = transf; scalef = scaleFactor; tri_cnt=cnt;
    };
    PQP_Model* pqp_model;
    mpkTransform Tr;
    SbVec3f scalef;
    int tri_cnt;
  };

//   the triangulation callback (to get the triangles from the
 //  inventor nodes into the PQP models).  data will be passed as a
 //  pointer to struct tri_info (see above).
 
  static void triang_CB(void *data, SoCallbackAction *,
			const SoPrimitiveVertex *vertex1,
			const SoPrimitiveVertex *vertex2,
			const SoPrimitiveVertex *vertex3);
};

#endif


